package chessTrainingWheels;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Image;
import java.util.Stack;

public class Board extends Sprite {

	//==========================================================================
	//===   DESIGN IDEAS =======================================================
	
	// MUST DO
	// TODO King move calculations - show in check
	// TEST Switch between CPU and human control
	// TODO Pawn promotion
	
	// BUG
	// FIXME When taking human control of Black, if the black move is sent before the CPU can send its move and be told to undo, it complains about being sent a move
	
	// SHOULD DO
	// TODO Human vs human option
	// TEST Performance - make sure all calls to recalculate happen on mouse movement.

	
	// IDEAS
	// TODO Undo during CPU turn (race conditions...)
	// TODO Display attacks horizontally
	// TODO Highlight king when in check
	// TODO Add menu with view option toggles
	// TODO Add menu with new game options
	// TODO Add menu with undo
	// TODO Add menu item with CPU difficulty
	// TODO Animation of CPU piece moving, with line drawn on path
	// TODO Click on square to show attacks
	// TODO Show new attacks when dragging
	// TODO Show change in attacks (opening, closing) when moving a piece
	// TODO Calculate squares in check for king
	// TODO Hints (from engine)
	// TODO Show which side's turn it is
	
	
	// CHESS RULES TO IMPLEMENT
	// Rules: 	http://www.chessvariants.org/d.chess/chess.html
	// xboard: 	http://www.tim-mann.org/xboard/engine-intf.html
	// TODO Enpassent
	// TODO Promotion
	// TODO Draw
	// TODO Don't allow moves that put you in check
	// TODO Don't allow moves that take you out of check
	// TODO Determine checkmate (can engine do this??)
	// TODO Undo on illegal move reported by engine

	//===   DESIGN IDEAS =======================================================
	//==========================================================================
	
	
	
	
	
	//==========================================================================
	//===   CONSTANTS   ========================================================

	/**
	 * Board border color.  Set to be always black.
	 */
	final static Color BORDER_COLOR = Color.black;

	/**
	 * Board border width.
	 */
	final static Integer BORDER_WIDTH = 2;

	/**
	 * Board border stroke.  Set to be a simple 1 pixel basic stroke.
	 */
	final static BasicStroke BORDER_STROKE = new BasicStroke(BORDER_WIDTH);

	/**
	 * Stroke for drawing the new attack lines
	 */
	final static BasicStroke NEW_ATTACK_STROKE = new BasicStroke(1);

	/**
	 * Color for drawing the last move effect
	 */
	final static Color LAST_MOVE_COLOR = new Color(33, 33, 33);
	
	/**
	 * Stroke for drawing the last move effect
	 */
	final static BasicStroke LAST_MOVE_STROKE = new BasicStroke(1.5f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f, new float[] {12f,12f}, 0);
	
	/**
	 * Size of the dot indication the source of the last move.
	 */
	final static int LAST_MOVE_DOT_SIZE = 5;

	/**
	 * Board contents horizontal offset.
	 */
	final static Integer contentHOffset = BORDER_WIDTH;

	/**
	 * Board contents vertical offset.
	 */
	final static Integer contentVOffset = BORDER_WIDTH;

	/**
	 * The number of columns on the board
	 */
	public static int MAX_COLUMNS = 8;

	/**
	 * The number of rows on the board
	 */
	public static int MAX_ROWS = 8;

	/**
	 * The number of pieces on each side
	 */
	public static int PIECES_PER_SIDE = 16;

	/**
	 * Used for calculating moves.  
	 * Indicates that current attacks are being calculated.
	 */
	public final static byte ATTACK = 1;

	/**
	 * Used for calculating moves.  
	 * Indicates that attacks based on the proposed move are being calculated.
	 */
	public final static byte NEW_ATTACK = 2;
	
	/**
	 * Used for calculating moves.  
	 * Indicates that possible moves for the selected piece are being calculated.
	 */
	public final static byte MOVE_POSSIBLE = 3;
	
	/**
	 * Used for calculating moves.  
	 * Indicates that blocked moves for the selected piece are being calculated.
	 */
	public final static byte MOVE_BLOCKED= 4;
	
	/**
	 * Used for calculating moves.  
	 * Indicates that possible attacks for the selected piece are being calculated.
	 */
	public final static byte ATTACK_POSSIBLE = 5;
	
	/**
	 * Used for calculating moves.  
	 * Indicates that attacks based on the proposed move, not including the currently selected piece, are being calculated.
	 */
	public final static byte NEW_ATTACK_NOT_SELECTED = 6;
	
	/**
	 * Used for calculating moves.  
	 * Indicates that attacks based on the proposed move, ONLY including the currently selected piece, are being calculated.
	 */
	public final static byte NEW_ATTACK_SELECTED = 7;
	
	
	/*
	 * 
	 * Attack Lines
	 * 
	 */
	
	/**
	 * Stroke for drawing the attack lines
	 */
	final static BasicStroke ATTACK_LINE_STROKE = new BasicStroke(1.5f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f, new float[] {8f,16f}, 0);

	/**
	 * Color for drawing the black attack lines
	 */
	final static Color BLACK_ATTACK_LINE_COLOR = Square.SQUARE_THREATENED_COLOR;

	/**
	 * Color for drawing the black attack lines
	 */
	final static Color WHITE_ATTACK_LINE_COLOR = Square.SQUARE_DEFENDED_COLOR;
	
	//===   CONSTANTS   ========================================================
	//==========================================================================





	//==========================================================================
	//===   INSTANCE VARIABLES   ===============================================

	/**
	 * The squares on the board
	 */
	private Square[][] squares;

	/**
	 * The chess pieces in the game.
	 */
	protected ChessPiece[][] pieces;

	/**
	 * The images used to display the chess pieces.
	 */
	protected Image[][] chessIcons; 

	/**
	 * The currently selected square
	 */
	protected Square selectedSquare;

	/**
	 * The square currently being hovered over
	 */
	protected Square hoverSquare;
	
	/**
	 * Is a piece being dragged on the board?
	 */
	protected boolean dragging;
	
	/**
	 * Whose turn is it?  Black side or white side.
	 */
	protected boolean isBlackTurn;
	
	/**
	 * Is the white side controlled by the CPU?
	 */
	protected boolean isWhiteCpu;
	
	/**
	 * Is the black side controlled by the CPU?
	 */
	protected boolean isBlackCpu;
	
	/**
	 * The computerized chess opponent.
	 */
	private EngineInterface chessEngine;
	
	/**
	 * Stack of moves to undo
	 */
	private Stack<Move> undoStack;
	
	/**
	 * Stack of moves to redo
	 */
	private Stack<Move> redoStack;
	
	/**
	 * Display Attack lines?
	 */
	private Boolean displayAttackLines;
	
	/**
	 * Display Attack statistics?
	 */
	private Boolean displayAttack;

	/**
	 * Display possible moves?
	 */
	private Boolean displayPossibleMoves;
	
	/**
	 * Display relative threats for each square?
	 */
	private Boolean displayRelativeThreats;
	
	/**
	 * Display the last move?
	 */
	private Boolean displayLastMove;
	
	/**
	 * The source of the last move.
	 */
	private Square lastMoveSource;
	
	/**
	 * The destination of the last move.
	 */
	private Square lastMoveDest;
	
	/**
	 * Is the board currently in redo move?
	 * Do to the way the xboard protocol is designed, after a redo a command must
	 * be sent to the chess engine AFTER the user's next move.  Therefore, the 
	 * board must keep track of the "redo" status.
	 */
	private Boolean CPUWaitingForControl;
	
	/**
	 * The parent application of the board
	 * Needed for popping up alerts, etc.
	 */
	protected ChessTrainingWheels chessUI;
	
	//===   INSTANCE VARIABLES   ===============================================
	//==========================================================================





	//==========================================================================
	//===   CONSTRUCTOR    =====================================================

	/**
	 * Create a new Board
	 * @param HOffset horizontal offset of the board
	 * @param VOffset vertical offset of the board
	 * @param chessIcons an array of images for the chess pieces
	 */
	public Board(int HOffset, int VOffset, Image[][] chessIcons, ChessTrainingWheels chessUI) {
		// Call the super constructor to do basic layout setup for the Artist
		super(HOffset, VOffset, new Dimension(Square.SIZE * MAX_COLUMNS + (2 * BORDER_WIDTH), Square.SIZE * MAX_ROWS + (2 * BORDER_WIDTH)));

		this.chessUI = chessUI;
		
		// White player goes first
		this.isBlackTurn = false;
		
		// CPU is black, human is white
		this.isBlackCpu = true;
		this.isWhiteCpu = false;
		
		// Setup Undo
		this.undoStack = new Stack<Move>();
		this.redoStack = new Stack<Move>();
		this.CPUWaitingForControl = false;
		
		// Display preferences
		this.displayAttack = false;
		this.displayAttackLines = true;
		this.displayPossibleMoves = true;
		this.displayRelativeThreats = true;
		this.displayLastMove = true;
		
		// Size of the content area inside of the board's boarders.
		Dimension contentSize = new Dimension(Square.SIZE * MAX_COLUMNS, Square.SIZE * MAX_ROWS);

		// Do basic layout for the Artist's Content Area
		this.setContentLayout(contentHOffset, contentVOffset);
		this.setContentSize(contentSize);

		// Setup the squares and pieces on the board
		this.setupSquares();
		this.setupPieces(chessIcons);
		
		// Calculate initial attack statistics
		this.updateCalculations(ATTACK);
		this.updateCalculations(NEW_ATTACK);
		
		this.chessEngine = new EngineInterface(this);
		this.chessEngine.start();
	}

	//===   CONSTRUCTORS   =====================================================
	//==========================================================================





	//==========================================================================
	//===   BOARD METHODS     ==================================================

	/**
	 * Create a new game
	 */
	public void newGame() {		
		// White player goes first
		this.isBlackTurn = false;
		
		// CPU is black, human is white
		this.isBlackCpu = true;
		this.isWhiteCpu = false;
		
		// Setup Undo
		this.undoStack = new Stack<Move>();
		this.redoStack = new Stack<Move>();
		this.CPUWaitingForControl = false;
		
		// Setup the squares and pieces on the board
		// this.setupSquares();
		this.resetSquares();
		this.resetPieces();
		
		// Reset the "last move"
		this.setLastMoveEffect(null, null);
	
		
		// Calculate initial attack statistics
		this.updateCalculations(ATTACK);
		this.updateCalculations(NEW_ATTACK);
		
		this.chessEngine.newGame();
	}
	
	/**
	 * Setup the squares on the chess board.
	 */
	private void setupSquares() {
		int i, j;

		this.squares = new Square[MAX_COLUMNS][MAX_ROWS];

		// Create squares
		// Turn off borders on squares
		// TODO Do squares still need to be buttons?
		for (i = 0; i < MAX_COLUMNS; i++) {
			for (j = 0; j < MAX_ROWS; j++) {
				this.squares[i][j] = new Square(i, j, this);
			}
		}	

	}


	/**
	 * Setup the pieces on the chess board.
	 * @param chessIcons A 2D array containing the icons for each piece for each side.
	 */
	private void setupPieces(Image[][] chessIcons) {
		this.chessIcons = chessIcons;
		this.pieces = new ChessPiece[2][PIECES_PER_SIDE];

		// White Pawns
		this.pieces[0][0] = new Pawn	(false, this.squares[0][6], this, this.chessIcons[0][0]);
		this.pieces[0][1] = new Pawn	(false, this.squares[1][6], this, this.chessIcons[0][0]);
		this.pieces[0][2] = new Pawn	(false, this.squares[2][6], this, this.chessIcons[0][0]);
		this.pieces[0][3] = new Pawn	(false, this.squares[3][6], this, this.chessIcons[0][0]);
		this.pieces[0][4] = new Pawn	(false, this.squares[4][6], this, this.chessIcons[0][0]);
		this.pieces[0][5] = new Pawn	(false, this.squares[5][6], this, this.chessIcons[0][0]);
		this.pieces[0][6] = new Pawn	(false, this.squares[6][6], this, this.chessIcons[0][0]);
		this.pieces[0][7] = new Pawn	(false, this.squares[7][6], this, this.chessIcons[0][0]);

		// White non-pawns
		this.pieces[0][8]  = new Rook	(false, this.squares[0][7], this, this.chessIcons[0][3]);
		this.pieces[0][9]  = new Knight	(false, this.squares[1][7], this, this.chessIcons[0][1]);
		this.pieces[0][10] = new Bishop	(false, this.squares[2][7], this, this.chessIcons[0][2]);
		this.pieces[0][11] = new Queen	(false, this.squares[3][7], this, this.chessIcons[0][4]);
		this.pieces[0][12] = new King	(false, this.squares[4][7], this, this.chessIcons[0][5]);
		this.pieces[0][13] = new Bishop	(false, this.squares[5][7], this, this.chessIcons[0][2]);
		this.pieces[0][14] = new Knight	(false, this.squares[6][7], this, this.chessIcons[0][1]);
		this.pieces[0][15] = new Rook	(false, this.squares[7][7], this, this.chessIcons[0][3]);

		// Black Pawns
		this.pieces[1][0] = new Pawn	(true, this.squares[0][1], this, this.chessIcons[1][0]);
		this.pieces[1][1] = new Pawn	(true, this.squares[1][1], this, this.chessIcons[1][0]);
		this.pieces[1][2] = new Pawn	(true, this.squares[2][1], this, this.chessIcons[1][0]);
		this.pieces[1][3] = new Pawn	(true, this.squares[3][1], this, this.chessIcons[1][0]);
		this.pieces[1][4] = new Pawn	(true, this.squares[4][1], this, this.chessIcons[1][0]);
		this.pieces[1][5] = new Pawn	(true, this.squares[5][1], this, this.chessIcons[1][0]);
		this.pieces[1][6] = new Pawn	(true, this.squares[6][1], this, this.chessIcons[1][0]);
		this.pieces[1][7] = new Pawn	(true, this.squares[7][1], this, this.chessIcons[1][0]);

		// Black non-pawns
		this.pieces[1][8]  = new Rook	(true, this.squares[0][0], this, this.chessIcons[1][3]);
		this.pieces[1][9]  = new Knight	(true, this.squares[1][0], this, this.chessIcons[1][1]);
		this.pieces[1][10] = new Bishop	(true, this.squares[2][0], this, this.chessIcons[1][2]);
		this.pieces[1][11] = new Queen	(true, this.squares[3][0], this, this.chessIcons[1][4]);
		this.pieces[1][12] = new King	(true, this.squares[4][0], this, this.chessIcons[1][5]);
		this.pieces[1][13] = new Bishop	(true, this.squares[5][0], this, this.chessIcons[1][2]);
		this.pieces[1][14] = new Knight	(true, this.squares[6][0], this, this.chessIcons[1][1]);
		this.pieces[1][15] = new Rook	(true, this.squares[7][0], this, this.chessIcons[1][3]);
	}
	
	/**
	 * Reset the pieces for a new game
	 */
	private void resetPieces() {

		// White Pawns
		this.pieces[0][0].resetPiece(this.squares[0][6]);
		this.pieces[0][1].resetPiece(this.squares[1][6]);
		this.pieces[0][2].resetPiece(this.squares[2][6]);
		this.pieces[0][3].resetPiece(this.squares[3][6]);
		this.pieces[0][4].resetPiece(this.squares[4][6]);
		this.pieces[0][5].resetPiece(this.squares[5][6]);
		this.pieces[0][6].resetPiece(this.squares[6][6]);
		this.pieces[0][7].resetPiece(this.squares[7][6]);

		// White non-pawns
		this.pieces[0][8].resetPiece(this.squares[0][7]);
		this.pieces[0][9].resetPiece(this.squares[1][7]);
		this.pieces[0][10].resetPiece(this.squares[2][7]);
		this.pieces[0][11].resetPiece(this.squares[3][7]);
		this.pieces[0][12].resetPiece(this.squares[4][7]);
		this.pieces[0][13].resetPiece(this.squares[5][7]);
		this.pieces[0][14].resetPiece(this.squares[6][7]);
		this.pieces[0][15].resetPiece(this.squares[7][7]);

		// Black Pawns
		this.pieces[1][0].resetPiece(this.squares[0][1]);
		this.pieces[1][1].resetPiece(this.squares[1][1]);
		this.pieces[1][2].resetPiece(this.squares[2][1]);
		this.pieces[1][3].resetPiece(this.squares[3][1]);
		this.pieces[1][4].resetPiece(this.squares[4][1]);
		this.pieces[1][5].resetPiece(this.squares[5][1]);
		this.pieces[1][7].resetPiece(this.squares[7][1]);
		this.pieces[1][6].resetPiece(this.squares[6][1]);

		// Black non-pawns
		this.pieces[1][8].resetPiece(this.squares[0][0]);
		this.pieces[1][9].resetPiece(this.squares[1][0]);
		this.pieces[1][10].resetPiece(this.squares[2][0]);
		this.pieces[1][11].resetPiece(this.squares[3][0]);
		this.pieces[1][12].resetPiece(this.squares[4][0]);
		this.pieces[1][13].resetPiece(this.squares[5][0]);
		this.pieces[1][14].resetPiece(this.squares[6][0]);
		this.pieces[1][15].resetPiece(this.squares[7][0]);
	}
	
	/**
	 * Reset the squares for a new game
	 */
	private void resetSquares() {
		int i, j;
		for (i = 0; i < MAX_COLUMNS; i++) {
			for (j = 0; j < MAX_ROWS; j++) {
				this.squares[i][j].setPiece(null);
			}
		}	
	}
	
	
	/**
	 * Get the currently selected square.
	 * Return null if no square is selected.
	 * @return the currently selected square (or null if none is selected)
	 */
	public Square getSelectedSquare() {
		return this.selectedSquare;
	}

	/**
	 * Get the square that the mouse is currently hovering over (if any)
	 */
	public Square getHoverSquare() {
		return this.hoverSquare;
	}
	
	
	/**
	 * Get a square at the given row and column
	 * @param column the column of the desired square
	 * @param row the row of the desired square
	 * @return the square at the specified column and row, or null if the column or row are invalid
	 */
	public Square getSquare(int column, int row) {
		// TODO Throw an invalid square exception???
		
		// Sanity check that the column and row is valid
		if (column >= MAX_COLUMNS || row >= MAX_ROWS || column < 0 || row < 0) {
			return null;
		} else {
			return this.squares[column][row];
		}
	}
	
	
	/**
	 * Set the selected square
	 * @param newSelection
	 */
	public void setSelectedSquare (Square newSelection) {
		//  Unhighlight the old selected square
		if (this.selectedSquare != null) {
			this.selectedSquare.unsetSelected();
		}

		// Set the new selected square
		this.selectedSquare = newSelection;

		// If the selected square is not null, highlight it
		if (this.selectedSquare != null) {
			this.selectedSquare.setSelected();
		}
	}

	
	/**
	 * Unset the selected square.  ie set no square to be selected
	 */
	public void unsetSelectedSquare () {
		this.setSelectedSquare(null);
	}

	
	/**
	 * Is a piece on the board currently being dragged?
	 * @return
	 */
	public boolean isDragging() {
		return this.dragging;
	}

	
	/**
	 * Set whether a piece on the board is being dragged or not
	 * @param dragging
	 */
	public void setDragging(boolean dragging) {
		this.dragging = dragging;
	}

	
	/**
	 * Update the calculations of the number of pieces that can perform an action on each square.
	 * @param calculationType the type of calculation to perform
	 */
	protected void updateCalculations(byte calculationType) {
		int i;
		
		// Reset calculations first
		resetCalculations(calculationType);
		
		// Calculate White Piece Attacks
		for (i = 0; i < PIECES_PER_SIDE; i++) {
			this.pieces[0][i].calculateMoves(calculationType);
		}

		// Calculate Black Piece Attacks
		for (i = 0; i < PIECES_PER_SIDE; i++) {
			this.pieces[1][i].calculateMoves(calculationType);
		}
	}
	
	/**
	 * Reset the calculations for the designated calculation type
	 * @param calculationType the type of calculation to reset
	 */
	protected void resetCalculations(byte calculationType) {
		int i, j;
		
		for (i = 0; i < MAX_COLUMNS; i++) {
			for (j = 0; j < MAX_ROWS; j++) {
				this.squares[i][j].resetCalculations(calculationType);
			}
		}
	}
	
	/**
	 * End the current turn and give control over to the other player.
	 */
	public void endTurn() {
		// Switch control to the other side
		this.isBlackTurn = !(this.isBlackTurn);

		// Disable the toggle to switch black from computer to human when the CPU is planning a move
		if (this.isBlackTurn && this.isBlackCpu) {
			chessUI.disableComputerToggle();
		} else {
			chessUI.enableComputerToggle();
		}
		
		
		// If the board/engine is waiting for control 
		// (ie in redo mode, switching from human to CPU for black, etc)
		// then hand back control to the CPU
		if(this.isCPUWaitingForControl()) {
			this.returnCPUControl();
		}
	}
	
	/**
	 * Whose turn is it?  The black side or the white side?
	 * @return true for black's turn and false for white's turn
	 */
	public boolean getIsBlackTurn() {
		return this.isBlackTurn;
	}
	
	/**
	 * Manage illegal moves reported by the chess engine
	 * Undo the move and pop up an alert for the user
	 */
	public void handleIllegalMove() {
		Move oldMove;
		
		// Undo the move
		oldMove = this.undoStack.pop();
		oldMove.undoMove();
		
		// Reset the "Last Move" effect
		if (this.undoStack.size() > 0) {
			this.undoStack.peek().setLastMove();
		} else {
			this.setLastMoveEffect(null, null);	
		}
		
		// Update attack statistics
		this.updateCalculations(ATTACK);
		this.updateCalculations(NEW_ATTACK);
		
		// End the turn (ie set it back to the player's move)
		this.endTurn();
		
		// Put up an alert
		new IllegalMoveDialog(this.chessUI.frame).setVisible(true);	
	}
	
	
	/**
	 * Handle moves
	 * @param sourceColumn
	 * @param sourceRow
	 * @param destColumn
	 * @param destRow
	 */
	public void movePiece(int sourceColumn, int sourceRow, int destColumn, int destRow) {

		// Sanity check that the source and destination squares are valid
		if (sourceColumn >= 0 && sourceColumn < MAX_COLUMNS && sourceRow >= 0 && sourceRow < MAX_ROWS) {
			
			// Set source and destination squares from their rows and columns
			Square destSquare = this.squares[destColumn][destRow];
			this.setSelectedSquare(this.squares[sourceColumn][sourceRow]);

			// Sanity check that the source square has a piece and it belongs to the side whose turn it is now
			if (this.getSelectedSquare().getPiece() != null && this.selectedSquare.getPiece().isBlack() == this.getIsBlackTurn()) {

				// Store a reference to the piece being dragged for convenience
				ChessPiece currentPiece = this.getSelectedSquare().getPiece();

				// Check if dragging is occurring and do a sanity check that the current piece is not null
				if (currentPiece != null) {

					// Check that CPU is actually in control
					if ((! this.isBlackCpu && this.isBlackTurn) || (! this.isWhiteCpu && ! this.isBlackTurn)) {
						// tell the CPU to undo the move if its going out of turn
						System.out.println("-- Board sending urgent undo --");
						this.chessEngine.writeUrgentLine("undo");
				
					// Check that this move is valid
					} else if (currentPiece.isValidMove(destSquare)) {
						Move newMove = new Move(this.getSelectedSquare(), destSquare, currentPiece);
						newMove.execute();
						undoStack.push(newMove);
						redoStack.clear();

						// End the turn and pass control to the other player
						this.endTurn();

						// Update attack statistics
						this.updateCalculations(ATTACK);
						this.updateCalculations(NEW_ATTACK);

					} else {
						System.out.println("Board says: Invalid move");
					} // Check for valid move

				} else {
					System.out.println("Board says: null piece");
				} // Check for valid piece

				// Unhighlight the selected square
				this.unsetSelectedSquare();

				// Reset possible and blocked move effect
				this.resetCalculations(MOVE_POSSIBLE);
				this.resetCalculations(MOVE_BLOCKED);
				this.resetCalculations(ATTACK_POSSIBLE);
			} else {
				// Print out an error message to the console
				if (this.getSelectedSquare().getPiece() == null) {
					System.out.println("Board says: Invalid.  There is no piece on the source square");
				} else if (this.selectedSquare.getPiece().isBlack() == this.getIsBlackTurn()) {
					System.out.println("Board says: Invalid.  An attempt was made to move a piece from the wrong side.");
				}
				
				
			} // Check for valid square
		} else {
			System.out.println("Board says: Invalid square");
		} // Check for valid coordinates
	}
	
	
	/**
	 * Should possible moves be displayed?
	 * @return Should possible moves be displayed?
	 */
	public Boolean displayPossibleMoves() {
		return this.displayPossibleMoves;
	}
	
	/**
	 * Should relative threats for each square be displayed?
	 * @return Should relative threats for each square be displayed?
	 */
	public Boolean displayRelativeThreats() {
		return this.displayRelativeThreats;
	}
	
	/**
	 * Toggle the display of possible moves
	 */
	public void toggleDisplayPossibleMoves() {
		this.displayPossibleMoves = ! this.displayPossibleMoves;
	}
	
	/**
	 * Toggle the display of relative threats for each square
	 */
	public void toggleDisplayRelativeThreats() {
		this.displayRelativeThreats = ! this.displayRelativeThreats;
	}
	
	/**
	 * Toggle whether CPU plays the black side.
	 */
	public void toggleComputerPlaysBlack() {
		// Toggle whether CPU plays black
		this.isBlackCpu = ! this.isBlackCpu;
		
		// Update chess engine on the switch in players
		if (! this.isBlackCpu) {
			// If human is playing black now, tell the CPU to just listen for moves.
			this.chessEngine.sendMove("force");
			
		} else if (this.isBlackTurn) {
			// If CPU is playing black now AND its black's turn, tell the CPU to go.
			this.chessEngine.sendMove("go");
			
		} else {
			// If CPU is playing black now BUT its white's turn, remember to tell the CPU to go after the next move.
			this.CPUWaitingForControl = true;
		}
	}
	
	/**
	 * Set whether possible moves be displayed.
	 * @param display true is possible moves should be displayed, false otherwise.
	 */
	public void setDisplayPossibleMoves(Boolean display) {
		this.displayPossibleMoves = display;
	}
	
	/**
	 * Set whether relative threats for each square should be displayed.
	 * @param display true is relative threats for each square should be displayed, false otherwise.
	 */
	public void setDisplayRelativeThreats(Boolean display) {
		this.displayRelativeThreats = display;
	}

	/**
	 * Should attack lines be displayed?
	 * @return Should attack lines be displayed?
	 */
	public Boolean displayAttackLines() {
		return this.displayAttackLines;
	}
	
	/**
	 * Set whether attack lines should be displayed?
	 * @param display true is attack lines should be displayed, false otherwise.
	 */
	public void setDisplayAttackLines(Boolean display) {
		this.displayAttackLines = display;
	}
	
	/**
	 * Toggle the display of attack lines
	 */
	public void toggleDisplayAttackLines() {
		this.displayAttackLines = !this.displayAttackLines;
	}
	
	/**
	 * Should attack statistics be displayed?
	 * @return Should attack statistics be displayed?
	 */
	public Boolean displayAttack() {
		return this.displayAttack;
	}
	
	/**
	 * Set whether attack statistics should be displayed?
	 * @param display true is attack statistics should be displayed, false otherwise.
	 */
	public void setDisplayAttack(Boolean display) {
		this.displayAttack = display;
	}
	
	/**
	 * Toggle the display of attack statistics
	 */
	public void toggleDisplayAttacks() {
		this.displayAttack = !this.displayAttack;
	}
	
	/**
	 * Should the last move be displayed?
	 * @return Should the last move be displayed?
	 */
	public Boolean displayLastMove() {
		return this.displayLastMove;
	}
	
	/**
	 * Set whether the last move should be displayed?
	 * @param display true if the last move should be displayed, false otherwise.
	 */
	public void setDisplayLastMove(Boolean display) {
		this.displayLastMove = display;
	}
	
	/**
	 * Toggle the display of the last move 
	 */
	public void toggleLastMove() {
		this.displayLastMove = !this.displayLastMove;
	}

	
	/**
	 * Is the piece on the currently selected square movable by the player right now.
	 * Does not take into account whether there are any valid moves or not, just whether
	 * the color of the piece and the current turn make this a valid selection.
	 * Return false if the piece does not belongs to the side whose turn it is.
	 * Return false if it is the CPU's turn.
	 * Return true otherwise.
	 * @return if the selected piece can be moved.
	 */
	public Boolean isPieceSelectable() {
		if (this.isBlackCpu && this.getIsBlackTurn()) {
			return false;
		} else if (this.isWhiteCpu && !this.getIsBlackTurn()) {
			return false;
		} else if (this.selectedSquare.getPiece().isBlack() == this.getIsBlackTurn()) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Set the source and destination of the last move.
	 * @param lastMoveSource
	 * @param lastMoveDest
	 */
	public void setLastMoveEffect(Square lastMoveSource, Square lastMoveDest) {
		this.lastMoveSource = lastMoveSource;
		this.lastMoveDest = lastMoveDest;	
	}
	//===   BOARD METHODS     ==================================================
	//==========================================================================

	
	
	

	//==========================================================================
	//===   UNDO /REDO        ==================================================
	
	/**
	 * Undo the last move.
	 * If playing against the computer, undo both your move and the computer's move.
	 */
	public void undoMove() {

		// Make sure that there is at least one move to undo
		if (this.undoStack.size() > 0) {
			// Player vs Player
			if (!this.isBlackCpu && !this.isWhiteCpu) {
				Move oldMove;
				
				// Undo the move
				oldMove = this.undoStack.pop();
				oldMove.undoMove();
				this.redoStack.push(oldMove);
	
				// Update attack statistics
				this.updateCalculations(ATTACK);
				this.updateCalculations(NEW_ATTACK);
				
				
				this.chessEngine.writeLine("undo");

				// End the turn
				this.endTurn();
				
			// Player's turn playing against the CPU
			} else if ((this.isBlackTurn && !this.isBlackCpu) || (!this.isBlackTurn && !this.isWhiteCpu)) {
			
				if (this.undoStack.size() >= 2) {
					Move oldMove;
					
					// Undo the CPU move
					oldMove = this.undoStack.pop();
					oldMove.undoMove();
					this.redoStack.push(oldMove);
					
					// Undo the player move
					oldMove = this.undoStack.pop();
					oldMove.undoMove();
					this.redoStack.push(oldMove);
					
					// Update attack statistics
					this.updateCalculations(ATTACK);
					this.updateCalculations(NEW_ATTACK);
					
					this.chessEngine.writeLine("remove");
				}
				
			// CPU's turn
			} else {
				// Don't do anything right now, avoid race conditions.
			}
			
			// "Last Move" effect update based on the move on top of the undo stack
			if (this.undoStack.size() > 0) {
				this.undoStack.peek().setLastMove();
			} else {
				this.setLastMoveEffect(null, null);
			}
			
		}
	}
		
	/**
	 * Redo the last move.
	 * If playing against the computer, redo both your move and the computer's move.
	 */
	public void redoMove() {
		
		// Make sure there are moves to redo
		if (this.redoStack.size() > 0) {
			// Player vs Player
			if (!this.isBlackCpu && !this.isWhiteCpu) {
				Move newMove;
				
				// Redo the move
				newMove = this.redoStack.pop();
				newMove.execute();
				this.chessEngine.sendMove(newMove.getCommand());
				this.undoStack.push(newMove);
				
				// End the turn
				this.endTurn();
				
			// Player's turn playing against the CPU
			} else if ((this.isBlackTurn && !this.isBlackCpu) || (!this.isBlackTurn && !this.isWhiteCpu)) {
			
				if (this.redoStack.size() >= 2) {
					Move newMove;
					
					// Put the board (and engine) in redo mode
					this.startRedo();
					
					// Redo the player move and send it to the chess enginge
					newMove = this.redoStack.pop();
					newMove.execute();
					this.chessEngine.sendMove(newMove.getCommand());
					this.undoStack.push(newMove);
					
					// Redo the CPU move and send it to the chess enginge
					newMove = this.redoStack.pop();
					newMove.execute();
					this.chessEngine.sendMove(newMove.getCommand());
					this.undoStack.push(newMove);
					
					// Update attack statistics
					this.updateCalculations(ATTACK);
					this.updateCalculations(NEW_ATTACK);
				}
				
			// CPU's turn
			} else {
				// Don't do anything right now, avoid race conditions.
			}
		}
	}
	
	/**
	 * Is the board currently in "redo" mode?
	 * @return Is the board currently in 
	 */
	public Boolean isCPUWaitingForControl () {
		// Leave redo mode
		return this.CPUWaitingForControl;
	}
	
	/**
	 * Set the board to "redo" mode.
	 */
	public void startRedo() {
		// Put the CPU in "force mode" so that moves can be redone and 
		this.chessEngine.writeLine("force");
		
		// Set the board to be in redo mode
		this.CPUWaitingForControl = true;
	}
	
	/**
	 * Take the board out of "redo" mode.
	 */
	public void returnCPUControl() {
		// Take the CPU out of "force mode" so that the game can continue
		this.chessEngine.writeLine("go");
		
		// Set the board to NOT be in redo mode
		this.CPUWaitingForControl = false;
	}
	
	//===   UNDO /REDO        ==================================================
	//==========================================================================


	
	

	//==========================================================================
	//===   MOUSE METHODS   ====================================================

	/**
	 * Determine the column that the current mouse action is over.
	 * @param x the position of the mouse press, translated into the object's coordinates
	 */
	protected int getMouseColumn(int x) {
		return (int)Math.min(MAX_COLUMNS - 1, Math.max(0, Math.floor(x / Square.SIZE)));
	}

	/**
	 * Determine the row that the current mouse action is over.
	 * @param y the position of the mouse press, translated into the object's coordinates
	 */
	protected int getMouseRow(int y) {
		return (int)Math.min(MAX_ROWS - 1, Math.max(0, Math.floor(y / Square.SIZE)));
	}

	/**
	 * Determine the if the current mouse action is this object.
	 * @param x the position of the mouse press, translated into the object's coordinates
	 * @param y the position of the mouse press, translated into the object's coordinates
	 */
	protected boolean testAction(int x, int y) {
		// System.out.println("Board = " + this.getContentX() + "/" + (this.getContentWidth() + this.getContentX()) + ", " + ", " + this.getContentY() + "/" + (this.getContentHeight() + this.getContentY()) + " -- " + x + ", " + y);
		
		// Test is the mouse press is inside the objects bounds.
		// Note that since the x and y are translated into the object's coordinates, negative values are possible
		if (x >= 0 && x< this.getContentWidth() && y >= 0 && y < this.getContentHeight()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Handle mouse presses
	 * @param x The x coordinate of the mouse action in the parent's coordinate space
	 * @param y The y coordinate of the mouse action in the parent's coordinate space
	 * @return was the action handled by the board?
	 */
	public boolean handlePress(int x, int y) {
		int localX, localY, column, row;

		// Translate the x and y coordinates into the objects coordinates
		localX = x - this.getHOffset() - BORDER_WIDTH;
		localY = y - this.getVOffset() - BORDER_WIDTH;

		// Test to see if the mouse action was inside the current object
		// Perform the appropriate action if it is
		if (this.testAction(localX, localY)) {
			column = this.getMouseColumn(localX);
			row = this.getMouseRow(localY);	
		 
			// If this is a newly selected square, update everything
			if (this.getSelectedSquare() != this.squares[column][row]) {
				this.setSelectedSquare(this.squares[column][row]);

				// System.out.println("Board press: (" + localX + ", " + localY + "). " + this.getSelectedSquare().getPiece());

				if (this.getSelectedSquare().getPiece() != null && this.isPieceSelectable()) {
					this.setDragging(true);
					this.getSelectedSquare().getPiece().startDrag(localX, localY);

					// Calculate possible moves for this piece
					this.getSelectedSquare().getPiece().calculateMoves(MOVE_POSSIBLE);

					// Update hover effects
					if (this.getHoverSquare() != null) {
						this.hoverSquare.unsetHover();
					}
					this.hoverSquare = this.squares[column][row];
					this.hoverSquare.setHover();
				}
			}

//			System.out.println("Clicked on col " + column + ", row " + row);

			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Handle mouse releases.
	 * @param x x coordinate of the mouse event
	 * @param y y coordinate of the mouse event
	 * @return whether the event was handled
	 */
	public boolean handleRelease(int x, int y) {
		// TODO Deal with missed releases
		
		int localX, localY, column, row;
		Square destSquare;

		// Translate the x and y coordinates into the objects coordinates
		localX = x - this.getHOffset() - BORDER_WIDTH;
		localY = y - this.getVOffset() - BORDER_WIDTH;
	
		// Store a reference to the piece being dragged for convenience
		ChessPiece currentPiece = null;
		if (this.getSelectedSquare() != null) {
			currentPiece = this.getSelectedSquare().getPiece();
		}
		
		// Check if dragging is occurring and do a sanity check that the current piece is not null
		if (this.isDragging() && currentPiece != null) {
			
			// Check that the release was over a valid coordinate on the board
			if (this.testAction(localX, localY)) {
				// Get the column and row of the mouse release
				column = this.getMouseColumn(localX);
				row = this.getMouseRow(localY);
				destSquare = this.squares[column][row];
				
				// Check that this move is valid
				if (currentPiece.isValidMove(destSquare)) {
					
					Move newMove = new Move(this.getSelectedSquare(), destSquare, currentPiece);
					newMove.execute();
					undoStack.push(newMove);
					redoStack.clear();

					// Inform the CPU about the move
					if ((this.isBlackTurn && !this.isBlackCpu) || (!this.isBlackTurn && !this.isWhiteCpu)) {
						this.chessEngine.sendMove(newMove.getCommand());
					}
					
					// End the turn and pass control to the other player
					this.endTurn();
					
					// Update attack statistics
					// this.updateCalculations(ATTACK);
					// this.updateCalculations(NEW_ATTACK);
					
				} // Check for valid move
			} // Check for valid coordinate
			
			// Stop dragging the piece
			currentPiece.endDrag();
		}
		
		// Turn off hover effects
		if (this.getHoverSquare() != null) {
			this.hoverSquare.unsetHover();
			this.hoverSquare = null;
		}
		
		// Unhighlight the selected square
		this.unsetSelectedSquare();
		
		// Reset possible and blocked move effect
		this.resetCalculations(MOVE_POSSIBLE);
		this.resetCalculations(MOVE_BLOCKED);
		this.resetCalculations(ATTACK_POSSIBLE);

		// Update attack statistics
		this.updateCalculations(ATTACK);
		this.updateCalculations(NEW_ATTACK);
		
		// Turn off the board's dragging flag
		this.setDragging(false);
		
		return true;
	}

	/**
	 * Handle mouse drags
	 * @param x The x coordinate of the mouse action in the parent's coordinate space
	 * @param y The y coordinate of the mouse action in the parent's coordinate space
	 * @return was the action handled by the board?
	 */
	public boolean handleDrag(int x, int y) {
		int localX, localY, column, row;

		// Translate the x and y coordinates into the objects coordinates
		localX = x - this.getHOffset() - BORDER_WIDTH;
		localY = y - this.getVOffset() - BORDER_WIDTH;

		// System.out.println("Board drag: (" + localX + ", " + localY + "). " + this.isDragging());

		if (this.isDragging() && this.testAction(localX, localY) && this.getSelectedSquare() != null) {
			column = this.getMouseColumn(localX);
			row = this.getMouseRow(localY);	
			
			ChessPiece selectedPiece = this.getSelectedSquare().getPiece();
			
    		if (selectedPiece != null) {
    			selectedPiece.updateDrag(localX, localY);
    			
    			// Update effects if this is a new square
    			Square currentSquare = this.squares[column][row];
    			if (this.getHoverSquare() != currentSquare) {
    				if (this.hoverSquare != null) {
    					this.hoverSquare.unsetHover();
    				}
    				this.hoverSquare = currentSquare;
    				this.hoverSquare.setHover();
    		
    				// If this is a possible move, update the attack stats for what would happen if the move was selected
    				// if (this.hoverSquare.getCalculations(selectedPiece.isBlack(), MOVE_POSSIBLE) > 0) {
    					this.updateCalculations(NEW_ATTACK);
    				// }
    			}
    		}
			
			// System.out.println("Dragging on col " + column + ", row " + row);

			return true;
		} else {
			return false;
		}
	}

	//===   MOUSE METHODS   ====================================================
	//==========================================================================





	//==========================================================================
	//===   PAINT METHOD   =====================================================

	/**
	 * Paints the Frame Artist as a black rectangular border, along with any zoom or rotation 
	 * and the Artist's children, using the provided <code>Graphics2D</code> object at the 
	 * location stored within the object.  
	 *
	 * @param g the graphics object through which to draw this object.
	 */
	public void paintComponent(Graphics2D g) {	
		
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		// Save the old clipping area (ie the parent's clipping area) so that it
		// can be restored after this object and its children have been drawn
		Shape oldClip = g.getClip();

		// Translate coordinates for current Artist's position
		g.translate(this.getX(), this.getY());

		// Set up the clipping area
		// Clip to the intersection of the inherited (ie parent's) clipping area 
		// and the bounds of the object itself
		g.clipRect(0, 0, this.getWidth(), this.getHeight());


		/*
		 * Draw the game
		 */
		this.paintSquares(g);
		
		// this.paintNewAttackEffect(g);
		
		this.paintLastMoveEffect(g);
		this.paintAttackLines(g);
		
		this.paintPieces(g);
		this.paintSquareOverlay(g);
		this.paintActivePiece(g);

		// Set the color and stroke to the defaults for the frame around the board.
		g.setColor(BORDER_COLOR);
		g.setStroke(BORDER_STROKE);

		// Draw the frame Artist, adjusting the drawn dimensions to 
		// take into account the stroke size
		g.draw(new Rectangle(this.getWidth() - 1, this.getHeight() - 1));

		// Call the super-class's version of the paint method to setup content area and draw the children
		// super.paintComponent(g);

		// Undo translation of coordinates for current Artist's position
		g.translate(-this.getX(), -this.getY());

		// Restore the previous clipping area
		g.setClip(oldClip);
	}


	/**
	 * Paint the squares on the board
	 * @param g the graphics object used for painting
	 */
	public void paintSquares(Graphics2D g) {
		int i, j;

		// Save the old clipping area (ie the parent's clipping area) so that it
		// can be restored after this object and its children have been drawn
		Shape oldClip = g.getClip();

		// Clip to the intersection of the content area and the previous clipping area.
		g.clipRect(this.getContentX(), this.getContentY(), this.getContentWidth(), this.getContentHeight());

		// Translate coordinates for Content Offset
		g.translate(this.getContentX(), this.getContentY());

		for (i = 0; i < MAX_COLUMNS; i++) {
			for (j = 0; j < MAX_ROWS; j++) {
				this.squares[i][j].paintComponent(g);
			}
		}

		// Undo translation of coordinates for Content Offset
		// (Clip will be reset by the Sprite)
		g.translate(-this.getContentX(), -this.getContentY());

		// Restore the previous clipping area
		g.setClip(oldClip);
	}
	

	/**
	 * Paint the squares on the board
	 * @param g the graphics object used for painting
	 */
	public void paintSquareOverlay(Graphics2D g) {
		int i, j;

		// Save the old clipping area (ie the parent's clipping area) so that it
		// can be restored after this object and its children have been drawn
		Shape oldClip = g.getClip();

		// Clip to the intersection of the content area and the previous clipping area.
		g.clipRect(this.getContentX(), this.getContentY(), this.getContentWidth(), this.getContentHeight());

		// Translate coordinates for Content Offset
		g.translate(this.getContentX(), this.getContentY());

		for (i = 0; i < MAX_COLUMNS; i++) {
			for (j = 0; j < MAX_ROWS; j++) {
				this.squares[i][j].paintOverlay(g);
			}
		}

		// Undo translation of coordinates for Content Offset
		// (Clip will be reset by the Sprite)
		g.translate(-this.getContentX(), -this.getContentY());

		// Restore the previous clipping area
		g.setClip(oldClip);
	}

	/**
	 * Draw the chess pieces on the board
	 * @param g the graphics object used for painting
	 */
	public void paintPieces(Graphics2D g) {
		int i;

		// Draw White Pieces
		for (i = 0; i < PIECES_PER_SIDE; i++) {
			this.pieces[0][i].paintComponent(g);
		}

		// Draw Black Pieces
		for (i = 0; i < PIECES_PER_SIDE; i++) {
			this.pieces[1][i].paintComponent(g);
		}
	}
	
	/**
	 * Draw the active piece on the board
	 * @param g the graphics object used for painting
	 */
	public void paintActivePiece(Graphics2D g) {
		// Draw the piece being dragged to make sure it is on top
		if (this.isDragging() && this.getSelectedSquare() != null && this.getSelectedSquare().getPiece() != null) {
			this.getSelectedSquare().getPiece().paintComponent(g);
		}
	}
	
	/**
	 * Draw the attack lines for the selected piece
	 * @param g the graphics object used for painting
	 */
	public void paintAttackLines(Graphics2D g) {
		int col, row, i;
		if (this.displayAttackLines()) {
			
			// Setup graphics context
			Shape oldClip = g.getClip();
			g.clipRect(this.getContentX(), this.getContentY(), this.getContentWidth(), this.getContentHeight());
			g.translate(this.getContentX(), this.getContentY());
			
			Square attackingSquare;
			Square currentSquare;
			ChessPiece[] attackingPieces;
			int currentX, currentY;
			
			for (col = 0; col < MAX_COLUMNS; col++) {
				for (row = 0; row < MAX_ROWS; row++) {
					currentSquare = this.squares[col][row];
					
					if (currentSquare.isSelected()) {
						currentX = currentSquare.getCenter().x;
						currentY = currentSquare.getCenter().y;
					
						/*
						 * Black Attack lines
						 */
						if (currentSquare.getBlackAttackingPieces() != null) {
							attackingPieces = currentSquare.getBlackAttackingPieces();
						
							g.setColor(BLACK_ATTACK_LINE_COLOR);
							g.setStroke(ATTACK_LINE_STROKE);
								
							for (i = 0; i < Square.MAX_ATTACKS && attackingPieces[i] != null; i++) {
								attackingSquare = attackingPieces[i].getSquare();
								g.drawLine(currentX, currentY, attackingSquare.getCenter().x, attackingSquare.getCenter().y);
							} 
						}
						
						/*
						 * White Attack lines
						 */
						if (currentSquare.getWhiteAttackingPieces() != null) {
							attackingPieces = currentSquare.getWhiteAttackingPieces();
							
							g.setColor(WHITE_ATTACK_LINE_COLOR);
							g.setStroke(ATTACK_LINE_STROKE);
							
							for (i = 0; i < Square.MAX_ATTACKS && attackingPieces[i] != null; i++) {
								attackingSquare = attackingPieces[i].getSquare();
								g.drawLine(currentX, currentY, attackingSquare.getCenter().x, attackingSquare.getCenter().y);
							}
						}
						
					}
				}
			}
			
			// Undo translation of coordinates for Content Offset
			// (Clip will be reset by the Sprite)
			g.translate(-this.getContentX(), -this.getContentY());
			g.setClip(oldClip);
		}
	}
	

	/**
	 * 
	 * @param g
	 */
	public void paintLastMoveEffect(Graphics2D g) {
		if (this.displayLastMove() && this.lastMoveSource != null && this.lastMoveDest != null) {
			// Setup graphics context
			Shape oldClip = g.getClip();
			g.clipRect(this.getContentX(), this.getContentY(), this.getContentWidth(), this.getContentHeight());
			g.translate(this.getContentX(), this.getContentY());
			
			Point sourcePoint, destPoint;
			sourcePoint = this.lastMoveSource.getCenter();
			destPoint = this.lastMoveDest.getCenter();
			
			g.setStroke(LAST_MOVE_STROKE);
			g.setColor(LAST_MOVE_COLOR);
			g.drawLine(sourcePoint.x, sourcePoint.y, destPoint.x, destPoint.y);
			g.fillOval(sourcePoint.x - LAST_MOVE_DOT_SIZE, sourcePoint.y - LAST_MOVE_DOT_SIZE, (2* LAST_MOVE_DOT_SIZE), (2 * LAST_MOVE_DOT_SIZE));
			
			// Undo translation of coordinates for Content Offset
			// (Clip will be reset by the Sprite)
			g.translate(-this.getContentX(), -this.getContentY());
			g.setClip(oldClip);
		}
	}
	
	/**
	 * Draw the attack effect for the proposed move on the board
	 * @param g the graphics object used for painting
	 */
	public void paintNewAttackEffect(Graphics2D g) {
		int i, j;
		ChessPiece activePiece;
		Square activeSquare, moveSquare;
		Point selectedCenterPoint, moveCenterPoint;
		
		activeSquare = this.hoverSquare;
		
		if (this.isDragging() && activeSquare != null && activeSquare.getPiece() != null) {
		
			activePiece = this.getSelectedSquare().getPiece();
			selectedCenterPoint = activeSquare.getCenter();
			
			// System.out.println("*********** New Attack Effect (" + activePiece + ") **********");
			
			if (activePiece != null) {

				g.setStroke(NEW_ATTACK_STROKE);
				if (activePiece.isBlack()) {
					g.setColor(Square.BLACK_ATTACK_COLOR);
				} else {
					g.setColor(Square.WHITE_ATTACK_COLOR);
				}
				
				for (i = 0; i < MAX_COLUMNS; i++) {
					for (j = 0; j < MAX_ROWS; j++) {
						moveSquare = this.squares[i][j];
						moveCenterPoint = moveSquare.getCenter();
						// System.out.println("Attacks for (" + i + ", " + j + ") = " + moveSquare.getCalculations(true, MOVE_POSSIBLE) + " - " + moveSquare.getCalculations(false, MOVE_POSSIBLE));
						if (moveSquare.getCalculations(activePiece.isBlack(), MOVE_POSSIBLE) > 0) {
						//	System.out.println("Go   from (" + activeSquare.getColumn() + ", " + activeSquare.getRow() + ") to (" + moveSquare.getColumn() + ", " + moveSquare.getRow() + ")");
						//	System.out.println("Draw from (" + selectedCenterPoint.x + ", " + selectedCenterPoint.y + ") to (" + moveCenterPoint.x + ", " + moveCenterPoint.y + ")");
							g.drawLine(selectedCenterPoint.x, selectedCenterPoint.y, moveCenterPoint.x, moveCenterPoint.y);
						}
					}
				}
			
			}
		}
		
		// getCalculations
	}

	//===   PAINT METHOD   =====================================================
	//==========================================================================

}
