package chessTrainingWheels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Image;
import java.awt.Point;


public abstract class ChessPiece extends Sprite {

	//==========================================================================
	//===   INSTANCE VARIABLES   ===============================================


	/**
	 * The square that this piece is currently located on.
	 */
	private Square square;	

	/**	
	 * Is the piece a black piece or not (ie a white piece).
	 */
	private boolean isBlack;

	/**
	 * The image used to draw the chess piece.
	 */
	java.awt.Image pieceImage;

	/**
	 * Is the piece currently on the board?
	 */
	private boolean isVisible;

	/**
	 * How many times has the piece been moved during the game so far?
	 * Used for determining validity of castling.
	 */
	private int movedCount;

	/**
	 * Is the current piece being dragged right now?
	 */
	private boolean isDragging;

	/**
	 * Current location of dragging.
	 */
	private Point dragLocation;

	/**
	 * Position that dragging started from.
	 */
	private Point dragStart;
	
	/**
	 * Reference to the parent board of the piece
	 */
	private Board board;

	//===   INSTANCE VARIABLES   ===============================================
	//==========================================================================





	//==========================================================================
	//===   CONSTANTS   ========================================================

	/**
	 * The (max) size of one side of a piece.
	 * This is a fixed size for doing calculations within a sprite, with the
	 * idea that the entire board can be scaled later if needed.
	 */
	protected final static int PIECE_SIZE 	= 45;
	
	
	/**
	 * Piece "white" color.
	 */
	final static Color whiteColor = Color.green;

	/**
	 * Piece "black" color.
	 */
	final static Color blackColor = Color.black;

	/**
	 * Piece horizontal offset.
	 */
	// final static Integer HOffset = 16;
	final static Integer HOffset = (Square.SIZE - ChessPiece.PIECE_SIZE) / 2 + 3;


	/**
	 * Piece vertical offset.
	 */
	final static Integer VOffset = (Square.SIZE - PIECE_SIZE) / 2;


	/**
	 * Piece contents horizontal offset.
	 */
	final static Integer contentHOffset = 0;


	/**
	 * Piece contents vertical offset.
	 */
	final static Integer contentVOffset = 0;


	//===   CONSTANTS   ========================================================
	//==========================================================================





	//==========================================================================
	//===   CONSTRUCTOR    =====================================================


	public ChessPiece(boolean isBlack, Square square, Board board, Image pieceImage) {
		super(HOffset, VOffset, new Dimension(PIECE_SIZE, PIECE_SIZE));

		this.isBlack = isBlack;
		this.setSquare(square);
		this.pieceImage = pieceImage;
		this.isVisible = true;
		this.movedCount = 0;
		this.isDragging = false;
		this.board = board;
	}

	//===   CONSTRUCTOR    =====================================================
	//==========================================================================





	//==========================================================================
	//===   PIECE METHODS    ===================================================

	/**
	 * Gets the column of the square that the piece is sitting on.
	 */
	public int getColumn() {
		return this.square.getColumn();
	}


	/**
	 * Gets the row of the square that the piece is sitting on.
	 */
	public int getRow() {
		return this.square.getRow();
	}

	/**
	 * Get the board that this piece belongs to.
	 * @return the board that this piece belongs to.
	 */
	public Board getBoard() {
		return this.board;
	}
	
	/**
	 * Get the standard chess piece value of the current piece
	 * @return the value of the current piece
	 */
	public Integer getValue() {
		return 1;
	}

	/**
	 * Sets the column that the piece belongs to.
	 */
	public void setSquare(Square square) {
		// Tell the old square that this piece is no longer located on that square
		if (this.square != null) {
			this.square.setPiece(null);
		}

		// Set the pieces square to the new square
		this.square = square;

		// Set the square's piece to this piece
		if (this.square != null) {
			this.square.setPiece(this);
		}
	}


	/**
	 * Sets the row that the piece belongs to.
	 */
	public Square getSquare() {
		return this.square;
	}

	/**
	 * Is the current piece black (true) or white (false).
	 */
	public boolean isBlack() {
		return this.isBlack;
	}

	/**
	 * Is the current piece visible?  Or has it been removed from the game?
	 */
	public boolean isVisible() {
		return this.isVisible;
	}

	/**
	 * Has the current piece been moved this game?
	 * @return whether the current piece has been moved this game.
	 */
	public boolean hasMoved() {
		return (this.movedCount != 0);
	}
	
	/**
	 * Increment the count of how many times the current piece has been moved this game
	 */
	public void incrementMoved() {
		this.movedCount++;
	}
	
	/**
	 * Decrement the count of how many times the current piece has been moved this game
	 */
	public void decrementMoved() {
		if (this.movedCount > 0)
			this.movedCount--;
	}
	
	/**
	 * Reset this piece (used for starting a new game)
	 * @param startSquare the starting square for this piece
	 */
	public void resetPiece(Square startSquare) {
		this.setSquare(startSquare);
		this.isVisible = true;
		this.movedCount = 0;
		this.isDragging = false;
	}
	
	/**
	 * Is the current piece being dragged?
	 * @return whether the current piece is being dragged or not.
	 */
	public boolean isDragging() {
		return this.isDragging;
	}

	/**
	 * Start dragging the piece.
	 * @param x the x coordinate of the mouse press that started the drag
	 * @param y the y coordinate of the mouse press that started the drag
	 */
	public void startDrag (int x, int y) {
		this.isDragging = true;
		this.dragStart = new Point(x, y);
	}

	/**
	 * Continue dragging the piece
	 * @param x the x coordinate of the mouse drag
	 * @param y the y coordinate of the mouse drag
	 */
	public void updateDrag (int x, int y) {
		this.dragLocation = new Point(x, y);
	}

	/**
	 * Stop dragging the current piece
	 */
	public void endDrag() {
		this.dragLocation = null;
		this.dragStart = null;
		this.isDragging = false;
	}

	/**
	 * Is the new square a valid move for this piece?
	 * @param newColumn the new column to move the piece to.
	 * @param newRow the new square to move the piece to.
	 * @return whether this is a valid move
	 */
	public boolean isValidMove(Square newSquare) {
		// Without knowing the type of piece, all the generic check can do is 
		// to make sure a piece of the same color is not on this square.
		
		// If the new square is empty, then this test is passed.
		if (newSquare.getPiece() == null) {
			return true;
		}
		
		// If the two pieces are not the same color, then this test is passed.
		// Otherwise, return false.
		if (newSquare.getPiece().isBlack() != this.isBlack()) {
			return true;
		} else {
			System.out.println("INVALID MOVE: square not empty - " + newSquare.getColumn() + ", " + newSquare.getRow());
			return false;
		}
	}

	/**
	 * Does moving this piece to the designated square include capturing a piece?
	 * @param newSquare the square that this piece is moving to
	 * @return whether this move is a capture or not
	 */
	public boolean isCapture(Square newSquare) {
		// This move is a capture if the new square has a piece on it and that piece is of
		// a different color than the current piece.
		if (newSquare.getPiece() != null && this.isBlack() != newSquare.getPiece().isBlack()) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Is this move a (valid) castling move?
	 * Return false at all times.  King will override with castling details.
	 * @param newSquare the square that this piece is moving to 
	 * @return whether this is a (valid) castling move
	 */
	public boolean isCastling (Square newSquare) {
		return false;
	}
	
	
	/**
	 * Is this piece in check (right now)?
	 * @return is this piece in check (right now)
	 */
	public boolean isCheck() {
		return false;
	}

	
	/**
	 * Is this piece in check with the proposed move?
	 * @return is this piece in check with the proposed move
	 */
	public boolean isNewCheck() {
		return false;
	}
	
	
	/**
	 * Capture the piece on the new square.
	 * @param newSquare the square that this piece is moving to
	 */
	public void capture (Square newSquare) {
		if (newSquare.getPiece() != null) {
			newSquare.getPiece().remove();
		}
	}
	
	/**
	 * Restore the piece to the board.  
	 * Used for undoing a capture.
	 * @param square the location of the piece on the board
	 */
	public void restore(Square square) {
		this.isVisible = true;
		this.setSquare(square);
	}

	/**
	 * Remove this piece from the game
	 */
	public void remove() {
		this.isVisible = false;
		this.setSquare(null);
	}

	/**
	 * Calculate the possible moves for the current piece.
	 * Store the values based on the calculationType passed in.
	 * @param calucationType the type of action to take when storing the calculation.
	 */
	public void calculateMoves(byte calculationType) {}
	
	/**
	 * 
	 * @param calculationType
	 * @param nextSquare
	 * @param hoverSquare
	 * @return
	 */
	protected Boolean movementBlocked(byte calculationType, Square nextSquare, Square hoverSquare) {
		// If blocked by another piece, stop calculations
		if (calculationType == Board.NEW_ATTACK) {
			// For new attack calculations, don't count the piece being dragged in its current location
			if (nextSquare.getPiece() != null && nextSquare.getPiece().isDragging() == false)
				return true;
			// For new attack calculations, count the piece being dragged in the square its hovered over
			if (nextSquare == hoverSquare)
				return true;
		} else {
			if (nextSquare.getPiece() != null)
				return true;
		}
		return false;
	}
	
	/**
	 * 
	 * @param calculationType
	 * @param nextSquare
	 */
	protected void squareCalculation (byte calculationType, Square nextSquare) {
		// New Attack Calculations for pieces not being dragged
		if (calculationType == Board.NEW_ATTACK && this.isDragging() == false) {
			nextSquare.incrementCalculation(this, Board.NEW_ATTACK);	
			nextSquare.incrementCalculation(this, Board.NEW_ATTACK_NOT_SELECTED);

		} else if (calculationType == Board.NEW_ATTACK && this.isDragging() == true) {
			nextSquare.incrementCalculation(this, Board.NEW_ATTACK);	
			nextSquare.incrementCalculation(this, Board.NEW_ATTACK_SELECTED);
			
		// Don't count squares occupied by a friendly piece when calculating possible moves
		} else if (calculationType != Board.MOVE_POSSIBLE || nextSquare.getPiece() == null || nextSquare.getPiece().isBlack() != this.isBlack()) {
			nextSquare.incrementCalculation(this, calculationType);
		} else if (calculationType == Board.MOVE_POSSIBLE) {
			nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
		}
		
		// Calculate attack possibilities when calculating move possibilities 
		if (calculationType == Board.MOVE_POSSIBLE && nextSquare.getPiece() != null && nextSquare.getPiece().isBlack() != this.isBlack()) {
			nextSquare.incrementCalculation(this, Board.ATTACK_POSSIBLE);
		}
	}
	
	//===   PIECE METHODS    ===================================================
	//==========================================================================





	//==========================================================================
	//===   SIZE AND POSITION METHODS   ========================================

	/**
	 * Gets the x position of this object within the coordinate system
	 * defined by its parent's content region.  Note that his position 
	 * should be calculated based on the layout rules and offsets currently
	 * in effect for the object.
	 * 
	 * @return the current x position of this object
	 */
	public int getX() {
		if (this.isDragging() && this.getDragLocation() != null && this.getDragStart() != null) {
			return (this.getColumn() * Square.SIZE + this.getHOffset() + (this.getDragLocation().x - this.getDragStart().x));
		} else {
			return (this.getColumn() * Square.SIZE + this.getHOffset());
		}
	}


	/**
	 * Gets the y position of this object within the coordinate system
	 * defined by its parent's content region.  Note that his position 
	 * should be calculated based on the layout rules and offsets currently
	 * in effect for the object.
	 * 
	 * @return the current y position of this object
	 */
	public int getY() {
		if (this.isDragging() && this.getDragLocation() != null && this.getDragStart() != null) {
			return (this.getRow() * Square.SIZE + this.getVOffset() + (this.getDragLocation().y - this.getDragStart().y));
		} else {
			return (this.getRow() * Square.SIZE + this.getVOffset());
		}
	}


	/**
	 * Return the starting position of the current drag.
	 * @return
	 */
	protected Point getDragStart() {
		return this.dragStart;
	}


	/**
	 * Return the current position of the drag.
	 * @return
	 */
	protected Point getDragLocation() {
		return this.dragLocation;
	}


	//===   SIZE AND POSITION METHODS   ========================================
	//==========================================================================





	//==========================================================================
	//===   PAINT METHOD   =====================================================

	/**
	 * Paints the chess piece as an image using the provided <code>Graphics2D</code> 
	 * object at the location stored within the object.  
	 * 
	 * @param g the graphics object through which to draw this object.
	 */
	public void paintComponent(Graphics2D g) {
		if (this.isVisible()) {
			// Get and save the current position of the piece to avoid issues when
			// the piece is dragged in the middle of a redraw
			int x, y;
			x = this.getX();
			y = this.getY();

			// Save the old clipping area (ie the parent's clipping area) so that it
			// can be restored after this object and its children have been drawn
			Shape oldClip = g.getClip();

			// Translate coordinates for current Artist's position
			g.translate(x, y);

			// Set up the clipping area
			// Clip to the intersection of the inherited (ie parent's) clipping area 
			// and the bounds of the object itself
			g.clipRect(0, 0, this.getWidth(), this.getHeight());


			// Draw the icon Artist.
			g.drawImage(this.pieceImage, 0, 0, null);

			// Call the super-class's version of the paint method to setup content area and draw the children
			// super.paint(g);

			// Undo translation of coordinates for current Artist's position
			g.translate(-x, -y);

			// Restore the previous clipping area
			g.setClip(oldClip);
		}
	}


	//===   PAINT METHOD   =====================================================
	//==========================================================================

}
