/**
 * ChessTrainingWheels
 * A chess GUI for GNU Chess with game visualations meant to help novice to intermediate chess players.
 * Written by Josh Weihnacht.
 * @author Josh Weihnacht
 * 
 * In addition to the work of Josh Weihnacht, this program contains or depends on code 
 * from the following sources / authors:
 * 
 * Xan Gregg (chessbox_gnu4j, a Java port of GNU Chess (5.0.7) is called as a JAR to provide a CPU opponent, GPL'd)
 * MD. Z. Hossain (xChess, whose code was the model for connecting to GNUChess, GPL'd)
 * Dr. Andrew Davison (animation code, info and links below)
 */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301, USA.
 */

/* This program relies on chessbox_gnu4j, a (GPL'd) Java port of GNU Chess (5.0.7).
 * Those portions are:
 *   Copyright (c) 2005 Xan Gregg (Java port)
 *   Copyright (c) 1999-2002 Free Software Foundation, Inc.
 *
 *   Contact Info:
 *     bug-gnu-chess@gnu.org
 *     xan.gregg@pobox.com
 *	   http://www.forthgo.com/chessbox/
 */

/* Portions of this program's GNU Chess interface are based on 
 * xChess, a multiplatform graphical chess user interface
 * Copyright (C) 2007  MD. Z. Hossain <z_hossain@users.sf.net>
 * For more information visit http://xchess.sourceforge.net
 */

/* This program's screen updating code was based on code written by Dr. Andrew Davison 
 * who requests that a link be included to his book, Killer Game Programming in Java.
 * Web Site for the book: http://fivedots.coe.psu.ac.th/~ad/jg
 * Web page for the code: http://fivedots.coe.psu.ac.th/~ad/jg/code/index.html
 */

package chessTrainingWheels;

//Import Declarations

import javax.swing.*;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.GraphicsEnvironment;
import java.awt.Color;
import java.io.File;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.text.*;
import java.awt.Dimension;


public class ChessTrainingWheels extends JComponent implements MouseListener, MouseMotionListener, Runnable {


	//==========================================================================
	//===   CONSTANTS   ========================================================

	/**
	 * Making Java warnings happy.
	 */
	private static final long serialVersionUID = -791205732531291840L;

	/**
	 * Help Area Width
	 */
	final static int HELP_WIDTH = 250;
	
	/**
	 * Help area height
	 */
	final static int HELP_HEIGHT = 800;
	
	/**
	 * Help area horizontal margin
	 */
	final static int HELP_H_MARGIN = 5;
	
	/**
	 * Help area vertical margin
	 */
	final static int HELP_V_MARGIN = 10;
	
	/**
	 * Width of the chess board
	 */
	final static int BOARD_WIDTH = 600;

	/**
	 * Height of the chess board
	 */
	final static int BOARD_HEIGHT = 650;
	
	/**
	 * Width of the screen
	 */
	final static int SCREEN_WIDTH = BOARD_WIDTH + HELP_H_MARGIN + HELP_WIDTH;

	/**
	 * Height of the screen
	 */
	final static int SCREEN_HEIGHT = BOARD_HEIGHT;
	
	
	/**
	 * Default directory to look for images of the chess pieces
	 */
	public static String DEFAULT_IMG_DIR = "data/images";
	
	//===   CONSTANTS   ========================================================
	//==========================================================================



	//==========================================================================
	//===   INSTANCE VARIABLES   ===============================================

	/**
	 * Frame used to hold the widgets drawn on screen
	 */
	public JFrame frame;

	/**
	 * The board on which the game is played
	 */
	protected Board board;

	/**
	 * Directory to look for images in
	 */
	private String imageDir;

	/**
	 * X Offset from the top left corner of the window for drawing the game
	 */
	private int xOffset;
	
	/**
	 * X Offset from the top left corner of the window for drawing the game
	 */
	private int yOffset;

	/**
	 * Is the game over yet?
	 */
	private boolean gameOver = false;
	
	/**
	 * Various menu items used in the game
	 */
	private JMenuBar menuBar;
	private JMenuItem newGameItem;
	private JCheckBoxMenuItem computerPlaysBlackItem;
	private JMenuItem aboutItem;
	private JMenu helpMenu;
	private JMenuItem exitItem;
	private JMenu fileMenu;
	private JCheckBoxMenuItem possibleMovesItem;
	private JCheckBoxMenuItem relativeThreatsItem;
	private JCheckBoxMenuItem attackLinesItem;
	private JCheckBoxMenuItem attackItem;
	private JCheckBoxMenuItem lastMoveItem;
	private JMenu viewMenu;
	private JMenuItem undoItem;
	private JMenuItem redoItem;
	private JMenu editMenu;
	private Image guideImage;

	//===   INSTANCE VARIABLES   ===============================================
	//==========================================================================





	//==========================================================================
	//===   TIMER VARIABLES    =================================================

	/*
	 * This section's code was whittled down from code written by Dr. Andrew Davison 
	 * who requests that a link be included to his book, Killer Game Programming in Java.
	 * Web Site for the book: http://fivedots.coe.psu.ac.th/~ad/jg
	 * Web page for the code: http://fivedots.coe.psu.ac.th/~ad/jg/code/index.html
	 */

	/**
	 * Number of frames to draw per second
	 */
	private static int DEFAULT_FPS = 50;

	private static final int NO_DELAYS_PER_YIELD = 16;
	/* Number of frames with a delay of 0 ms before the animation thread yields
	 to other running threads. */

	private static int MAX_FRAME_SKIPS = 5;   // was 2;
	// no. of frames that can be skipped in any one animation loop
	// i.e the games state is updated but not rendered

	private static int NUM_FPS = 10;
	// number of FPS values stored to get an average

	private long gameStartTime;
	private double fpsStore[];
	private long framesSkipped = 0L;
	private double upsStore[];

	private Thread animator;           // the thread that performs the animation
	private boolean running = false;   // used to stop the animation thread
	private boolean isPaused = false;
	private long period;                // period between drawing in _nanosecs_

	// off screen rendering
	private Graphics dbg; 
	private Image dbImage = null;

	//===   TIMER VARIABLES    =================================================
	//==========================================================================





	//==========================================================================
	//===   CONSTRUCTOR    =====================================================

	/**
	 * Create a ChessTrainingWheels game.
	 * @param imageDir directory for images of chess pieces
	 */
	public ChessTrainingWheels(String imageDir) {
		super();

		// Set up directory for images of chess pieces
		this.imageDir = imageDir;
		if (this.imageDir==null)
			this.imageDir = new String( DEFAULT_IMG_DIR + File.separator);		
		
		// Set up the frame
		this.frame = new JFrame( "Chess Training Wheels" );
		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.frame.setSize( SCREEN_WIDTH, SCREEN_HEIGHT);
		this.frame.getContentPane().setLayout( new GridLayout( 1, 1 ) );
		this.frame.getContentPane().add( this );
		
		// Listen for mouse movements
		this.frame.addMouseListener(this);
		this.frame.addMouseMotionListener(this);
		
		// Make the frame visible
		this.frame.setVisible( true );
	
	}

	//===   CONSTRUCTOR    =====================================================	
	//==========================================================================


	
	

	//==========================================================================
	//===   CHESS TRAINING WHEELS FUNCTIONS   ==================================
	
	/**
	 * Kick off the setup work for a chess game.
	 * @param args optional path to a folder that contains images of chess pieces.
	 */
	public static void main(String[] args) {

		// Check to see if an optional path to images of chess pieces was passed in
		String dir = null;
		if (args.length>0) dir = args[0];

		// Create a chess game
		ChessTrainingWheels game = new ChessTrainingWheels(dir);
		
		// Setup the chess game
		game.populate();
		
		// Setup the menus
		game.populateMenus();
	
		// Determine the offsets for the board, including the menu bar
		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		Rectangle bounds = env.getMaximumWindowBounds();
		game.xOffset = (int)bounds.getX();
		game.yOffset = (int)bounds.getY() + game.menuBar.getHeight();

		// Kick off the animation
		game.initAnimation();
	}
	
	
	/**
	 * Setup up the chess board and screen
	 */
	public void populate() {
		try {
			// An array of images for the chess pieces
			Image[][] chessIcons = new Image[2][6];

			// Image for the guide on the right
			guideImage = loadImage( this.imageDir + "guide.gif" );;
			
			// Load the images for the white chess pieces
			chessIcons[0][0] = loadImage( this.imageDir + "white" + File.separator + "pawn.gif" );
			chessIcons[0][1] = loadImage( this.imageDir + "white" + File.separator + "knight.gif" );
			chessIcons[0][2] = loadImage( this.imageDir + "white" + File.separator + "bishop.gif" );
			chessIcons[0][3] = loadImage( this.imageDir + "white" + File.separator + "rook.gif" );
			chessIcons[0][4] = loadImage( this.imageDir + "white" + File.separator + "queen.gif" );
			chessIcons[0][5] = loadImage( this.imageDir + "white" + File.separator + "king.gif" );

			// Load the images for the black chess pieces
			chessIcons[1][0] = loadImage( this.imageDir + "black" + File.separator + "pawn.gif" );
			chessIcons[1][1] = loadImage( this.imageDir + "black" + File.separator + "knight.gif" );
			chessIcons[1][2] = loadImage( this.imageDir + "black" + File.separator + "bishop.gif" );
			chessIcons[1][3] = loadImage( this.imageDir + "black" + File.separator + "rook.gif" );
			chessIcons[1][4] = loadImage( this.imageDir + "black" + File.separator + "queen.gif" );
			chessIcons[1][5] = loadImage( this.imageDir + "black" + File.separator + "king.gif" );
			
			// Create the chess board
			board = new Board(5, 5, chessIcons, this);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}// populate
	
	
	/**
	 * Setup the menus.
	 */
	public void populateMenus() {
		
		
		/*
		 * Setup basics of the menu bar
		 */ 
		this.menuBar = new JMenuBar();
		this.frame.setJMenuBar(this.menuBar);
		
		// File menu
		this.fileMenu = new JMenu();
		this.menuBar.add(this.fileMenu);
		this.fileMenu.setText("File");
		
		// Edit menu
		this.editMenu = new JMenu();
		this.menuBar.add(this.editMenu);
		this.editMenu.setText("Edit");
		
		// View menu
		this.viewMenu = new JMenu();
		this.menuBar.add(this.viewMenu);
		this.viewMenu.setText("View");
		
		// Help menu
		this.helpMenu = new JMenu();
		this.menuBar.add(this.helpMenu);
		this.helpMenu.setText("Help");
				
		
		/*
		 * File menu
		 */
		
		// New Game
		this.newGameItem = new JMenuItem("New Game");
		this.fileMenu.add(this.newGameItem);
		newGameItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		newGameItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				System.out.println("New game!");
				newGame();
			}
		});
		
		// Computer Plays Black
		this.computerPlaysBlackItem = new JCheckBoxMenuItem("Computer Plays Black");
		this.computerPlaysBlackItem.setState(true);
		this.computerPlaysBlackItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				board.toggleComputerPlaysBlack();
			}
		});
		this.fileMenu.add(this.computerPlaysBlackItem);
		
		// Quit
		this.exitItem = new JMenuItem();
		this.fileMenu.add(this.exitItem);
		this.exitItem.setText("Quit");
		exitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		this.exitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				System.exit(0);
			}
		});
		

		/*
		 * Edit menu
		 */
		
		// Undo
		UndoAction undoAct = new UndoAction(this.board);
		undoItem = new JMenuItem(undoAct);
		undoItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		editMenu.add(undoItem);

		// Redo
		RedoAction redoAct = new RedoAction(this.board);
		this.redoItem = new JMenuItem(redoAct);
		redoItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		this.editMenu.add(redoItem);
		
		
		/*
		 * View menu
		 */
		
		// Show possible moves
		this.possibleMovesItem = new JCheckBoxMenuItem("Possible Moves");
		this.possibleMovesItem.setState(this.board.displayPossibleMoves());
		this.possibleMovesItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				board.toggleDisplayPossibleMoves();
			}
		});
		this.viewMenu.add(this.possibleMovesItem);
		
		// Show relative threats
		this.relativeThreatsItem = new JCheckBoxMenuItem("Relative Threats");
		this.relativeThreatsItem.setState(this.board.displayRelativeThreats());
		this.relativeThreatsItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				board.toggleDisplayRelativeThreats();
			}
		});
		this.viewMenu.add(this.relativeThreatsItem);
		
		// Show attack lines
		this.attackLinesItem = new JCheckBoxMenuItem("Attack Lines");
		this.attackLinesItem.setState(this.board.displayAttackLines());
		this.attackLinesItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				board.toggleDisplayAttackLines();
			}
		});
		this.viewMenu.add(this.attackLinesItem);
		
		// Show attack counts
		this.attackItem = new JCheckBoxMenuItem("Attack Counts");
		this.attackItem.setState(this.board.displayAttack());
		this.attackItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				board.toggleDisplayAttacks();
			}
		});
		this.viewMenu.add(this.attackItem);
		
		// Show last move
		this.lastMoveItem = new JCheckBoxMenuItem("Last Move");
		this.lastMoveItem.setState(this.board.displayLastMove());
		this.lastMoveItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				board.toggleLastMove();
			}
		});
		this.viewMenu.add(this.lastMoveItem);

		
		/*
		 * Help Menu
		 */
		
		// Show about dialog
		this.aboutItem = new JMenuItem("About Chess Training Wheels");
		this.helpMenu.add(this.aboutItem);
		aboutItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new AboutDialog(ChessTrainingWheels.this.frame).setVisible(true);
			}
		});
	} // populate menus
	
	
	/**
	 * Enable the menu item to allow switching the black side from computer to human.
	 */
	public void enableComputerToggle() {
		this.computerPlaysBlackItem.setEnabled(true);
	}
	
	/**
	 * Disable the menu item to allow switching the black side from computer to human.
	 */
	public void disableComputerToggle() {
		this.computerPlaysBlackItem.setEnabled(false);
	}
	
	
	//===   CHESS TRAINING WHEELS FUNCTIONS   ==================================
	//==========================================================================
	
	
	
	

	//==========================================================================
	//===   UTILITIES   ========================================================

	private java.awt.MediaTracker mt = new java.awt.MediaTracker(this);


	/**
	 * Creates an image object and ensures that it is completely
	 * loaded before returning the object.
	 */
	protected synchronized Image loadImage( String sFile ) {
		if (!(new java.io.File(sFile)).exists()) return null;

		Image img = Toolkit.getDefaultToolkit().getImage( sFile );
		mt.addImage(img,1);

		try { 
			mt.waitForAll(); 
		} catch (InterruptedException  e) {} 
		return img;

	} // loadImage

	//===   UTILITIES   ========================================================
	//==========================================================================

	
	
	
	
	//==========================================================================
	//===   PAINT FUNCTIONS   ==================================================

	/**
	 * Paint the game to the screen.
	 * Implements double buffering by actually drawing a buffered 
	 * image of the game to the screen.
	 */
	private void paintScreen() {
		Graphics g;
		try {
			g = this.getGraphics();
			if ((g != null) && (dbImage != null))
				g.drawImage(dbImage, 0, 0, null);
			g.dispose();
		}
		catch (Exception e)
		{ System.out.println("Graphics context error: " + e);  }
	} // end of paintScreen()


	/**
	 * Paint this game.
	 * Basically a call to the paint method of the board.
	 */
	protected void paintComponent( Graphics g ) {
		g.setColor(Color.white);
		g.fillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
		
		if (board!=null) {
			board.paintComponent((Graphics2D)g);
		}
		
		if (this.guideImage != null) {
			g.drawImage(this.guideImage, BOARD_WIDTH + HELP_H_MARGIN, HELP_V_MARGIN, null);
		}
	}

	//===   PAINT FUNCTIONS   ==================================================	
	//========================================================================== 	
	
	
	
	
	
	//==========================================================================
	//===   ACTIONS        =====================================================

	/**
	 * Action to undo the last move
	 */
	class UndoAction extends AbstractAction {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 9157040340988775995L;
		
		/**
		 * the chess board that this action is associated with
		 */
		private Board board;
		
		/**
		 * Create the undo action
		 * @param board the chess board associated with this action
		 */
		public UndoAction (Board board) {
			super("Undo");
			this.board = board;
		}
		
		/**
		 * Perform the undo by calling undo on the board
		 */
		public void actionPerformed (ActionEvent e) {
			if (this.board != null) {
				this.board.undoMove();
			}
		}
	}
	
	
	/**
	 * Action to redo the previous move
	 */
	class RedoAction extends AbstractAction {
		
		/**
		 * Making Java warnings happy.
		 */
		private static final long serialVersionUID = 2246303169981405331L;
		
		/**
		 * the chess board that this action is associated with
		 */
		private Board board;
		
		/**
		 * Create the redo action
		 * @param board the chess board associated with this action
		 */
		public RedoAction (Board board) {
			super("Redo");
			this.board = board;
		}
		
		/**
		 * Perform the redo by calling redo on the board
		 */
		public void actionPerformed (ActionEvent e) {
			if (this.board != null) {
				this.board.redoMove();
			}
		}
	}
	
	/**
	 * Start a new game
	 */
	public void newGame() {
		this.board.newGame();  // Tell the board to start a new game
		this.computerPlaysBlackItem.setState(true); // Reset the menu to stay in sync with the board
	}
	//===   ACTIONS        =====================================================
	//==========================================================================
	
	
	
	
	
	//==========================================================================
	//===   ANIMATION      =====================================================

	/*
	 * This section's code was whittled down from code written by Dr. Andrew Davison 
	 * who requests that a link be included to his book, Killer Game Programming in Java.
	 * Web Site for the book: http://fivedots.coe.psu.ac.th/~ad/jg
	 * Web page for the code: http://fivedots.coe.psu.ac.th/~ad/jg/code/index.html
	 * 
	 * My butchering of this code could probably be improved upon as the original was a little more
	 * than I needed for this game.  But it works, so it hasn't been a development priority yet.
	 */
	
	/**
	 * Set up animation related stuff
	 */
	private void initAnimation() {

		// TODO Why are animation timing calculations so far off?
		this.period = (long) (1000.0/DEFAULT_FPS) *1000000L;

		// initialize timing elements
		fpsStore = new double[NUM_FPS];
		upsStore = new double[NUM_FPS];
		for (int i=0; i < NUM_FPS; i++) {
			fpsStore[i] = 0.0;
			upsStore[i] = 0.0;
		}
	}

	public void addNotify()
	// wait for the JPanel to be added to the JFrame before starting
	{ super.addNotify();   // creates the peer
	startGame();         // start the thread
	}


	private void startGame()
	// initialise and start the thread 
	{ 
		if (animator == null || !running) {
			animator = new Thread(this);
			animator.start();
		}
	} // end of startGame()


	public void resumeGame()
	// called when the JFrame is activated / deiconified
	{  isPaused = false;  } 


	public void pauseGame()
	// called when the JFrame is deactivated / iconified
	{ isPaused = true;   } 


	public void stopGame() 
	// called when the JFrame is closing
	{  running = false;   }


	public void run()
	/* The frames of the animation are drawn inside the while loop. */
	{
		long beforeTime, afterTime, timeDiff, sleepTime;
		long overSleepTime = 0L;
		int noDelays = 0;
		long excess = 0L;

		gameStartTime = System.currentTimeMillis()* 1000000L;
		beforeTime = gameStartTime;

		running = true;

		while(running) {
			//  gameUpdate();
			gameRender();
			paintScreen();

			afterTime = System.currentTimeMillis() * 1000000L;
			timeDiff = afterTime - beforeTime;
			sleepTime = (period - timeDiff) - overSleepTime;  
		
			if (sleepTime > 0) {   // some time left in this cycle
				try {
					Thread.sleep(sleepTime/1000000L);
				}
				catch(InterruptedException ex){}
				overSleepTime = ((System.currentTimeMillis() * 1000000L) - afterTime) - sleepTime;
			} else {    // sleepTime <= 0; the frame took longer than the period
				excess -= sleepTime;  // store excess time value
				overSleepTime = 0L;

				if (++noDelays >= NO_DELAYS_PER_YIELD) {
					Thread.yield();   // give another thread a chance to run
					noDelays = 0;
				}
			}

			beforeTime = System.currentTimeMillis() * 1000000L;

			/* If frame animation is taking too long, update the game state
	         without rendering it, to get the updates/sec nearer to
	         the required FPS. */
			int skips = 0;
			while((excess > period) && (skips < MAX_FRAME_SKIPS)) {
				excess -= period;
				// gameUpdate();    // update state but don't render
				skips++;
			}
			framesSkipped += skips;
		}

		System.exit(0);   // so window disappears
	} // end of run()

/*
	private void gameUpdate()  { 
		if (!isPaused && !gameOver) {
			//System.out.println("Game update called"); 
		}
	}  // end of gameUpdate()
*/

	private void gameRender() {
		if (dbImage == null){
			dbImage = createImage(SCREEN_WIDTH, SCREEN_HEIGHT);
			if (dbImage == null) {
				System.out.println("dbImage is null");
				return;
			}
			else
				dbg = dbImage.getGraphics();
		}

		// clear the background
		this.paintComponent(dbg);

	}  // end of gameRender()

	//===   ANIMATION      =====================================================
	//==========================================================================

	
	
	
	
	//==========================================================================
	//===   MOUSE METHODS   ====================================================
	
	/**
	 * Handle mouse presses
	 * Convert the coordinates to board coordinates and pass them along to the chess board
	 */
	public void mousePressed(MouseEvent e) {
		// Pass the converted mouse action to the board for processing
		this.board.handlePress(e.getX() - this.xOffset, e.getY() - this.yOffset);
	}	

	
	/**
	 * Handle mouse releases
	 * Convert the coordinates to board coordinates and pass them along to the chess board
	 */
	public void mouseReleased(MouseEvent e) {
		// End any dragging when the mouse is released
		this.board.handleRelease(e.getX() - this.xOffset, e.getY() - this.yOffset);
	}

	/**
	 * Handle mouse dragged
	 * Convert the coordinates to board coordinates and pass them along to the chess board
	 */
	public void mouseDragged(MouseEvent e) {
		// Pass the converted mouse action to the board for processing
		this.board.handleDrag(e.getX() - this.xOffset, e.getY() - this.yOffset);
	}

	/* empty methods to satisfy MouseListener/MouseMovedListener interface */
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {} 
	public void mouseClicked(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}

	//===   MOUSE METHODS   ====================================================
	//==========================================================================



}
