// Written by Josh Weihnacht 2007
// Based on xChess written by MD. Z. Hossain:
//
// xChess is a multiplatform graphical chess user interface
// Copyright (C) 2007  MD. Z. Hossain <z_hossain@users.sf.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
// 02110-1301, USA.
//
// For more information visit http://xchess.sourceforge.net
//

package chessTrainingWheels;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.Vector;

public class EngineInterface {
	
	
	//==========================================================================
	//===   INSTANCE VARIABLES   ===============================================
	
	/**
	 * Reference to the chess engine
	 */
	private Process process;
	
	/**
	 * Pipe for receiving messages from the chess engine
	 */
	private BufferedReader reader;
	
	/**
	 * Pipe for sending messages to the chess engine
	 */
	private PrintWriter writer;
	
	/**
	 * Thread to check for messages from the chess engine
	 */
	private Thread parseThread;
	
	/**
	 * Reference to the game board so that messages can be sent back
	 */
	private Board board;
	
	/**
	 * Regular expression for identifying a move from the chess engine
	 */
	Pattern patternMove = Pattern.compile("(my)?\\s*move\\s*(is)?\\s*[:>=\\-]?\\s*([a-h][1-8][a-h][1-8][nbrq]?)", Pattern.CASE_INSENSITIVE);
	
	/**
	 * Regular expression for identifying a complaint of an illegal move from the chess engine
	 */
	Pattern patternPong = Pattern.compile("Pong ([0-9]+)", Pattern.CASE_INSENSITIVE);

	/**
	 * Regular expression for identifying a complaint of an illegal move from the chess engine
	 */
	Pattern patternIllegal = Pattern.compile("(Illegal move.+)|(Error.+)", Pattern.CASE_INSENSITIVE);
	
	/**
	 * Queue of commands to be sent to the chess engine.
	 * Yes, I know its implemented as a vector even though its called a queue.
	 * While it normally acts as a queue, occasionally items need to be pushed 
	 * to the top of the queue (namely undo commands).  Java's implementation of
	 * vectors seem to be the most convenient data structure to allow items to be 
	 * added both at the front and the back of a queue.
	 */
	Vector <EngineCommand> commandQue;
	
	/**
	 * Counter to create unique ping IDs for each command
	 */
	Integer pingCounter;
	
	//===   INSTANCE VARIABLES   ===============================================
	//==========================================================================
	
	
	
	
	
	//==========================================================================
	//===   CONSTRUCTORS   =====================================================
	
	/**
	 * Create a new chess interface
	 * @param board a reference to the chess board
	 */
	public EngineInterface (Board board) {	
		this.board = board;
		this.commandQue = new Vector<EngineCommand>();
		this.pingCounter = 0;
	}
	
	//===   CONSTRUCTORS   =====================================================
	//==========================================================================
	
	
	
	
	
	//==========================================================================
	//===   ENGINE INTERFACE FUNCTIONS   =======================================
	
	public void start () {	
		start(null);
	}
	
	/**
	 * Start the engine
	 * @param engineStart
	 */
	private void start(String engineStart) {	
		if (engineStart == null) {
			engineStart = "java -cp data/chessbox_gnu4j.jar org.chessbox.gnu4j.Main";
		}
		
		// Start the chess engine
		Runtime runtime = Runtime.getRuntime();
		try {
			process = runtime.exec(engineStart);
			reader = new BufferedReader(new InputStreamReader(
					process.getInputStream()));
			writer = new PrintWriter(new OutputStreamWriter(
					process.getOutputStream()), true);
			runtime.addShutdownHook(new Thread() {
				@Override
				public void run() {
					shutdownPerformed();
				}
			});
		} catch (IOException ex) {
			ex.printStackTrace();
		}
		
		this.initiate();
	}
	
	/**
	 * Parse commands received from the chess engine
	 * @throws IOException
	 */
	protected void parseCommand() throws IOException {
		final String line = readLine();
		if(line == null) return;
	
		
		Matcher matcher = patternMove.matcher(line);
		Matcher pongMatcher = patternPong.matcher(line);
		
		if (matcher.matches()) {
			String move = matcher.group(3).substring(0, 4);
			int sourceColumn, sourceRow, destColumn, destRow;
			
			// Parse move into source and destination columns and rows
			sourceColumn = Notation.toSourceColumn(move);
			sourceRow = Notation.toSourceRow(move);
			destColumn = Notation.toDestColumn(move);
			destRow = Notation.toDestRow(move);
			
			System.out.println("Parser: command = " + move + "; (" + sourceColumn + ", " + sourceRow + ") to (" + destColumn + ", " + destRow + ")");
			this.board.movePiece(sourceColumn, sourceRow, destColumn, destRow);
			
			// Ask the engine for a suggested move
			writeLine("hint");
			
		} else if (pongMatcher.matches()) {
			
			// Sanity check that there are actually commands in the queue
			if (! this.commandQue.isEmpty()) {
				
				// If the pong corresponds to the command at the front of the queue,
				// send that command
				if (pongMatcher.group(1).equals( this.commandQue.firstElement().getPingId().toString() )) {
					EngineCommand currCommand = this.commandQue.remove(0);
					executeCommand(currCommand.getCommand());
					
					// If there are more commands in the queue, send the ping for the next command
					if (! this.commandQue.isEmpty()) {
						sendPing(this.commandQue.firstElement().sendPingId());
					}
					
				} else {
					// If the pong does not match the command at the front of the queue, 
					// ignore it and send a ping for the command at the front of the queue
					System.out.println("Got Pong: " + pongMatcher.group(1) + ", was expecting " + this.commandQue.firstElement().getPingId());
					sendPing(this.commandQue.firstElement().sendPingId());
					
				} // Check command is at front of queue
			} // Check command queue is not empty
			
		} else {
			matcher = patternIllegal.matcher(line);
			if(matcher.matches()) {
				// System.out.println("Parser: received");
				this.board.handleIllegalMove();
			} else {
				// System.out.println("Parser: other");
			}
		
		}
		
	}
	
	/**
	 * Create a new ping ID
	 * @return a new ping ID
	 */
	private Integer createPingID() {
		this.pingCounter++;
		return this.pingCounter;
	}
	
	/**
	 * Shut down everything
	 * Tell the chess engine to quit and free all resources
	 */
	private void shutdownPerformed() {
		if(writer != null) {
			executeCommand("quit");
			writer.close();
		}
		process.destroy();
	}
	
	
	/**
	 * 
	 * @return
	 * @throws IOException
	 */
	public String readLine() throws IOException {
		if (reader.ready()) {
			String string = reader.readLine();
			if(string != null) System.out.println("ENGINE: " + string);
			
			if(string != null && string.length() > 0) {
				// fireDataPrinted(new EngineEvent(this, line));
				return string;
			}
		}
		return null;
	}

	
	/**
	 * 
	 * @param string
	 */
	public void writeLine(String string) {
		// Create a new command
		EngineCommand newCommand = new EngineCommand(string, this.createPingID());
		
		// If the queue is empty, go ahead and send out a ping
		if (this.commandQue.size() == 0) {
			sendPing(newCommand.sendPingId());
		}
		
		// Add the command to the back of the queue
		this.commandQue.add(newCommand);
	}
	
	/**
	 * 
	 * @param string
	 */
	public void writeUrgentLine(String string) {
		// Create a new command
		EngineCommand newCommand = new EngineCommand(string, this.createPingID());
		
		// If the queue is empty, go ahead and send out a ping
		if (this.commandQue.size() == 0) {
			sendPing(newCommand.sendPingId());
		}
		
		// Add the command to the back of the queue
		this.commandQue.add(0, newCommand);
		
		System.out.println("----- Urgent Queue Status -----");
		int i;
		for (i = 0; i < this.commandQue.size(); i++) {
			System.out.println("Ping ID: " + this.commandQue.elementAt(i).getPingId() + " - " + this.commandQue.elementAt(i).getCommand());
		}
	}
	
	
	
	/**
	 * 
	 * @param pingId
	 */
	private void sendPing(Integer pingId) {
		executeCommand("ping " + pingId);
	}
	
	/**
	 * 
	 * @param string
	 */
	private void executeCommand(String string) {
		System.out.println("Writing: " + string);
		writer.println(string);
	}
	
	/**
	 * Send a move to the chess engine
	 * @param sourceColumn
	 * @param sourceRow
	 * @param destColumn
	 * @param destRow
	 */
	public void sendMove(int sourceColumn, int sourceRow, int destColumn, int destRow) {
		String command = Notation.toString(sourceColumn, sourceRow) + Notation.toString(destColumn, destRow);
		sendMove(command);
	}
	
	/**
	 * Send a move to the chess engine
	 * @param command the move
	 */
	public void sendMove(String command) {
		writeLine(command);
	}
	
	/**
	 * Tell the chess board about a move from the chess engine
	 * @param sourceColumn
	 * @param sourceRow
	 * @param destColumn
	 * @param destRow
	 */
	public void receiveMove(int sourceColumn, int sourceRow, int destColumn, int destRow) {
		// String command = Notation.toString(sourceColumn, sourceRow) + Notation.toString(destColumn, destRow);
		this.board.movePiece(sourceColumn, sourceRow, destColumn, destRow);
	}
	
	/**
	 * 
	 */
	public void restart() {
		process.destroy();
		start();
	}
	
	
	/**
	 * 
	 */
	public void destroy() {
		this.parseThread = null;
		quitEngine();
		process.destroy();
	}
	
	
	public void initiate() {
		// Setup parsing
		parseThread = new Thread() {
			@Override
			public void run() {
				while(true) {
					// See if the chess engine has said anything
					try { parseCommand(); } 
					catch (IOException e) { e.printStackTrace(); break; }
					
					// Sleep for a bit
					try {Thread.sleep(50L);}
					catch(InterruptedException ex){}
				}
			}
		};
		parseThread.start();
		
		executeCommand("xboard");
		executeCommand("protover 2");
	}	
	
	public void newGame() {
		writeLine("new");
		setPondering(false);
	}
	
	
	public void quitEngine() {
		executeCommand("quit");
	}
	
	public void setPondering(boolean on) {
		if(on) writeLine("hard");
		else writeLine("easy");
	}
	
	///////////////////////////////////////

	/*
	public EngineIO(String command) {
		this.command = command;
		start();
	}
	

	public Process getProcess() {
		return process;
	}

	public String getCommand() {
		return command;
	}
	
	*/
	
	//===   ENGINE INTERFACE FUNCTIONS   =======================================
	//==========================================================================
	
}
