package chessTrainingWheels;

import java.awt.Image;

public class King extends ChessPiece {

	//==========================================================================
	//===   CONSTRUCTOR    =====================================================

	/**
	 * Constructor for the King piece.
	 * @param isBlack is this piece black or not (ie white)?
	 * @param square the square that this piece is on.
	 * @param pieceImage the image for this piece.
	 */
	public King(boolean isBlack, Square square, Board board, Image pieceImage) {
		super(isBlack, square, board, pieceImage);
	}

	//===   CONSTRUCTOR    =====================================================
	//==========================================================================





	//==========================================================================
	//===   PIECE METHODS    ===================================================

	/**
	 * Get the standard chess piece value of the current piece
	 * @return the value of the current piece
	 */
	public Integer getValue() {
		return 100;
	}
	
	/**
	 * Is this piece in check (right now)?
	 * @return is this piece in check (right now)
	 */
	public boolean isCheck() {
		// Is this piece under attack from the other side?
		if (this.getSquare().getCalculations(!this.isBlack(), Board.ATTACK) > 0) {
			return true;
		}
		
		// Return false by default
		return false;
	}

	
	/**
	 * Is this piece in check with the proposed move?
	 * @return is this piece in check with the proposed move
	 */
	public boolean isNewCheck() {
		// Is this piece being dragged and is under attack from the other side in its proposed new location?
		if (this.isDragging() && this.getBoard().getHoverSquare().getCalculations(!this.isBlack(), Board.NEW_ATTACK) > 0) {
			return true;
		}
		
		// Is this piece under attack from the other side?
		if (this.getSquare().getCalculations(!this.isBlack(), Board.NEW_ATTACK) > 0) {
			return true;
		}
		
		// Return false by default
		return false;
	}
	
	
	/**
	 * Is the new square a valid move for this piece?
	 * @param newSquare the new square to move the piece to.
	 * @return whether this is a valid move
	 */
	public boolean isValidMove(Square newSquare) {
		// Check super class for validity first, then do piece specific tests
		if (!super.isValidMove(newSquare)) {
			return false;
		}

		/*
		 * King movement logic
		 * Kings can move one square horizontally, vertically or diagonally.  
		 */
		
		// Move one square in any direction
		if (Math.abs(this.getRow() - newSquare.getRow()) <= 1 && Math.abs(this.getColumn() - newSquare.getColumn()) <= 1) {
			return true;
			
		// Castling
		} else if (this.isCastling(newSquare)) {
			return true;
			
		// Otherwise, invalid
		} else {
			return false;
		}
	}
	
	/**
	 * Calculate the possible moves for the current piece.
	 * Store the values based on the calculationType passed in.
	 * @param calucationType the type of action to take when storing the calculation.
	 */
	public void calculateMoves(byte calculationType) {
		
		if (this.isVisible()) {
			
			int moves[][] = { {1,1}, {0,1}, {-1,1}, {-1,0}, {-1,-1}, {0,-1}, {1,-1}, {1,0} };
			int i, currentColumn, currentRow;
			Square nextSquare;
			
			// New Attack location for piece being dragged vs normal pieces
			if (this.isDragging() && calculationType == Board.NEW_ATTACK) {
				currentColumn = this.getBoard().getHoverSquare().getColumn();
				currentRow = this.getBoard().getHoverSquare().getRow();
			} else {
				currentColumn = this.getColumn();
				currentRow = this.getRow();
			}
			
			// Loop through all possible moves for Knights
			for (i = 0; i < moves.length; i++) {
				// Check if this move is valid (ie the square exists on the board)
				nextSquare = this.getBoard().getSquare(currentColumn + moves[i][0], currentRow + moves[i][1]);
				if (nextSquare != null) {
					
					// Don't count squares occupied by a friendly piece when calculating possible moves
					if (calculationType != Board.MOVE_POSSIBLE || nextSquare.getPiece() == null || nextSquare.getPiece().isBlack() != this.isBlack()) {
						// Increment the calculation if it is
						nextSquare.incrementCalculation(this, calculationType);
					} else if (calculationType == Board.MOVE_POSSIBLE && nextSquare.getPiece() != null && nextSquare.getPiece().isBlack() == this.isBlack()) {
						nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
					}
					
					// Calculate attack possibilities when calculating move possibilities 
					if (calculationType == Board.MOVE_POSSIBLE && nextSquare.getPiece() != null && nextSquare.getPiece().isBlack() != this.isBlack()) {
						nextSquare.incrementCalculation(this, Board.ATTACK_POSSIBLE);
					}
				}
			}
			
			// Castling
			if (this.hasMoved() == false && calculationType == Board.MOVE_POSSIBLE) {
				Square destSquare;
				
				// Check for castling on the left
				destSquare = this.getBoard().getSquare(2, currentRow);
				if (this.isCastling(destSquare)) {
					destSquare.incrementCalculation(this, calculationType);
				}
				
				// Check for castling on the right
				destSquare = this.getBoard().getSquare(6, currentRow);
				if (this.isCastling(destSquare)) {
					destSquare.incrementCalculation(this, calculationType);
				}
			}
		} // Is visible
	}
	
	/**
	 * Castling
	 * 
	 * Is this move a (valid) castling move?
	 * Castling requires that:
	 * The king that makes the castling move has not yet moved in the game.
	 * The rook that makes the castling move has not yet moved in the game.
	 * The king is not in check.
	 * The king does not move over a square that is attacked by an enemy piece during the castling move, 
	 *   i.e., when castling, there may not be an enemy piece that can move (in case of pawns: by diagonal movement) to a square that is moved over by the king.
	 * The king does not move to a square that is attacked by an enemy piece during the castling move, 
	 *   i.e., you may not castle and end the move with the king in check.
	 * * All squares between the rook and king before the castling move are empty.
	 * 
	 * @param newSquare the square that this piece is moving to 
	 * @return whether this is a (valid) castling move
	 */
	public boolean isCastling (Square newSquare) {
		if (this.hasMoved() == false && (newSquare.getColumn() == 2 || newSquare.getColumn() == 6) && ((this.isBlack() == true && newSquare.getRow() == 0) || (this.isBlack() == false && newSquare.getRow() == 7))) {
			
			// Get a reference to the rook being moved
			Square rookSquare;
			if (newSquare.getColumn() == 2) {
				rookSquare = this.getBoard().getSquare(0, this.getRow());
			} else {
				rookSquare = this.getBoard().getSquare(7, this.getRow());
			}
	
			// Invalid castling if the rook has moved
			if (rookSquare.getPiece() == null || rookSquare.getPiece().hasMoved()) {
				return false;
			}
			
			// Determine start and end points of the King's path of movement to check for squares in check
			Square testSquare;
			int i, pathStart, pathEnd;
			if (newSquare.getColumn() < this.getColumn()) {
				pathStart = newSquare.getColumn();
				pathEnd = this.getColumn();
			} else {
				pathStart = this.getColumn();
				pathEnd = newSquare.getColumn();
			}
			
			// Check that no squares along the path are in check
			for (i = pathStart; i <= pathEnd; i++) {
				testSquare = this.getBoard().getSquare(i, this.getRow());
				if (testSquare.getCalculations(!this.isBlack(), Board.ATTACK) > 0) {
					return false;
				}
			}
			
			// Determine start and end points of path between the King and Rook to ensure that no pieces are in the way
			if (rookSquare.getColumn() == 0) {
				pathStart = rookSquare.getColumn() + 1;
				pathEnd = this.getColumn() - 1;
			} else {
				pathStart = this.getColumn() + 1;
				pathEnd = rookSquare.getColumn() - 1;
			}
			
			// Check that there are no pieces in the way
			for (i = pathStart; i <= pathEnd; i++) {
				testSquare = this.getBoard().getSquare(i, this.getRow());
				if (testSquare.getPiece() != null) {
					return false;
				}
			}
			
			// No tests failed, this is a valid castling move
			return true;
			
		} else {
			return false;
		}
	}
	
	// TODO King move calculations for attack
	// TODO King move calculations for possible moves

	//===   PIECE METHODS    ===================================================
	//==========================================================================
}