package chessTrainingWheels;

import java.awt.Image;

public class Knight extends ChessPiece {

	//==========================================================================
	//===   CONSTRUCTOR    =====================================================
	
	/**
	 * Constructor for the Knight piece.
	 * @param isBlack is this piece black or not (ie white)?
	 * @param square the square that this piece is on.
	 * @param pieceImage the image for this piece.
	 */
	public Knight(boolean isBlack, Square square, Board board, Image pieceImage) {
		super(isBlack, square, board, pieceImage);
	}
	
	//===   CONSTRUCTOR    =====================================================
	//==========================================================================





	//==========================================================================
	//===   PIECE METHODS    ===================================================

	/**
	 * Get the standard chess piece value of the current piece
	 * @return the value of the current piece
	 */
	public Integer getValue() {
		return 3;
	}

	/**
	 * Is the new square a valid move for this piece?
	 * @param newSquare the new square to move the piece to.
	 * @return whether this is a valid move
	 */
	public boolean isValidMove(Square newSquare) {
		// Check super class for validity first, then do piece specific tests
		if (!super.isValidMove(newSquare)) {
			return false;
		}


		/*
		 * Knight movement logic
		 * Knights move in an "L", ie one square in one axis and two squares on the other axis.
		 * Knights can also move over other pieces in their path.
		 */

		// Move in a horizontal "L"
		if (Math.abs(this.getRow() - newSquare.getRow()) == 1 && Math.abs(this.getColumn() - newSquare.getColumn()) == 2) {
			return true;
		
		// Move in a vertical "L"
		} else if (Math.abs(this.getRow() - newSquare.getRow()) == 2 && Math.abs(this.getColumn() - newSquare.getColumn()) == 1) {
			return true;
			
		// Otherwise, invalid
		} else {
			return false;
		}
	}
	

	/**
	 * Calculate the possible moves for the current piece.
	 * Store the values based on the calculationType passed in.
	 * @param calucationType the type of action to take when storing the calculation.
	 */
	public void calculateMoves(byte calculationType) {
		
		if (this.isVisible()) {
			
			int moves[][] = { {2,1}, {1,2}, {-1,2}, {-2,1}, {-2,-1}, {-1,-2}, {1,-2}, {2,-1} };
			int i, currentColumn, currentRow;
			Square nextSquare;
			
			// New Attack location for piece being dragged vs normal pieces
			if (this.isDragging() && calculationType == Board.NEW_ATTACK) {
				currentColumn = this.getBoard().getHoverSquare().getColumn();
				currentRow = this.getBoard().getHoverSquare().getRow();
			} else {
				currentColumn = this.getColumn();
				currentRow = this.getRow();
			}
				
			// Loop through all possible moves for Knights
			for (i = 0; i < moves.length; i++) {
				// Check if this move is valid (ie the square exists on the board)
				nextSquare = this.getBoard().getSquare(currentColumn + moves[i][0], currentRow + moves[i][1]);
				if (nextSquare != null) {
					
					// Attack calculations are split into several parts
					if (calculationType == Board.NEW_ATTACK || nextSquare.getPiece() == null || nextSquare.getPiece().isBlack() != this.isBlack()) {
						nextSquare.incrementCalculation(this, calculationType);
						
						// Separate out attack calculations for the piece being dragged
						if (this.isDragging()) {
							nextSquare.incrementCalculation(this, Board.NEW_ATTACK_SELECTED);
						} else {
							nextSquare.incrementCalculation(this, Board.NEW_ATTACK_NOT_SELECTED);
						}
					
					// Don't count squares occupied by a friendly piece when calculating possible moves
					} else if (calculationType != Board.MOVE_POSSIBLE || nextSquare.getPiece() == null || nextSquare.getPiece().isBlack() != this.isBlack()) {
						// Increment the calculation if it is
						nextSquare.incrementCalculation(this, calculationType);
					} else if (calculationType == Board.MOVE_POSSIBLE && nextSquare.getPiece() != null && nextSquare.getPiece().isBlack() == this.isBlack()) {
						nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
					}
					
					// Calculate attack possibilities when calculating move possibilities 
					if (calculationType == Board.MOVE_POSSIBLE && nextSquare.getPiece() != null && nextSquare.getPiece().isBlack() != this.isBlack()) {
						nextSquare.incrementCalculation(this, Board.ATTACK_POSSIBLE);
					}
				}
			}
			
		} // Is visible
	}
	//===   PIECE METHODS    ===================================================
	//==========================================================================
}
