package chessTrainingWheels;

/**
 * Move in a chess game.
 * 
 * @author Josh Weihnacht
 *
 */
public class Move {

	private Square sourceSquare;
	private Square destSquare;
	private Square rookSourceSquare;
	private Square rookDestSquare;
	private ChessPiece piece;
	private ChessPiece capturedPiece;
	private ChessPiece castlingPiece;
	private Boolean isCastling;
	private Boolean isCapture;
	private Board board;
	private String command;
	
	/**
	 * Create a new Move object.
	 * Calculate and store the information needed to perform, undo, and redo the move.
	 * 
	 * @param sourceSquare the square the piece starts on
	 * @param destSquare the square the piece ends on
	 * @param piece the piece being moved
	 */
	public Move (Square sourceSquare, Square destSquare, ChessPiece piece) {
		this.sourceSquare = sourceSquare;
		this.destSquare = destSquare;
		this.piece = piece;
		this.board = this.piece.getBoard();
		
		// Determine the xboard command for this move
		this.command = Notation.toString(sourceSquare.getColumn(), sourceSquare.getRow()) + Notation.toString(destSquare.getColumn(), destSquare.getRow());
		
		// If this move is a capture of another piece, remember the piece being captured
		if (this.piece.isCapture(destSquare)) {
			this.isCapture = true;
			this.capturedPiece = destSquare.getPiece();
		} else {
			this.isCapture = false;
			this.capturedPiece = null;	
		}
		
		// Castling.  Move the Rook as well.
		if (this.piece.isCastling(destSquare)) {
			int row = this.sourceSquare.getRow();
			
			// Remember that this is castling
			this.isCastling = true;
			
			// Figure out which rook is being moved and where it is moved to
			if (destSquare.getColumn() == 2) {
				this.castlingPiece = this.board.getSquare(0, row).getPiece();
				this.rookSourceSquare = this.board.getSquare(0, row);
				this.rookDestSquare = this.board.getSquare(3, row);
			} else {
				this.castlingPiece = this.board.getSquare(7, row).getPiece();
				this.rookSourceSquare = this.board.getSquare(7, row);
				this.rookDestSquare = this.board.getSquare(5, row);
			} // Rook selection
			
		} else {
			this.isCastling = false;
			this.castlingPiece = null;
			this.rookSourceSquare = null;
			this.rookDestSquare = null;
		} // End Castling
	} // End Constructor
	
	
	/**
	 * Perform the move
	 */
	public void execute() {
		// If this move is a capture of another piece, capture that piece
		if (this.isCapture()) {
			this.piece.capture(this.destSquare);
		}
		
		// Castling.  Move the Rook as well.
		if (this.isCastling()) {
			this.castlingPiece.setSquare(this.rookDestSquare);
		}

		// Move the piece to the new square
		this.piece.setSquare(this.destSquare);
		
		// Increment the count of how many times this piece has been moved (important for pawns and castling)
		this.piece.incrementMoved();
		
		this.board.setLastMoveEffect(this.sourceSquare, this.destSquare);
	}
	
	/**
	 * Undo this move
	 */
	public void undoMove() {
		// TODO Take into account hasMoved for undo.  Need to make hasMoved a counter.
		
		// Move the piece to its previous square
		this.piece.setSquare(this.sourceSquare);
		
		// Decrement the count of how many times this piece has been moved (important for pawns and castling)
		this.piece.decrementMoved();
		
		// If this move is a capture of another piece, capture that piece
		if (this.isCapture()) {
			this.capturedPiece.restore(this.destSquare);
		}
		
		// Castling.  Move the Rook as well.
		if (this.isCastling()) {
			this.castlingPiece.setSquare(this.rookSourceSquare);
		}
	}
	
	/**
	 * Set the last move.
	 * Needed when this move is at the top of the stack again due to an undo
	 */
	public void setLastMove() {
		this.board.setLastMoveEffect(this.sourceSquare, this.destSquare);
	}
	
	/**
	 * Return the xboard command for this move
	 * @return the xboard command for this move
	 */
	public String getCommand() {
		return this.command;
	}
	
	/**
	 * Is this move a capture of another piece?
	 * @return Is this move a capture of another piece?
	 */
	protected Boolean isCapture() {
		return this.isCapture;
	}
	
	/**
	 * Is this move castling?
	 * @return Is this move castling?
	 */
	protected Boolean isCastling() {
		return this.isCastling;
	}
}
