// Modified by Josh Weihnacht 2007
// Based on xChess written by MD. Z. Hossain:
//
//$Id: Notation.java 2 2007-04-17 19:39:30Z z_hossain $
//
//xChess is a multiplatform graphical chess user interface
//Copyright (C) 2007  MD. Z. Hossain <z_hossain@users.sf.net>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
//02110-1301, USA.
//
//For more information visit http://xchess.sourceforge.net
//

package chessTrainingWheels;

/**
 * Helper object to deal with notation conversion between chess engine notation and board coordinates
 * @author Josh Weihnacht
 */
public class Notation {

	/**
	 * List of columns/files in chess notation
	 */
	public static final char[] FILE_CHAR = { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h' };
	
	/**
	 * List of rows/ranks in chess notation
	 */
	public static final char[] RANK_CHAR = { '1', '2', '3', '4', '5', '6', '7', '8' };

	/**
	 * Convert from a file to a column
	 * @param f the file of a square
	 * @return the column corresponding to the given file
	 */
	public static int getColumn(char f) {
		return (f - 'a');
	}

	/**
	 * Convert from a rank to a row
	 * @param r the rank of a square
	 * @return the row corresponding to the given rank
	 */
	public static int getRow(char r) {
		int row = r - '0';
		return Board.MAX_ROWS - row;
	}

	/**
	 * Convert from a column to a file
	 * @param f the column of a square
	 * @return the file corresponding to the given column
	 */
	public static char getFile(int f) {
		return (char) ('a' + f);
	}

	/**
	 * Convert from a row to a rank
	 * @param r the row of a square
	 * @return the rank corresponding to the given row
	 */
	public static char getRank(int r) {
		return (char) ('0' + (Board.MAX_ROWS - r));
	}
	
	/**
	 * Return the column of the source square for a move
	 * @param move a chess move in normal notation
	 * @return the column of the source square for this move
	 */
	public static int toSourceColumn(String move) {
		return Notation.getColumn(move.charAt(0));
	}
	
	/**
	 * Return the rank of the source square for a move
	 * @param move a chess move in normal notation
	 * @return the rank of the source square for this move
	 */
	public static int toSourceRow(String move) {
		return Notation.getRow(move.charAt(1));
	}
	
	/**
	 * Return the column of the destination square for a move
	 * @param move a chess move in normal notation
	 * @return the column of the destination square for this move
	 */
	public static int toDestColumn(String move) {
		return Notation.getColumn(move.charAt(2));
	}
	
	/**
	 * Return the row of the destination square for a move
	 * @param move a chess move in normal notation
	 * @return the row of the destination square for this move
	 */
	public static int toDestRow(String move) {
		return Notation.getRow(move.charAt(3));
	}
	
	/**
	 * Return chess notation for a square identified by a column and a row
	 * @param column the column of the square
	 * @param row the row of the square
	 * @return the chess notation for the given square
	 */
	public static String toString(int column, int row) {
		return new String(getFile(column) + "" + getRank(row));
	}
	
	
}
