package chessTrainingWheels;


import java.awt.Image;

public class Pawn extends ChessPiece {

	/**
	 * Constructor for the Pawn piece.
	 * @param isBlack is this piece black or not (ie white)?
	 * @param square the square that this piece is on.
	 * @param pieceImage the image for this piece.
	 */
	public Pawn(boolean isBlack, Square square, Board board, Image pieceImage) {
		super(isBlack, square, board, pieceImage);
	}
	
	
	
	//==========================================================================
	//===   PIECE METHODS    ===================================================
	
	/**
	 * Is the new square a valid move for this piece?
	 * @param newSquare the new square to move the piece to.
	 * @return whether this is a valid move
	 */
	public boolean isValidMove(Square newSquare) {
		// Check super class for validity first, then do piece specific tests
		if (!super.isValidMove(newSquare)) {
			return false;
		}
		
		// Check for direction of movement for pawns
		int direction;
		if (this.isBlack()) {
			direction = 1;
		} else {
			direction = -1;
		}
		
		/*
		 * Pawn movement logic
		 * There are 3 cases that return true:
		 * 1. This is a capture, in which case movement is one column in either direction 
		 * and one row in the direction of movement
		 * 2. This is this piece's first move and is not a capture, in which case movement 
		 * is in the same column and can be two rows in the direction of movement
		 * 3. This is not a capture and movement is in the same column and is one row in the
		 * direction of movement
		 */
		
		// Capture move.  Move diagonally when capturing.
		if (this.isCapture(newSquare)) {
			if (Math.abs(this.getColumn() - newSquare.getColumn()) == 1 && (this.getRow() + direction) == newSquare.getRow()) {
				return true;
			} else {
				return false;
			}
			
		// First move can be two squares.
		} else if (this.hasMoved() == false && this.getColumn() == newSquare.getColumn() && (this.getRow() + (2 * direction)) == newSquare.getRow()) {
			return true;
			
		// Normal move is one square
		} else if (this.getColumn() == newSquare.getColumn() && (this.getRow() + direction) == newSquare.getRow()) {
			return true;
			
		// If none of these, its an invalid move
		} else {
			return false;
		}
	}
	
	/**
	 * Calculate the possible moves for the current piece.
	 * Store the values based on the calculationType passed in.
	 * @param calucationType the type of action to take when storing the calculation.
	 */
	public void calculateMoves(byte calculationType) {

		if (this.isVisible()) {

			int nextColumn, nextRow;
			Square nextSquare;

			// Check for direction of movement for pawns
			int direction;
			if (this.isBlack()) {
				direction = 1;
			} else {
				direction = -1;
			}
			
			// Possible moves
			if (calculationType == Board.MOVE_POSSIBLE) {
				
				// Test left capture
				nextRow = this.getRow() + direction;
				nextColumn = this.getColumn() - 1;
				if (nextColumn >= 0 && nextRow >= 0  && nextRow < Board.MAX_ROWS) {
					nextSquare = this.getBoard().getSquare(nextColumn, nextRow);
					if (nextSquare.getPiece() != null && this.isBlack() != nextSquare.getPiece().isBlack()) {
						nextSquare.incrementCalculation(this, calculationType);
						nextSquare.incrementCalculation(this, Board.ATTACK_POSSIBLE);
					}
				}
				
				// Test right capture
				nextRow = this.getRow() + direction;
				nextColumn = this.getColumn() + 1;
				if (nextColumn < Board.MAX_COLUMNS && nextRow >= 0  && nextRow < Board.MAX_ROWS) {
					
					nextSquare = this.getBoard().getSquare(nextColumn, nextRow);
					if (nextSquare.getPiece() != null && this.isBlack() != nextSquare.getPiece().isBlack()) {
						nextSquare.incrementCalculation(this, calculationType);
						nextSquare.incrementCalculation(this, Board.ATTACK_POSSIBLE);
					}
				}
				
				// Test one square ahead
				nextRow = this.getRow() + direction;
				nextColumn = this.getColumn();
				if (nextRow >= 0  && nextRow < Board.MAX_ROWS && this.getBoard().getSquare(nextColumn, nextRow).getPiece() == null) {
					this.getBoard().getSquare(nextColumn, nextRow).incrementCalculation(this, calculationType);
					
					// Test two squares ahead if the piece has not moved yet this game
					if (this.hasMoved() == false) {
						nextRow = this.getRow() + (2 * direction);
						if (nextRow >= 0  && nextRow < Board.MAX_ROWS && this.getBoard().getSquare(nextColumn, nextRow).getPiece() == null) {
							this.getBoard().getSquare(nextColumn, nextRow).incrementCalculation(this, calculationType);
						
						// If failed, mark it as a blocked move
						} else if (nextRow >= 0  && nextRow < Board.MAX_ROWS) {
							this.getBoard().getSquare(nextColumn, nextRow).incrementCalculation(this, Board.MOVE_BLOCKED);
						}
					} // end two squares test
					
				// If failed, mark it as a blocked move
				} else if (nextRow >= 0  && nextRow < Board.MAX_ROWS) {
					this.getBoard().getSquare(nextColumn, nextRow).incrementCalculation(this, Board.MOVE_BLOCKED);
					// Test for blocked two square moves
					nextRow = this.getRow() + (2 * direction);
					nextColumn = this.getColumn();
					if (this.hasMoved() == false && nextRow >= 0  && nextRow < Board.MAX_ROWS)
						this.getBoard().getSquare(nextColumn, nextRow).incrementCalculation(this, Board.MOVE_BLOCKED);
				}
		
			// New Attack calculations if this is the selected piece
			} else if (this.isDragging() && calculationType == Board.NEW_ATTACK && this.getBoard().getHoverSquare() != null) {
				
				int newColumn = this.getBoard().getHoverSquare().getColumn();
				int newRow = this.getBoard().getHoverSquare().getRow();
				
				// Don't calculate attacks when Pawn gets to the home row of the opponent
				// They aren't valid and a promotion is needed anyways
				if (newRow + direction >= 0 && newRow + direction < Board.MAX_ROWS) {
					// Calculate left capture
					if (newColumn - 1 >= 0) {
						nextSquare = this.getBoard().getSquare(newColumn - 1, newRow + direction);
						nextSquare.incrementCalculation(this, calculationType);
						nextSquare.incrementCalculation(this, Board.NEW_ATTACK_SELECTED);
					}
					
					// Calculate right capture
					if (newColumn + 1 < Board.MAX_COLUMNS) {
						nextSquare = this.getBoard().getSquare(newColumn + 1, newRow + direction);
						nextSquare.incrementCalculation(this, calculationType);
						nextSquare.incrementCalculation(this, Board.NEW_ATTACK_SELECTED);
					}
				} // End check for valid row
				
			// Attack calculations			
			} else if ((this.getRow() + direction) >= 0  && (this.getRow() + direction) < Board.MAX_ROWS ) {

				// Calculate left capture
				if (this.getColumn() - 1 >= 0) {
					nextSquare = this.getBoard().getSquare(this.getColumn() - 1, this.getRow() + direction);
					nextSquare.incrementCalculation(this, calculationType);	
					nextSquare.incrementCalculation(this, Board.NEW_ATTACK_NOT_SELECTED);
				}

				// Calculate right capture
				if (this.getColumn() + 1 < Board.MAX_COLUMNS) {
					nextSquare = this.getBoard().getSquare(this.getColumn() + 1, this.getRow() + direction);
					nextSquare.incrementCalculation(this, calculationType);
					nextSquare.incrementCalculation(this, Board.NEW_ATTACK_NOT_SELECTED);
				}
			} // Calculation Type
			
			
		} // Is visible
	}

	//===   PIECE METHODS    ===================================================
	//==========================================================================
}
