package chessTrainingWheels;

import java.awt.Image;

public class Queen extends ChessPiece {

	/**
	 * Constructor for the Queen piece.
	 * @param isBlack is this piece black or not (ie white)?
	 * @param square the square that this piece is on.
	 * @param pieceImage the image for this piece.
	 */
	public Queen(boolean isBlack, Square square, Board board, Image pieceImage) {
		super(isBlack, square, board, pieceImage);
	}
	
	//===   CONSTRUCTOR    =====================================================
	//==========================================================================





	//==========================================================================
	//===   PIECE METHODS    ===================================================
	
	// TODO Queen move calculations for attack
	// TODO Queen move calculations for possible moves
	
	/**
	 * Get the standard chess piece value of the current piece
	 * @return the value of the current piece
	 */
	public Integer getValue() {
		return 9;
	}
	
	/**
	 * Is the new square a valid move for this piece?
	 * @param newSquare the new square to move the piece to.
	 * @return whether this is a valid move
	 */
	public boolean isValidMove(Square newSquare) {
		// Check super class for validity first, then do piece specific tests
		if (!super.isValidMove(newSquare)) {
			return false;
		}
		
		int i, rowDirection, startColumn, startRow, distance;
		boolean conflictFound = false;
		Square testSquare;
	
		/*
		 * Queens movement logic
		 * Queens can move horizontally, vertically or diagonally.  
		 * There must also be no other pieces in the path.
		 */
		
		
		// Vertical move
		if (this.getRow() == newSquare.getRow()) {
			
			// Test the path for any pieces that may be in the way, moving down the board
			if (this.getColumn() < newSquare.getColumn()) {
				for (i = this.getColumn() + 1; i < newSquare.getColumn(); i++) {
					testSquare = this.getBoard().getSquare(i, this.getRow());
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
				
			// Test the path for any pieces that may be in the way, moving up the board
			} else {
				for (i = newSquare.getColumn() + 1; i < this.getColumn(); i++) {
					testSquare = this.getBoard().getSquare(i, this.getRow());
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
			}
			
			// Return false if there was a conflict, true if there was no conflict along the path
			if (conflictFound) {
				return false;
			} else {
				return true;
			}
			
			
		// Horizontal move
		} else if (this.getColumn() == newSquare.getColumn()) {
			// Test the path for any pieces that may be in the way, moving left along the board
			if (this.getRow() < newSquare.getRow()) {
				for (i = this.getRow() + 1; i < newSquare.getRow(); i++) {
					testSquare = this.getBoard().getSquare(this.getColumn(), i);
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
				
			// Test the path for any pieces that may be in the way, moving right along the board
			} else {
				for (i = newSquare.getRow() + 1; i < this.getRow(); i++) {
					testSquare = this.getBoard().getSquare(this.getColumn(), i);
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
			}
			
			// Return false if there was a conflict, true if there was no conflict along the path
			if (conflictFound) {
				return false;
			} else {
				return true;
			}
			
			
		// Diagonal movement
		} else if (Math.abs(this.getRow() - newSquare.getRow()) == Math.abs(this.getColumn() - newSquare.getColumn())) {

			/*
			 *  Test the path for any pieces that may be in the way, moving down the board
			 */

			// Determine the start column, start row and distance for the path test
			if (this.getColumn() < newSquare.getColumn()) {
				startColumn = this.getColumn();
				startRow = this.getRow();
				distance = newSquare.getColumn() - this.getColumn();
			} else {
				startColumn = newSquare.getColumn();
				startRow = newSquare.getRow();
				distance = this.getColumn() - newSquare.getColumn();
			}

			// Determine whether the path goes up or down
			if (this.getColumn() - newSquare.getColumn() == this.getRow() - newSquare.getRow()) {
				rowDirection = 1;
			} else {
				rowDirection = -1;
			}

			// Test the path
			for (i = 1; i < distance; i++) {
				testSquare = this.getBoard().getSquare((startColumn + i), (startRow + (i * rowDirection)));

				// If a piece (or an invalid square) is found in the path, record that there is a conflict
				if (testSquare == null || testSquare.getPiece() != null) {
					conflictFound = true;
					// break;
				}
			}

			// Return false if there was a conflict, true if there was no conflict along the path
			if (conflictFound) {
				return false;
			} else {
				return true;
			}


		// Otherwise not horizontal, vertical or diagonal and thus invalid
		} else {
			return false;
		}
	}
	
	/**
	 * Calculate the possible moves for the current piece.
	 * Store the values based on the calculationType passed in.
	 * @param calucationType the type of action to take when storing the calculation.
	 */
	public void calculateMoves(byte calculationType) {
		
		if (this.isVisible()) {
		
			int i, currentColumn, currentRow;
			Square nextSquare;
			Square hoverSquare = this.getBoard().getHoverSquare();
			
			// New Attack location for piece being dragged vs normal pieces
			if (this.isDragging() && calculationType == Board.NEW_ATTACK) {
				currentColumn = this.getBoard().getHoverSquare().getColumn();
				currentRow = this.getBoard().getHoverSquare().getRow();
			} else {
				currentColumn = this.getColumn();
				currentRow = this.getRow();
			}
			
			/*
			 * Calculate vertical moves
			 */
			
			// Calculate squares down
			for (i = currentRow + 1; i < Board.MAX_ROWS; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);

				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate down
			
			// Calculate blocked squares down 
			for (; i < Board.MAX_ROWS && calculationType == Board.MOVE_POSSIBLE; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			
			// Calculate squares up
			for (i = currentRow - 1; i >= 0; i--) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate up
			
			// Calculate blocked squares up
			for (; i >= 0 && calculationType == Board.MOVE_POSSIBLE; i--) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			/*
			 * Calculate horizontal moves
			 */
			
			// Calculate squares to the left
			for (i = currentColumn + 1; i < Board.MAX_COLUMNS; i++) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate left
			
			// Calculate blocked squares to the left
			for (; i < Board.MAX_COLUMNS && calculationType == Board.MOVE_POSSIBLE; i++) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			// Calculate squares to the right
			for (i = currentColumn - 1; i >= 0; i--) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate right
			
			// Calculate blocked squares down
			for (; i >= 0 && calculationType == Board.MOVE_POSSIBLE; i--) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			/*
			 * Calculate diagonal moves
			 */
			
			// Calculate squares going South West
			for (i = 1; (currentColumn - i) >= 0 && (currentRow + i) < Board.MAX_ROWS; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn - i, currentRow + i);

				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate South West
			
			// Calculate blocked squares South West
			for (;calculationType == Board.MOVE_POSSIBLE && (currentColumn - i) >= 0 && (currentRow + i) < Board.MAX_ROWS; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn - i, currentRow + i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			// Calculate squares going South East
			for (i = 1; (currentColumn + i) < Board.MAX_COLUMNS && (currentRow + i) < Board.MAX_ROWS; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn + i, currentRow + i);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate South East
			
			// Calculate blocked squares South East
			for (;calculationType == Board.MOVE_POSSIBLE && (currentColumn + i) < Board.MAX_COLUMNS && (currentRow + i) < Board.MAX_ROWS; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn + i, currentRow + i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			// Calculate squares going North East
			for (i = 1; (currentColumn + i) < Board.MAX_COLUMNS && (currentRow - i) >= 0; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn + i, currentRow - i);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate North East
			
			// Calculate blocked squares North East
			for (;calculationType == Board.MOVE_POSSIBLE && (currentColumn + i) < Board.MAX_COLUMNS && (currentRow - i) >= 0; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn + i,  currentRow - i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			// Calculate squares going North West
			for (i = 1; (currentColumn - i) >= 0 && (currentRow - i) >= 0; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn - i, currentRow - i);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate North West
			
			// Calculate blocked squares North West
			for (;calculationType == Board.MOVE_POSSIBLE && (currentColumn - i) >= 0 && (currentRow - i) >= 0; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn - i, currentRow - i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
		} // Is visible
	}

	//===   PIECE METHODS    ===================================================
	//==========================================================================
	
}