package chessTrainingWheels;

import java.awt.Image;

public class Rook extends ChessPiece {

	//==========================================================================
	//===   CONSTRUCTOR    =====================================================
	
	/**
	 * Constructor for the Rook piece.
	 * @param isBlack is this piece black or not (ie white)?
	 * @param square the square that this piece is on.
	 * @param pieceImage the image for this piece.
	 */
	public Rook(boolean isBlack, Square square, Board board, Image pieceImage) {
		super(isBlack, square, board, pieceImage);
	}
	
	//===   CONSTRUCTOR    =====================================================
	//==========================================================================





	//==========================================================================
	//===   PIECE METHODS    ===================================================
	
	/**
	 * Get the standard chess piece value of the current piece
	 * @return the value of the current piece
	 */
	public Integer getValue() {
		return 5;
	}
	
	/**
	 * Is the new square a valid move for this piece?
	 * @param newSquare the new square to move the piece to.
	 * @return whether this is a valid move
	 */
	public boolean isValidMove(Square newSquare) {
		// Check super class for validity first, then do piece specific tests
		if (!super.isValidMove(newSquare)) {
			return false;
		}
		
		int i;
		boolean conflictFound = false;
		Square testSquare;
	
		/*
		 * Rook movement logic
		 * Rooks can move either horizontally or vertically.  
		 * Thus either the row or the column must remain the same.
		 * There must also be no other pieces in the path.
		 */
		
		
		// Vertical move
		if (this.getRow() == newSquare.getRow()) {
			
			// Test the path for any pieces that may be in the way, moving down the board
			if (this.getColumn() < newSquare.getColumn()) {
				for (i = this.getColumn() + 1; i < newSquare.getColumn(); i++) {
					testSquare = this.getBoard().getSquare(i, this.getRow());
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
				
			// Test the path for any pieces that may be in the way, moving up the board
			} else {
				for (i = newSquare.getColumn() + 1; i < this.getColumn(); i++) {
					testSquare = this.getBoard().getSquare(i, this.getRow());
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
			}
			
			// Return false if there was a conflict, true if there was no conflict along the path
			if (conflictFound) {
				return false;
			} else {
				return true;
			}
			
			
		// Horizontal move
		} else if (this.getColumn() == newSquare.getColumn()) {
			// Test the path for any pieces that may be in the way, moving left along the board
			if (this.getRow() < newSquare.getRow()) {
				for (i = this.getRow() + 1; i < newSquare.getRow(); i++) {
					testSquare = this.getBoard().getSquare(this.getColumn(), i);
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
				
			// Test the path for any pieces that may be in the way, moving right along the board
			} else {
				for (i = newSquare.getRow() + 1; i < this.getRow(); i++) {
					testSquare = this.getBoard().getSquare(this.getColumn(), i);
					// If a piece (or an invalid square) is found in the path, record that there is a conflict
					if (testSquare == null || testSquare.getPiece() != null) {
						conflictFound = true;
						break;
					}
				}
			}
			
			// Return false if there was a conflict, true if there was no conflict along the path
			if (conflictFound) {
				return false;
			} else {
				return true;
			}
			
			
		// Otherwise, invalid
		} else {
			return false;
		}
	}
	
	/**
	 * Calculate the possible moves for the current piece.
	 * Store the values based on the calculationType passed in.
	 * @param calucationType the type of action to take when storing the calculation.
	 */
	public void calculateMoves(byte calculationType) {
		
		if (this.isVisible()) {
		
			int i, currentColumn, currentRow;
			Square nextSquare;
			Square hoverSquare = this.getBoard().getHoverSquare();
			
			// New Attack location for piece being dragged vs normal pieces
			if (this.isDragging() && calculationType == Board.NEW_ATTACK) {
				currentColumn = this.getBoard().getHoverSquare().getColumn();
				currentRow = this.getBoard().getHoverSquare().getRow();
			} else {
				currentColumn = this.getColumn();
				currentRow = this.getRow();
			}
			
			// Calculate squares down
			for (i = currentRow + 1; i < Board.MAX_ROWS; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate down
			
			// Calculate blocked squares down 
			for (; i < Board.MAX_ROWS && calculationType == Board.MOVE_POSSIBLE; i++) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			
			// Calculate squares up
			for (i = currentRow - 1; i >= 0; i--) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);

				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate up
			
			// Calculate blocked squares up
			for (; i >= 0 && calculationType == Board.MOVE_POSSIBLE; i--) {
				nextSquare = this.getBoard().getSquare(currentColumn, i);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			// Calculate squares to the left
			for (i = currentColumn + 1; i < Board.MAX_COLUMNS; i++) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate left
			
			// Calculate blocked squares to the left
			for (; i < Board.MAX_COLUMNS && calculationType == Board.MOVE_POSSIBLE; i++) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
			
			// Calculate squares to the right
			for (i = currentColumn - 1; i >= 0; i--) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				
				// Update the calculations for this square
				this.squareCalculation(calculationType, nextSquare);
				
				// If blocked by another piece, stop calculations
				if (this.movementBlocked(calculationType, nextSquare, hoverSquare)) 
					break;
			} // calculate right
			
			// Calculate blocked squares down
			for (; i >= 0 && calculationType == Board.MOVE_POSSIBLE; i--) {
				nextSquare = this.getBoard().getSquare(i, currentRow);
				nextSquare.incrementCalculation(this, Board.MOVE_BLOCKED);
			}
			
		} // Is visible
	}

	//===   PIECE METHODS    ===================================================
	//==========================================================================
}