/**
 * @author Josh Weihnacht
 *
 */
package chessTrainingWheels;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;


public abstract class Sprite {


	//==========================================================================
	//===   INSTANCE VARIABLES   ===============================================

	/**
	 * The horizontal layout offset for this object. 
	 */
	private int HOffset;

	/**
	 * The vertical layout offset for this object.
	 */
	private int VOffset;


	/**
	 * The two dimensional size of the object.
	 */
	private Dimension size;


	/**
	 * The horizontal layout offset for the content area of this object. 
	 */
	private int contentHOffset;
	

	/**
	 * The vertical layout offset for the content area of this object. 
	 */
	private int contentVOffset;


	/**
	 * The two dimensional size of the content area of this object.
	 */
	private Dimension contentSize;


	//===   INSTANCE VARIABLES   ===============================================
	//==========================================================================





	//==========================================================================
	//===   CONSTRUCTORS   =====================================================


	/**
	 * Sprite constructor
	 * Does setup work for new Sprite.  Sets the layout and size of the Sprite.
	 * 
	 * @param HLayout specifies the horizontal layout type and must be one of 
	 *        LEFT_JUSTIFY, CENTER_JUSTIFY, or RIGHT_JUSTIFY. 
	 * @param HOffset specifies the horizontal layout distance from the parent 
	 * @param VLayout specifies the vertical layout type and must be one of 
	 *        TOP_JUSTIFY, CENTER_JUSTIFY, or BOTTOM_JUSTIFY.
	 * @param VOffset specifies the vertical layout distance from the parent 
	 * @param size specifies the size of the Sprite
	 */
	Sprite(int HOffset, int VOffset, Dimension size) {
		// Set the horizontal and vertical offsets from the Sprite's parent
		this.HOffset = HOffset;
		this.VOffset = VOffset;

		// Set the size of the Sprite
		this.size = size;
	}

	//===   CONSTRUCTORS   =====================================================
	//==========================================================================





	//==========================================================================
	//===   SIZE AND POSITION METHODS   ========================================

	/**
	 * Gets the layout offsets for this object. 
	 * 
	 * @return the horizontal and vertical offsets used for layout
	 */
	public Point getOffsets() {
		return new Point (this.getHOffset(), this.getVOffset());
	}


	/**
	 * Gets the horizontal layout offset for this object. 
	 * 
	 * @return the horizontal offset used for layout
	 */
	public int getHOffset() {
		return HOffset;
	}


	/**
	 * Gets the vertical layout offset for this object. 
	 * 
	 * @return the vertical offset used for layout
	 */
	public int getVOffset() {
		return VOffset;
	}


	/** 
	 * Set the layout parameters for this object.  
	 * 
	 * @param HOffset specifies the horizontal layout distance from the parent 
	 * @param VOffset specifies the vertical layout distance from the parent 
	 */
	public void setOffset(int HOffset, int VOffset) {
		// Set the horizontal and vertical offsets from the Sprite's parent
		this.HOffset = HOffset;
		this.VOffset = VOffset;
	}


	/**
	 * Gets the position of this object within the coordinate system
	 * defined by its parent's content region.
	 * 
	 * @return the current position of this object
	 */
	public Point getPosition() {
		return new Point(this.getX(), this.getY());
	}


	/**
	 * Gets the x position of this object within the coordinate system
	 * defined by its parent's content region.  Note that his position 
	 * should be calculated based on the layout rules and offsets currently
	 * in effect for the object.
	 * 
	 * @return the current x position of this object
	 */
	public int getX() {
		return this.HOffset;
	}

	/**
	 * Gets the y position of this object within the coordinate system
	 * defined by its parent's content region.  Note that his position 
	 * should be calculated based on the layout rules and offsets currently
	 * in effect for the object.
	 * 
	 * @return the current y position of this object
	 */
	public int getY() {
			return this.VOffset;
	}


	/**
	 * Gets the size of this object.
	 *
	 * @return The size of this object.
	 */
	public Dimension getSize() {
		return this.size;
	}


	/**
	 * Gets the width of this object.
	 *
	 * @return The width of this object.
	 */
	public int getWidth() {
		return this.size.width;
	}


	/**
	 * Gets the height of this object.
	 *
	 * @return The height of this object.
	 */
	public int getHeight() {
		return this.size.height;
	}


	

	/**
	 * Sets the size of this object.
	 *
	 * @param d the new size
	 * @exception AttemptToSetIntrinsicSizeException if one of the dimensions 
	 *            is intrinsic
	 */
	public void setSize(Dimension d) {
		this.size = d;
	}


	//===   SIZE AND POSITION METHODS   ========================================
	//==========================================================================





	//==========================================================================
	//===   CONTENT AREA SIZE METHODS   ========================================


	/**
	 * Gets the horizontal layout offset for this object. 
	 * 
	 * @return the horizontal offset used for layout
	 */
	public int getContentHOffset() {
		return this.contentHOffset;
	}


	/**
	 * Gets the vertical layout offset for this object
	 * . 
	 * @return the vertical offset used for layout
	 */
	public int getContentVOffset() {
		return this.contentVOffset;
	}


	/**
	 * Sets the layout parameters of the content area of this object.
	 * 
	 * @param HOffset specifies the horizontal layout distance from the bounds 
	 * @param VOffset specifies the vertical layout distance from the bounds
	 */    
	public void setContentLayout(int HOffset, int VOffset) {
		this.contentHOffset = HOffset;
		this.contentVOffset = VOffset;
	}


	/**
	 * Gets the position of the content area of this object.
	 *
	 * @return The location of the content area of this object.
	 */
	public Point getContentPosition() {
		// Return the content starting position
		// The getContentX() and getContentY() methods will handle the special
		// case of Sprites that do not support children.
		return new Point(this.getContentX(), this.getContentY());
	}


	/**
	 * Gets the x location of the content area of this object.
	 *
	 * @return The x location of the content area of this object.
	 */
	public int getContentX() {
		return this.getContentHOffset();
	}


	/**
	 * Gets the y location of the content area of this object.
	 *
	 * @return The y location of the content area of this object.
	 */
	public int getContentY() {
		return this.getContentVOffset();
	}


	/**
	 * Gets the size of the content area of this object.
	 *
	 * @return The size of the content area of this object.
	 */
	public Dimension getContentSize() {
		return this.contentSize;
	}


	/**
	 * Gets the width of the content area of this object.
	 *
	 * @return The width of the content area of this object.
	 */
	public int getContentWidth() {
		return this.contentSize.width;
	}



	/**
	 * Gets the height of the content area of this object.
	 *
	 * @return The height of the content area of this object.
	 */
	public int getContentHeight() {
		return this.contentSize.height;
	}



	/**
	 * Sets the size of the content area of this object.
	 *
	 * @param d the new size of the content area
	 */
	public void setContentSize(Dimension d) {
		this.contentSize = d;
	}



	/**
	 * Gets the bounding box of the content area of this object.
	 *
	 * @return The bounding box of the content area of this object.
	 */
	public Rectangle getContentBounds() {
		// Return the content bounds
		// The getContentPosition() and getContentSize() methods will handle the special
		// case of Sprites that do not support children.
		return new Rectangle(this.getContentPosition(), this.getContentSize());
	}

	//===   CONTENT AREA SIZE METHODS   ========================================
	//==========================================================================





	//==========================================================================
	//===   PAINT METHOD   =====================================================

	/**
	 * Paints the children of an Sprite inside of the Sprite's content area using 
	 * the provided <code>Graphics2D</code> object.  
	 *
	 * @param g the graphics object through which to draw this object.
	 */
	public void paintComponent(Graphics2D g) {}

	//===   PAINT METHOD   =====================================================
	//==========================================================================


}
