package chessTrainingWheels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Point;
import java.awt.BasicStroke;

public class Square extends Sprite {

	//==========================================================================
	//===   INSTANCE VARIABLES   ===============================================


	/**
	 * The column that this square belongs to.
	 */
	private int column;

	/**	
	 * The row that this square belongs to.
	 */
	private int row;

	/**	
	 * Is the square a black square or not (ie a white square).
	 */
	private boolean isBlack;

	/**
	 * Is the square currently selected
	 */
	private boolean isSelected;
	
	/**
	 * Is the square currently being hovered over?
	 */
	private boolean isHover;

	/**
	 * The board this square is on.
	 */
	private Board board;
	
	/**
	 * Chess piece currently on this square.
	 */
	private ChessPiece piece;
	
	/**
	 * How many black pieces can attack/defend this square?
	 */
	private int blackAttack;
	
	/**
	 * How many white pieces can attack/defend this square?
	 */
	private int whiteAttack;
	
	/**
	 * How many black pieces can attack/defend this square if the proposed move takes place?
	 */
	private int blackNewAttack;
	
	/**
	 * How many black pieces can attack/defend this square if the proposed move takes place, not counting the piece being dragged?
	 */
	private int blackNewAttackWithoutSelectedPiece;
	
	/**
	 * Can the piece being dragged attack/defend this square if the proposed move takes place?
	 */
	private int blackNewAttackSelectedPiece;
	
	/**
	 * How many white pieces can attack/defend this square if the proposed move takes place?
	 */
	private int whiteNewAttack;
	
	/**
	 * How many white pieces can attack/defend this square if the proposed move takes place, not counting the piece being dragged?
	 */
	private int whiteNewAttackWithoutSelectedPiece;
	
	/**
	 * Can the piece being dragged attack/defend this square if the proposed move takes place?
	 */
	private int whiteNewAttackSelectedPiece;
	
	/**
	 * Can the currently selected piece move to this square?
	 */
	private boolean movePossible;
	
	/**
	 * Could the currently selected piece move to this square if it wasn't blocked?
	 */
	private boolean moveBlocked;
	
	/**
	 * Can the currently selected piece attack this square?
	 */
	private boolean attackPossible;
	
	/**
	 * Array of white pieces that can attack this square
	 */
	private ChessPiece[] whiteAttackingPieces;
	
	/**
	 * Array of black pieces that can attack this square
	 */
	private ChessPiece[] blackAttackingPieces;
	
	//===   INSTANCE VARIABLES   ===============================================
	//==========================================================================





	//==========================================================================
	//===   CONSTANTS   ========================================================
	
	/**
	 * The size of one side of a square.
	 * This is a fixed size for doing calculations within a sprite, with the
	 * idea that the entire board can be scaled later if needed.
	 */

	protected final static int SIZE 	= 72;
	
	/**
	 * The maximum number of pieces from one side that can attack a square
	 */
	protected final static int MAX_ATTACKS = 10;

	/**
	 * Square "white" color.
	 */
	// final static Color whiteColor = new Color(200, 200, 200);  	// Gray scheme
	// final static Color WHITE_COLOR = new Color(255, 206, 158);	// Modified Wikipedia scheme
	// final static Color WHITE_COLOR = new Color(239, 183, 127);	// Modified 2 Wikipedia scheme
	// * final static Color WHITE_COLOR = new Color(216, 185, 91);	// Modified 3 Wikipedia scheme
	// final static Color whiteColor = new Color(245, 196, 146); 	// Wikipedia's scheme
	
	final static Color WHITE_COLOR = new Color(230, 226, 175);	// Kuler scheme 1 ***
	// final static Color WHITE_COLOR = new Color(239, 236, 202);	// Kuler scheme 2
	
	// final static Color WHITE_COLOR = new Color(218, 218, 218);	// YJ Gray

	/**
	 * Square "black" color.
	 */
	// final static Color blackColor = new Color(171, 171, 171);	// Gray scheme
	// *final static Color BLACK_COLOR = new Color(205, 137, 72); 	// Modified 2 Wikipedia's scheme
	// final static Color BLACK_COLOR = new Color(216, 165, 102); 	// Modified 2 Wikipedia's scheme
	// final static Color BLACK_COLOR = new Color(225, 157, 92); 	// Wikipedia's scheme

	// final static Color BLACK_COLOR = new Color(167, 163, 126); 	// Kuler scheme 1
	// final static Color BLACK_COLOR = new Color(180, 180, 138);	// Kuler scheme 1.2
	// final static Color BLACK_COLOR = new Color(204, 186, 143);	// Kuler scheme 1.3 ***
	
	final static Color BLACK_COLOR = new Color(215, 198, 154);	// Kuler scheme 1.4
	
	// final static Color BLACK_COLOR = new Color(184, 184, 184);	// YJ Gray
	
	/**
	 * Transparency of highlighting color effects.
	 */
	final static int HIGHLIGHT_ALPHA = 100;

	/**
	 * Alternate lower transparency of highlighting color effects.
	 */
	final static int HIGHLIGHT_HIGH_ALPHA = 195;
	
	
	/**
	 * Square color for selected effect.
	 */
	final static Color SELECTED_COLOR = new Color(200, 200, 0, HIGHLIGHT_ALPHA);

	/**
	 * Square color for move possible effect.
	 */
	//final static Color MOVE_POSSIBLE_COLOR = new Color(0, 205, 0, HIGHLIGHT_ALPHA);
	// final static Color MOVE_POSSIBLE_COLOR = new Color(20, 165, 20);
	final static Color MOVE_POSSIBLE_COLOR = new Color(135, 185, 110, HIGHLIGHT_HIGH_ALPHA);
	
	
	/**
	 * Square color for move blocked effect.
	 */
	// final static Color MOVE_BLOCKED_COLOR = new Color(20, 20, 20, HIGHLIGHT_ALPHA);
	// final static Color MOVE_BLOCKED_COLOR = new Color(180, 180, 180);
	// final static Color MOVE_BLOCKED_COLOR = new Color(200, 200, 200);
	final static Color MOVE_BLOCKED_COLOR = new Color(166, 163, 132, HIGHLIGHT_HIGH_ALPHA);

	/**
	 * Square color for attack possible effect.
	 */
	final static Color ATTACK_POSSIBLE_COLOR = new Color(255, 0, 0, 200);
		
	/**
	 * Square color for check effect.
	 */
	final static Color CHECK_COLOR = new Color(242, 55, 55, HIGHLIGHT_ALPHA);
	
	
	

	/*
	 * 
	 * Relative Threat Parameters
	 * 
	 */
	
	/**
	 * Size of the relative threat effect
	 */
	final static int RELATIVE_THREAT_SIZE = 56;
		
	/**
	 * Transparency of highlighting color effects.
	 */
	final static int RELATIVE_THREAT_ALPHA = 100;

	/**
	 * Transparency of highlighting color effects.
	 */
	final static int RELATIVE_THREAT_HIGH_ALPHA = 255;
	
	/**
	 * Square color for check effect.
	 */
	final static Color SQUARE_DEFENDED_COLOR = new Color(74, 130, 52, RELATIVE_THREAT_HIGH_ALPHA);

	/**
	 * Square color for check effect.
	 */
	final static Color SQUARE_THREATENED_COLOR = new Color(220, 100, 60, RELATIVE_THREAT_HIGH_ALPHA);

	/**
	 * Size of a half circle.  Used to calculate other parameters for dashed lines.
	 */
	final static float CIRCLE_SIZE = 176;
	
	/**
	 * The number of dash segments in a circle.
	 */
	final static float DASH_COUNT = 6;
	
	/**
	 * Equal threat dash parameter: defines the dash pattern for equal threat strokes
	 */
	final static float SEGMENT_SIZE = (CIRCLE_SIZE / DASH_COUNT);
	
	/**
	 * Equal threat dash offset: defines the space between the dash pattern for equal threat strokes
	 */
	final static float DASH_GAP = 10;

	/**
	 * Equal threat dash parameter: defines the dash pattern for equal threat strokes
	 */
	final static float DASH_SIZE = SEGMENT_SIZE - DASH_GAP;
	
	/**
	 * Tweak to get the gaps in equal threat effect to start at 12 O'Clock
	 */
	final static float DASH_ROTATE_OFFSET = (CIRCLE_SIZE / 4) + (DASH_GAP / 2);

	/**
	 * Relative attack stroke weight
	 */
	final static float RELATIVE_THREAT_STROKE_WEIGHT = 1.5f;
	/**
	 * Stroke for drawing the relative threat
	 */
	final static BasicStroke RELATIVE_THREAT_STROKE = new BasicStroke(RELATIVE_THREAT_STROKE_WEIGHT);

	/**
	 * Stroke for drawing the relative threat
	 */
	final static BasicStroke RELATIVE_THREAT_STROKE_SMALL = 	new BasicStroke(RELATIVE_THREAT_STROKE_WEIGHT, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 10.0f, new float[] {DASH_SIZE, DASH_GAP}, DASH_SIZE + DASH_ROTATE_OFFSET);
		
	/**
	 * Stroke for defended part of of equal threat display
	 */
	final static BasicStroke SQUARE_EQUAL_STROKE = 				new BasicStroke(RELATIVE_THREAT_STROKE_WEIGHT, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 10.0f, new float[] {DASH_SIZE, DASH_SIZE + (2 * DASH_GAP)}, DASH_SIZE + DASH_ROTATE_OFFSET);

	/**
	 * Stroke for threatened part of of equal threat display
	 */
	final static BasicStroke SQUARE_EQUAL_STROKE_2 = 			new BasicStroke(RELATIVE_THREAT_STROKE_WEIGHT, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 10.0f, new float[] {DASH_SIZE, DASH_SIZE + (2 * DASH_GAP)}, 2 * DASH_SIZE + DASH_GAP + DASH_ROTATE_OFFSET);
	
	
	
	
	/*
	 * 
	 * Hover related
	 * 
	 */
	
	/**
	 * Stroke for drawing the selected square
	 */
	final static BasicStroke HOVER_STROKE = new BasicStroke(4);
	
	/**
	 * Stroke for drawing the selected square
	 */
	final static BasicStroke INVALID_HOVER_STROKE = new BasicStroke(1.7F);
	
	/**
	 * Stroke for drawing the selected square
	 */
	final static int HOVER_OFFSET = 10;

	/**
	 * Square color for hovered effect.
	 */
	final static Color HOVER_COLOR = new Color(255, 255, 0, HIGHLIGHT_HIGH_ALPHA);

	
	
	/*
	 * 
	 * Attack effect related
	 * 
	 */
	
	/**
	 * Stroke for drawing the selected square
	 */
	final static BasicStroke EFFECT_STROKE = new BasicStroke(1.7F);
	
	/**
	 * Stroke for drawing the selected square
	 */
	final static BasicStroke MOVE_POSSIBLE_STROKE = new BasicStroke(2.5F);
	
	
	/**
	 * Stroke for drawing the outline around effect dots
	 */
	final static BasicStroke OUTLINE_STROKE = new BasicStroke(0.8F);
	
	/**
	 * Stroke for drawing the selected square
	 */
	final static int EFFECT_OFFSET = 0;
	
	/**
	 * Square offset for hover effect.
	 */
	final static int SELECTED_OFFSET = 0;
	
	/**
	 * Size of the stroke to cross out a lost attack when displaying the attack changes for a proposed move
	 */
	final static BasicStroke NEW_ATTACK_LOST_STROKE = new BasicStroke(1.7F);
	
	/**
	 * Color of the stroke to cross out a lost attack when displaying the attack changes for a proposed move
	 */
	final static Color NEW_ATTACK_LOST_COLOR = Color.red;
	
	/**
	 * Size of the effect to display a new attack when displaying the attack changes for a proposed move
	 */
	final static BasicStroke NEW_ATTACK_GAINED_STROKE = new BasicStroke(1.8F);
	
	/**
	 * Size of the effect to display a new attack when displaying the attack changes for a proposed move
	 */
	final static BasicStroke NEW_ATTACK_GAINED_OUTLINE_STROKE = new BasicStroke(2.0F);

	/**
	 * Color of the attack count for the black side
	 */
	final static Color BLACK_ATTACK_COLOR = Color.black;
	
	/**
	 * Color of the attack count for the white side
	 */
	final static Color WHITE_ATTACK_COLOR = new Color(250, 250, 250);
	
	/**
	 * Color of the attack count for the white side
	 */
	final static Color OUTLINE_COLOR = new Color(20, 20, 20);
	
	/**
	 * Color of the attack count for the white side
	 */
	// final static Color DRAG_ATTACK_COLOR = new Color(255, 255, 0, HIGHLIGHT_HIGH_ALPHA);
	// final static Color DRAG_ATTACK_COLOR = new Color(0, 180, 0, HIGHLIGHT_HIGH_ALPHA);
	// final static Color DRAG_ATTACK_COLOR = new Color(180, 180, 0, HIGHLIGHT_HIGH_ALPHA);
	final static Color DRAG_ATTACK_COLOR = new Color(255, 255, 0, HIGHLIGHT_HIGH_ALPHA);
	
	/**
	 * Size of the attack count effect
	 */
	final static int ATTACK_EFFECT_SIZE = 5;
	
	/**
	 * Size of the attack effect for pieces being dragged
	 */
	final static int SELECTED_ATTACK_EFFECT_SIZE = 20;
	
	/**
	 * Margin from the left of the attack count effect
	 */
	final static int ATTACK_EFFECT_MARGIN_X = 4;
	
	/**
	 * Margin from the top/bottom of the attack count effect
	 */
	final static int ATTACK_EFFECT_MARGIN_Y = 6;
	
	/**
	 * Space between attack indicators for the attack count effect
	 */
	final static int ATTACK_EFFECT_LEADING = 4;
	
	/**
	 * Amount to advance before displaying the next indicator for the attack count effect
	 */
	final static int ATTACK_EFFECT_OFFSET = ATTACK_EFFECT_SIZE + ATTACK_EFFECT_LEADING;

	/**
	 * Board contents horizontal offset.
	 */
	final static Integer contentHOffset = 0;

	/**
	 * Board contents vertical offset.
	 */
	final static Integer contentVOffset = 0;
	

	//===   CONSTANTS   ========================================================
	//==========================================================================





	//==========================================================================
	//===   CONSTRUCTORS   =====================================================

	public Square(int column, int row, Board board) {
		// Call the super constructor to do basic layout setup for the Artist
		super(0, 0, new Dimension(SIZE, SIZE));

		this.board = board;

		this.movePossible = false;
		this.moveBlocked = false;
		this.attackPossible = false;
		
		this.setGridPosition(column, row);
		this.calculateColor();

		/*
		// Size of the content area inside of the board's boarders.
		Dimension contentSize = new Dimension(size);

		// Do basic layout for the Artist's Content Area
		this.setContentLayout(contentHLayout, contentHOffset, contentVLayout, contentVOffset);
		this.setContentSize(contentSize);
		 */
	}

	//===   CONSTRUCTORS   =====================================================
	//==========================================================================





	//==========================================================================
	//===   SQUARE METHODS   ===================================================

	/**
	 * Gets the column that the square belongs to.
	 */
	public int getColumn() {
		return this.column;
	}


	/**
	 * Gets the row that the square belongs to.
	 */
	public int getRow() {
		return this.row;
	}


	/**
	 * Sets the column that the square belongs to.
	 */
	public void setColumn(int column) {
		this.column = column;
	}


	/**
	 * Sets the row that the square belongs to.
	 */
	public void setRow(int row) {
		this.row = row;
	}


	/**
	 * Sets the column and row that the square belongs to.
	 */
	public void setGridPosition(int column, int row) {
		this.setColumn(column);
		this.setRow(row);
	}

	/**
	 * Determine if the current square is black or white.
	 * Store the value for future use.
	 */
	public void calculateColor() {
		if ((this.getColumn() + this.getRow()) % 2 == 0) {
			this.isBlack = false;
		} else {
			this.isBlack = true;
		}
	}

	/**
	 * Is the current square black (true) or white (false).
	 */
	public boolean isBlack() {
		return this.isBlack;
	}

	/**
	 * Is the current square selected.
	 */
	public boolean isSelected() {
		return this.isSelected;
	}

	/**
	 * Set the current square to be selected.
	 */
	public void setSelected() {
		this.isSelected = true;
	}

	/**
	 * Unset the current square from being selected.
	 */
	public void unsetSelected() {
		this.isSelected = false;
	}
	
	/**
	 * Is the current square being hovered over?
	 */
	public boolean isHover() {
		return this.isHover;
	}

	/**
	 * Set the current square to be hovered over.
	 */
	public void setHover() {
		this.isHover = true;
	}

	/**
	 * Unset the current square from being hovered over.
	 */
	public void unsetHover() {
		this.isHover = false;
	}

	/**
	 * Get the piece (if any) currently on this square.
	 * @return the piece currently on this square, or null if no pieces are on this square
	 */
	public ChessPiece getPiece() {
		return this.piece;
	}
	
	/**
	 * Get the array of black pieces that can attack this square
	 * @return the array of black pieces that can attack this square
	 */
	public ChessPiece[] getBlackAttackingPieces() {
		return this.blackAttackingPieces;
	}
	
	/**
	 * Get the array of white pieces that can attack this square
	 * @return the array of white pieces that can attack this square
	 */
	public ChessPiece[] getWhiteAttackingPieces() {
		return this.whiteAttackingPieces;
	}
	
	/**
	 * Get the weakest black piece that can currently attack this square.
	 * @return the weakest black piece that can currently attack this square
	 */
	public ChessPiece getWeakestBlackThreat() {
		if (this.blackAttackingPieces == null) {
			return null;
		} else if (this.blackAttackingPieces[0] == null) {
			return null;
		} else {
			int i = 0;
			ChessPiece weakestThreat = this.blackAttackingPieces[i];
			for (i = 0; i < MAX_ATTACKS - 1 && this.blackAttackingPieces[i] != null && this.blackAttackingPieces[i + 1] != null; i++) {
				if (this.blackAttackingPieces[i].getValue() > this.blackAttackingPieces[i + 1].getValue() && this.blackAttackingPieces[i + 1].getValue() != null) {
					weakestThreat = this.blackAttackingPieces[i + 1];
				}
			}
			return weakestThreat;
		}
	}
	
	/**
	 * Get the value of the weakest black piece that can currently attack this square
	 * @return the value of the weakest black piece that can currently attack this square
	 */
	public Integer getWeakestBlackValue() {
		ChessPiece weakestThreat = getWeakestBlackThreat();
		if (weakestThreat == null) {
			return 0;
		} else {
			return weakestThreat.getValue();
		}
	}
	
	/**
	 * Get the value of the weakest black piece that can currently attack this square, 
	 * including the value passed into the function
	 * @param the threat value of a piece that will be able to attack this square
	 * @return the value of the weakest black piece that can currently attack this square
	 */
	/*
	public Integer getWeakestBlackValue(Integer extraThreat) {
		ChessPiece weakestThreat = getWeakestBlackThreat();
		if (weakestThreat != null && extraThreat > weakestThreat.getValue()) {
			return weakestThreat.getValue();
		} else {
			return extraThreat;
		}
	}
	*/
	
	/**
	 * Get the weakest white piece that can currently attack this square.
	 * @return the weakest white piece that can currently attack this square
	 */
	public ChessPiece getWeakestWhiteThreat() {
		if (this.whiteAttackingPieces == null) {
			return null;
		} else if (this.whiteAttackingPieces[0] == null) {
			return null;
		} else {
			int i = 0;
			ChessPiece weakestThreat = this.whiteAttackingPieces[i];
			for (i = 0; i < MAX_ATTACKS - 1 && this.whiteAttackingPieces[i] != null && this.whiteAttackingPieces[i + 1] != null; i++) {
				if (this.whiteAttackingPieces[i].getValue() > this.whiteAttackingPieces[i + 1].getValue() && this.whiteAttackingPieces[i + 1].getValue() != null) {
					weakestThreat = this.whiteAttackingPieces[i + 1];
				}
			}
			return weakestThreat;
		}
	}

	/**
	 * Get the value of the weakest white piece that can currently attack this square
	 * @return the value of the weakest white piece that can currently attack this square
	 */
	public Integer getWeakestWhiteValue() {
		ChessPiece weakestThreat = getWeakestWhiteThreat();
		if (weakestThreat == null) {
			return 0;
		} else {
			return weakestThreat.getValue();
		}
	}
	
	/**
	 * Get the value of the weakest white piece that can currently attack this square, 
	 * including the value passed into the function
	 * @param the threat value of a piece that will be able to attack this square
	 * @return the value of the weakest white piece that can currently attack this square
	 */
	/*
	public Integer getWeakestWhiteValue(Integer extraThreat) {
		ChessPiece weakestThreat = getWeakestWhiteThreat();
		if (weakestThreat != null && extraThreat > weakestThreat.getValue()) {
			return weakestThreat.getValue();
		} else {
			return extraThreat;
		}
	}
	*/
	
	/**
	 * Set the piece currently on this square.
	 * @param piece the piece to associate with this square
	 */
	public void setPiece (ChessPiece piece) {
		this.piece = piece;
	}
	
	/**
	 * Return calculations for this square, such as the number of 
	 * pieces that can attack it.
	 * @param isBlack are calculations requested for black pieces or white pieces?
	 * @param calculationType what type of calculations are requested
	 * @return
	 */
	public int getCalculations (boolean isBlack, byte calculationType) {
		if (calculationType == Board.ATTACK && isBlack == true) {
			return this.blackAttack;
		} else if (calculationType == Board.ATTACK && isBlack == false) {
			return this.whiteAttack;
		} else if (calculationType == Board.MOVE_POSSIBLE) {
			if (this.movePossible) {
				return 1;
			} else {
				return 0;
			}
			
		} else if (calculationType == Board.NEW_ATTACK && isBlack == true) {
			return this.blackNewAttack;
		} else if (calculationType == Board.NEW_ATTACK && isBlack == false) {
			return this.whiteNewAttack;
		} else if (calculationType == Board.NEW_ATTACK_NOT_SELECTED && isBlack == true) {
			return this.blackNewAttackWithoutSelectedPiece;
		} else if (calculationType == Board.NEW_ATTACK_NOT_SELECTED && isBlack == false) {
			return this.whiteNewAttackWithoutSelectedPiece;
		} else if (calculationType == Board.NEW_ATTACK_SELECTED && isBlack == true) {
			return this.blackNewAttackSelectedPiece;
		} else if (calculationType == Board.NEW_ATTACK_SELECTED && isBlack == false) {
			return this.whiteNewAttackSelectedPiece;
		} else {
			return 0;
		}
	}

	
	/**
	 * Reset the calculations of a given type for both black and white pieces.
	 * @param calculationType what type of calculation should be reset
	 */
	public void resetCalculations (byte calculationType) {
		if (calculationType == Board.ATTACK) {
			this.blackAttack = 0;
			this.whiteAttack = 0;
			this.blackAttackingPieces = null;
			this.whiteAttackingPieces = null;
		} else if (calculationType == Board.NEW_ATTACK) {
			this.blackNewAttack = 0;
			this.blackNewAttackSelectedPiece = 0;
			this.blackNewAttackWithoutSelectedPiece = 0;
			this.whiteNewAttack = 0;
			this.whiteNewAttackSelectedPiece = 0;
			this.whiteNewAttackWithoutSelectedPiece = 0;
		} else if (calculationType == Board.MOVE_POSSIBLE) {
			this.movePossible = false;
		} else if (calculationType == Board.MOVE_BLOCKED) {
			this.moveBlocked = false;
		} else if (calculationType == Board.ATTACK_POSSIBLE) {
			this.attackPossible = false;
		}
	}
	
	/**
	 * Increment the number of black pieces that can attack this square.
	 */
	public void incrementCalculation(ChessPiece piece, byte calculationType) {
		
		if (calculationType == Board.ATTACK && piece.isBlack() == true) {
			if (this.blackAttackingPieces == null) {this.blackAttackingPieces = new ChessPiece[MAX_ATTACKS];}
			this.blackAttackingPieces[blackAttack] = piece;
			this.blackAttack++;
		} else if (calculationType == Board.ATTACK && piece.isBlack() == false) {
			if (this.whiteAttackingPieces == null) {this.whiteAttackingPieces = new ChessPiece[MAX_ATTACKS];}
			this.whiteAttackingPieces[whiteAttack] = piece;
			this.whiteAttack++;
		} else if (calculationType == Board.NEW_ATTACK && piece.isBlack() == true) {
			this.blackNewAttack++;
		} else if (calculationType == Board.NEW_ATTACK && piece.isBlack() == false) {
			this.whiteNewAttack++;
		} else if (calculationType == Board.NEW_ATTACK_NOT_SELECTED && piece.isBlack() == true) {
			this.blackNewAttackWithoutSelectedPiece++;
		} else if (calculationType == Board.NEW_ATTACK_NOT_SELECTED && piece.isBlack() == false) {
			this.whiteNewAttackWithoutSelectedPiece++;
		} else if (calculationType == Board.NEW_ATTACK_SELECTED && piece.isBlack() == true) {
			this.blackNewAttackSelectedPiece++;
		} else if (calculationType == Board.NEW_ATTACK_SELECTED && piece.isBlack() == false) {
			this.whiteNewAttackSelectedPiece++;
		} else if (calculationType == Board.MOVE_POSSIBLE) {
			this.movePossible = true;
		} else if (calculationType == Board.MOVE_BLOCKED) {
			this.moveBlocked = true;
		} else if (calculationType == Board.ATTACK_POSSIBLE) {
			this.attackPossible = true;
		}
	}
	
	/**
	 * Can the currently selected piece move to this square?
	 * @return whether the currently selected piece can move to this square
	 */
	public boolean isMovePossible() {
		return this.movePossible;
	}
	
	/**
	 * Could the currently selected piece move to this square if it wasn't blocked?
	 * @return whether the currently selected piece could move to this square if it wasn't blocked
	 */
	public boolean isMoveBlocked() {
		return this.moveBlocked;
	}
	
	/**
	 * Can the currently selected piece attack this square?
	 * @return whether the currently selected piece can attack this square
	 */
	public boolean isAttackPossible() {
		return this.attackPossible;
	}
	
	/**
	 * Does the current square contain the king, which is in check.
	 * @return
	 */
	public boolean isCheck() {
		// Does this square have a piece and if so is that piece in check?
		// Obviously, the piece will only be in check if it is a king, but the piece
		// will worry about that detail.
		if (this.getPiece() != null && this.getPiece().isCheck()) {
			return true;
		} else {
			return false;
		}
	}
	
	//===   SQUARE METHODS   ===================================================
	//==========================================================================





	//==========================================================================
	//===   SIZE AND POSITION METHODS   ========================================


	/**
	 * Gets the x position of this square within the coordinate system
	 * defined by the board.  Note that his position is calculated based on 
	 * the column number and size of the square.
	 *
	 * @return the current x position of this object
	 */
	public int getX() {
		return this.getColumn() * this.getWidth();
	}

	/**
	 * Gets the y position of this square within the coordinate system
	 * defined by the board.  Note that his position is calculated based on 
	 * the row number and size of the square.
	 * 
	 * @return the current y position of this object
	 */
	public int getY() {

		return this.getRow() * this.getHeight();
	}
	
	/**
	 * Get the center point of the square in the board's coordinate system
	 * @return the center point of the square in the board's coordinate system
	 */
	public Point getCenter() {
		return new Point(this.getX() + (this.getSize().width / 2), this.getY() + (this.getSize().height / 2));
	}


	//===   SIZE AND POSITION METHODS   ========================================
	//==========================================================================





	//==========================================================================
	//===   PAINT METHOD   =====================================================

	/**
	 * Paints the Frame Artist as a black rectangular border, along with any zoom or rotation 
	 * and the Artist's children, using the provided <code>Graphics2D</code> object at the 
	 * location stored within the object.  
	 *
	 * @param g the graphics object through which to draw this object.
	 */
	public void paintComponent(Graphics2D g) {	
		// Save the old clipping area (ie the parent's clipping area) so that it
		// can be restored after this object and its children have been drawn
		Shape oldClip = g.getClip();

		// Translate coordinates for current Artist's position
		g.translate(this.getX(), this.getY());

		// Set up the clipping area
		// Clip to the intersection of the inherited (ie parent's) clipping area 
		// and the bounds of the object itself
		g.clipRect(0, 0, this.getWidth(), this.getHeight());

		
		// Set the color and stroke to the defaults for this frameArtists as a precaution.
		if (this.isCheck()) {
			g.setColor(CHECK_COLOR);
		} else if (this.isBlack()) {
			g.setColor(BLACK_COLOR);
		} else {
			g.setColor(WHITE_COLOR);
		}
		
		// Draw the square
		g.fill(new Rectangle(this.getWidth(), this.getHeight()));
		
		// Draw effects
		this.paintHighlight(g);
		this.paintAttackCount(g);

		// Undo translation of coordinates for current Artist's position
		g.translate(-this.getX(), -this.getY());

		// Restore the previous clipping area
		g.setClip(oldClip);
	}
	
	/**
	 * Draw highlighting effects for the square when dragging occurs.
	 * 
	 * @param g the graphics object through which to draw this object.
	 */
	void paintHighlight (Graphics2D g) {
		int i;
		
		/*
		 * Possible Moves
		 */
		// Draw move blocked effect
		if (this.isMoveBlocked() && this.board.displayPossibleMoves()) {
			g.setColor(MOVE_BLOCKED_COLOR);
			g.setStroke(EFFECT_STROKE);
			g.fill(new Rectangle(EFFECT_OFFSET, EFFECT_OFFSET, this.getWidth() - 2 * EFFECT_OFFSET, this.getHeight() - 2 * EFFECT_OFFSET));
		}
		
		// Draw move possible effect
		if (this.isMovePossible() && this.board.displayPossibleMoves()) {
			g.setColor(MOVE_POSSIBLE_COLOR);
			g.setStroke(MOVE_POSSIBLE_STROKE);
			g.fill(new Rectangle(EFFECT_OFFSET, EFFECT_OFFSET, this.getWidth() - 2 * EFFECT_OFFSET, this.getHeight() - 2 * EFFECT_OFFSET));
		}
		
		// Draw selected effect
		if (this.isSelected() && this.board.displayPossibleMoves()) {
			g.setColor(SELECTED_COLOR);
		//	g.fill(new Rectangle(selectedOffset, selectedOffset, this.getWidth() - (2 * selectedOffset), this.getHeight() - (2 * selectedOffset)));
			g.setStroke(EFFECT_STROKE);
			g.fill(new Rectangle(EFFECT_OFFSET, EFFECT_OFFSET, this.getWidth() - 2 * EFFECT_OFFSET, this.getHeight() - 2 * EFFECT_OFFSET));
		}
		
		// Draw move possible effect
		if (this.isAttackPossible() && this.board.displayPossibleMoves()) {
			g.setColor(ATTACK_POSSIBLE_COLOR);
			g.setStroke(MOVE_POSSIBLE_STROKE);
			//g.fill(new Rectangle(EFFECT_OFFSET, EFFECT_OFFSET, this.getWidth() - 2 * EFFECT_OFFSET, this.getHeight() - 2 * EFFECT_OFFSET));
			g.fillOval(HOVER_OFFSET, HOVER_OFFSET, this.getWidth() - 2 * HOVER_OFFSET, this.getHeight() - 2 * HOVER_OFFSET);
		}
		
		
		
		/*
		 * 
		 * Relative Threats
		 * 
		 */
				
		/* Debug code
		g.setColor(Color.black);
		//g.setFont(new Font());
		g.drawString(this.getWeakestWhiteValue().toString(), 5, 68);
		g.drawString(this.getWeakestBlackValue().toString(), 50, 68);
		*/
		
		if (this.board.displayRelativeThreats() && (this.getWeakestWhiteValue() > 0 || this.getWeakestBlackValue() > 0)) {
			g.setStroke(RELATIVE_THREAT_STROKE);
			if (this.getWeakestWhiteValue() > 0 && this.getWeakestBlackValue() == 0) {
				g.setColor(SQUARE_DEFENDED_COLOR);
			} else if (this.getWeakestWhiteValue() < this.getWeakestBlackValue() && this.getWeakestWhiteValue() > 0) {
				g.setColor(SQUARE_DEFENDED_COLOR);
				g.setStroke(RELATIVE_THREAT_STROKE_SMALL);
			} else if (this.getWeakestWhiteValue() == this.getWeakestBlackValue()) {
				// g.setColor(SQUARE_EQUAL_COLOR);
				// g.setStroke(RELATIVE_THREAT_STROKE_SMALL);
				g.setStroke(SQUARE_EQUAL_STROKE_2);
				g.setColor(SQUARE_THREATENED_COLOR);
			} else if (this.getWeakestWhiteValue() > this.getWeakestBlackValue() && this.getWeakestBlackValue() > 0) {
				g.setColor(SQUARE_THREATENED_COLOR);
				g.setStroke(RELATIVE_THREAT_STROKE_SMALL);
			} else if (this.getWeakestWhiteValue() == 0 && this.getWeakestBlackValue() > 0) {
				g.setColor(SQUARE_THREATENED_COLOR);
			} else {
				// Error suppression, set a clear color if things get to this point
				g.setColor(new Color(0, 0, 0, 0));
			}
			
			// g.fill(new Rectangle(EFFECT_OFFSET, EFFECT_OFFSET, this.getWidth() - 2 * EFFECT_OFFSET, this.getHeight() - 2 * EFFECT_OFFSET));
			//g.fillOval((SIZE-RELATIVE_THREAT_SIZE)/2, (SIZE-RELATIVE_THREAT_SIZE)/2, RELATIVE_THREAT_SIZE, RELATIVE_THREAT_SIZE);
			g.drawOval((SIZE-RELATIVE_THREAT_SIZE)/2, (SIZE-RELATIVE_THREAT_SIZE)/2, RELATIVE_THREAT_SIZE, RELATIVE_THREAT_SIZE);
			
			if (this.getWeakestWhiteValue() == this.getWeakestBlackValue()) {
				// g.setStroke(RELATIVE_THREAT_STROKE);
				
				g.setStroke(SQUARE_EQUAL_STROKE);
				g.setColor(SQUARE_DEFENDED_COLOR);
				g.drawOval((SIZE-RELATIVE_THREAT_SIZE)/2, (SIZE-RELATIVE_THREAT_SIZE)/2, RELATIVE_THREAT_SIZE, RELATIVE_THREAT_SIZE);
				
				// int offset = 0;
				// g.drawOval((SIZE-RELATIVE_THREAT_SIZE)/2 + offset, (SIZE-RELATIVE_THREAT_SIZE)/2 + offset, (RELATIVE_THREAT_SIZE - 2*offset), (RELATIVE_THREAT_SIZE - 2*offset));
				
				/*
				g.setStroke(RELATIVE_THREAT_STROKE_SMALL);
				g.setColor(SQUARE_THREATENED_COLOR);
				offset = 8;
				g.drawOval((SIZE-RELATIVE_THREAT_SIZE)/2 + offset, (SIZE-RELATIVE_THREAT_SIZE)/2 + offset, (RELATIVE_THREAT_SIZE - 2*offset), (RELATIVE_THREAT_SIZE - 2*offset));
				*/
			}
		}
		
		
		/*
		 * Hover Effect
		 */
		// Draw hover effect
		if (this.isHover()) {
			// Use a thick line when hovering over a valid move and a thin line for invalid moves
			if ((this.isMovePossible() || this.isSelected()) && this.board.displayPossibleMoves()) {
				g.setStroke(HOVER_STROKE);
			} else {
				g.setStroke(INVALID_HOVER_STROKE);
			}
			g.setColor(HOVER_COLOR);
			// g.fill(new Rectangle(HOVER_OFFSET, HOVER_OFFSET, this.getWidth() - 2 * HOVER_OFFSET, this.getHeight() - 2 * HOVER_OFFSET));
			g.fillOval(HOVER_OFFSET, HOVER_OFFSET, this.getWidth() - 2 * HOVER_OFFSET, this.getHeight() - 2 * HOVER_OFFSET);
		}
	}
	
	
	/**
	 * Draw the attack counts for both white and black on this square.
	 * 
	 * @param g the graphics object through which to draw this object.
	 */
	void paintAttackCount(Graphics2D g) {
		/**
		 * Attack Stats
		 */
		// Set the default text color as a precaution
		// g.setColor(attackColor);
		// g.setFont(attackFont);
		
		int i = 0, attackCount, newAttackCount, newAttackCountNotSelected, selectedAttackCount;
		int xPos, yPos, xStart = 0;

		/*
		 * White attack statistics
		 */
		
		// TODO Parameterize drawing better
		attackCount = this.getCalculations(false, Board.ATTACK);
		newAttackCount = this.getCalculations(false, Board.NEW_ATTACK);
		newAttackCountNotSelected = this.getCalculations(false, Board.NEW_ATTACK_NOT_SELECTED);
		
		// g.setColor(Color.black);
		// g.drawString(attackCount + " / " + newAttackCount + " / " + newAttackCountNotSelected, 0, 20);
		
		if ((attackCount > 0 || newAttackCount > 0) && this.board.displayAttack()) {
			//Set starting point of dots
			if (newAttackCountNotSelected > attackCount) { 
				xStart = (Square.SIZE - ((newAttackCountNotSelected - 1) * ATTACK_EFFECT_OFFSET) - ATTACK_EFFECT_SIZE) / 2;
			} else {
				xStart = (Square.SIZE - ((attackCount - 1) * ATTACK_EFFECT_OFFSET) - ATTACK_EFFECT_SIZE) / 2;
			}
			
			// Dot: Horizontal
			for (i = 0; i < attackCount; i++) {
				// White set as color
				g.setColor(WHITE_ATTACK_COLOR);
				xPos = xStart + (ATTACK_EFFECT_OFFSET * i);
				yPos = Square.SIZE - ATTACK_EFFECT_MARGIN_Y - ATTACK_EFFECT_SIZE;
				g.fillOval(xPos, yPos, ATTACK_EFFECT_SIZE, ATTACK_EFFECT_SIZE);
				
				// Draw a border around white dots
				g.setColor(OUTLINE_COLOR);
				g.setStroke(OUTLINE_STROKE);
				g.drawOval(xPos, yPos, ATTACK_EFFECT_SIZE, ATTACK_EFFECT_SIZE);
				
				// Indicate the loss of any attacks
				if (i >= newAttackCount) {
					g.setColor(NEW_ATTACK_LOST_COLOR);
					g.setStroke(NEW_ATTACK_LOST_STROKE);
					g.drawLine(xPos, yPos, xPos + ATTACK_EFFECT_SIZE, yPos + ATTACK_EFFECT_SIZE);

				}
			}
			
			for (; i < newAttackCountNotSelected; i++) {	
				xPos = xStart + (ATTACK_EFFECT_OFFSET * i);
				yPos = Square.SIZE - ATTACK_EFFECT_MARGIN_Y - ATTACK_EFFECT_SIZE;

				// Draw a border around white dots
				g.setColor(OUTLINE_COLOR);
				g.setStroke(NEW_ATTACK_GAINED_OUTLINE_STROKE);
				g.drawOval(xPos, yPos, ATTACK_EFFECT_SIZE, ATTACK_EFFECT_SIZE);
				
				// Draw effect circle
				g.setColor(WHITE_ATTACK_COLOR);
				g.setStroke(NEW_ATTACK_GAINED_STROKE);
				g.drawOval(xPos, yPos, ATTACK_EFFECT_SIZE, ATTACK_EFFECT_SIZE);
			}
		}
		
		
		/*
		 * Black attack statistics
		 */
		
		// TODO Parameterize drawing better
		attackCount = this.getCalculations(true, Board.ATTACK);
		newAttackCount = this.getCalculations(true, Board.NEW_ATTACK);
		newAttackCountNotSelected = this.getCalculations(true, Board.NEW_ATTACK_NOT_SELECTED);
		if ((attackCount > 0 || newAttackCount > 0)  && this.board.displayAttack()) {
		
			// Dot: Horizontal
			for (i = 0; i < attackCount; i++) {
				//Set starting point of dots
				if (newAttackCountNotSelected > attackCount) { 
					xStart = (Square.SIZE - ((newAttackCountNotSelected - 1) * ATTACK_EFFECT_OFFSET) - ATTACK_EFFECT_SIZE) / 2;
				} else {
					xStart = (Square.SIZE - ((attackCount - 1) * ATTACK_EFFECT_OFFSET) - ATTACK_EFFECT_SIZE) / 2;
				}
				
				// Black set as color
				g.setColor(BLACK_ATTACK_COLOR);
				
				xPos = xStart + (ATTACK_EFFECT_OFFSET * i);
				yPos = ATTACK_EFFECT_MARGIN_Y;
				
				g.fillOval(xPos, yPos, ATTACK_EFFECT_SIZE, ATTACK_EFFECT_SIZE);
				
				// Indicate the loss of any attacks
				if (i >= newAttackCount) {
					g.setColor(NEW_ATTACK_LOST_COLOR);
					g.setStroke(NEW_ATTACK_LOST_STROKE);
					g.drawLine(xPos, yPos, xPos + ATTACK_EFFECT_SIZE, yPos + ATTACK_EFFECT_SIZE);

				}
			}
						
			for (; i < newAttackCountNotSelected; i++) {	
				xPos = xStart + (ATTACK_EFFECT_OFFSET * i);
				yPos = ATTACK_EFFECT_MARGIN_Y;
				
				g.setColor(BLACK_ATTACK_COLOR);
				g.setStroke(NEW_ATTACK_GAINED_STROKE);
				g.drawOval(xPos, yPos, ATTACK_EFFECT_SIZE, ATTACK_EFFECT_SIZE);
			}
		}
	}
	
	/**
	 * Draw the attack counts for the piece being dragged.
	 * 
	 * @param g the graphics object through which to draw this object.
	 */
	void paintOverlay(Graphics2D g) {
		// Save the old clipping area (ie the parent's clipping area) so that it
		// can be restored after this object and its children have been drawn
		Shape oldClip = g.getClip();

		// Translate coordinates for current Artist's position
		g.translate(this.getX(), this.getY());

		// Set up the clipping area
		// Clip to the intersection of the inherited (ie parent's) clipping area 
		// and the bounds of the object itself
		g.clipRect(0, 0, this.getWidth(), this.getHeight());
		
		
		int selectedAttackCount, xPos, yPos;
		
		// Attack effect for piece being dragged
		selectedAttackCount = this.getCalculations(this.board.getIsBlackTurn(), Board.NEW_ATTACK_SELECTED);
		if (selectedAttackCount > 0) {
			// Draw effect dot
			xPos = (SIZE - SELECTED_ATTACK_EFFECT_SIZE) / 2;
			yPos = xPos;
			g.setColor(DRAG_ATTACK_COLOR);
			g.fillOval(xPos, yPos, SELECTED_ATTACK_EFFECT_SIZE, SELECTED_ATTACK_EFFECT_SIZE);
			
			// Draw outline around effect dot
/*			g.setColor(OUTLINE_COLOR);
			g.setStroke(OUTLINE_STROKE);
			g.drawOval(xPos, yPos, SELECTED_ATTACK_EFFECT_SIZE, SELECTED_ATTACK_EFFECT_SIZE);
			*/
		}
		
		// Undo translation of coordinates for current Artist's position
		g.translate(-this.getX(), -this.getY());

		// Restore the previous clipping area
		g.setClip(oldClip);
	}

	//===   PAINT METHOD   =====================================================
	//==========================================================================
}
