/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

import java.io.*;
import java.util.Arrays;

/**
 * Created by Xan Gregg.
 * Date: May 2, 2005
 */
public class Book extends Common {

    // Java Notes:
    // error handling is only partially converted to using exceptions.

    static final String BOOKSRC = "book.src";

/*
 * BookQuery() tries the following filenames in order to find
 * a valid opening book.
 */
    static String[] bookbin = {
        "book.dat",
        "/usr/share/games/gnuchess/book.dat",
        "/usr/lib/games/gnuchess/book.dat"
    };

/* The book which BookBuilder() uses */
    static final String BOOKRUN = "book.dat";

    static final int R_WHITE_WINS = 1;
    static final int R_BLACK_WINS = 2;
    static final int R_DRAW = 3;
    static final int R_NORESULT = 4;
    static final int BOOKDEPTH = 20;/* We gobble this many moves from each PGN book game */

    static final int MAXMOVES = 200;
    static final int MAXMATCH = 100;

    static int bookcnt;
    static /*HashType*/long[] posshash = new /*HashType*/long[MAXMOVES];

/*
 * This is the only authoritative variable telling us
 * whether a book has been allocated or not. (Other
 * parts may mess around with bookloaded.)
 */
    static boolean book_allocated = false;

/*
 * The last byte of magic_str should be the version
 * number of the format, in case we have to change it.
 *
 * Format 0x01 had an index field which is now gone.
 *
 * Format 0x03 uses an additional size header, which
 * comes directly after the magic string, and has the
 * number of entries as a big-endian encoded uint32_t
 * number.
 */

    static final int MAGIC_LENGTH = 5;

    static byte[] magic_str = {0x42, 0x23, 0x08, 0x15, 0x03};

    static boolean check_magic(InputStream f) {
        byte[] buf = new byte[MAGIC_LENGTH];
        int r;

        try {
            r = f.read(buf, 0, MAGIC_LENGTH);
            return (r == MAGIC_LENGTH && Arrays.equals(buf, magic_str));
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }

    }

    static int write_magic(OutputStream f) {

        try {
            f.write(magic_str);
            return BOOK_SUCCESS;
        } catch (IOException e) {
            e.printStackTrace();
            return BOOK_EIO;
        }


    }

/* Write and read size information for the binary book */

    static int write_size(OutputStream f, int size) throws IOException // uint32_t
    {
        byte[] sizebuf = new byte[4];

        for (int k = 0; k < 4; k++) {
            sizebuf[k] = (byte) ((size >>> ((3 - k) * 8)) & 0xff);
        }
        f.write(sizebuf);
        return BOOK_SUCCESS;
        //return BOOK_EIO;

    }

/* Returns 0 if some read error occurs */

    static int read_size(InputStream f) throws IOException  // uint32_t
    {
        byte[] sizebuf = new byte[4];
        int size = 0;

        if (-1 == f.read(sizebuf)) {
            return 0;
        }
        for (int k = 0; k < 4; k++) {
            size = (size << 8) | (sizebuf[k] & 0xff);
        }
        return size;
    }

/*
 * We now allocate memory for the book dynamically,
 * according to the size field in the header of the binary
 * book. However, during book building the following value
 * is used.
 */

    static final int MAX_DIGEST_BITS = 20;

    static int digest_bits;

    static int DIGEST_SIZE() { return (1 << digest_bits);}

    static int DIGEST_MASK() { return (DIGEST_SIZE() - 1);}

    static class hashtype {
        int wins; // uint16_t
        int losses;
        int draws;
        /*HashType*/long key;
    }

    static hashtype[] bookpos;

    static boolean is_empty(int index) {
        return
                bookpos[index].key == 0 &&
                bookpos[index].wins == 0 &&
                bookpos[index].draws == 0 &&
                bookpos[index].losses == 0;
    }

/*
 * Initial hash function, relies on the quality of the lower
 * bits of the 64 bit hash function
 */
    static int DIGEST_START(/*HashType*/long key) {return (int) (key & DIGEST_MASK()); }   // i =

/*
 * See if there is already a matching entry
 */
    static boolean DIGEST_MATCH(int i, /*HashType*/long the_key) {
        return ((the_key) == bookpos[i].key);
    }
/*
 * See if the entry is empty
 */
    static boolean DIGEST_EMPTY(int i) {return is_empty(i);}

/*
 * Check for collision
 */
    static boolean DIGEST_COLLISION(int i, /*HashType*/long key) { return (!DIGEST_MATCH(i, key) && !DIGEST_EMPTY(i));}

/*
 * The next macro is used in the case of hash collisions.
 * We use double hashing with the higher bits of the key.
 * In order to have the shift relatively prime to the hash
 * size, we OR by 1.
 */
    static int DIGEST_NEXT(int i, /*HashType*/long key) {
        return (int) ((i + (key >>> digest_bits) | 1) & DIGEST_MASK());

    }
/* Mainly for debugging purposes */
    static int bookhashcollisions = 0;

/*
 * This is considered to be the limit for the hash, I chose
 * 95% because it is Monday... No, I am open for suggestions
 * for the right value, I just don't know.
 */
    static double DIGEST_LIMIT() {return (0.95 * DIGEST_SIZE());}

/*
 * This is the record as it will be written in the binary
 * file in network byte order. HashType is uint64_t. To
 * avoid endianness and padding issues, we do not read or
 * write structs but put the values in an unsigned char array.
 */

    static byte[] buf = new byte[2 + 2 + 2 + 8];

/* Offsets */
    static final int wins_off = 0;
    static final int losses_off = 2;
    static final int draws_off = 4;
    static final int key_off = 6;

    static void buf_to_book() {
        /*HashType*/long key;//new /*HashType*/long();
        int i;

        key = ((long) (buf[key_off] & 0xff) << 56)
                | ((long) (buf[key_off + 1] & 0xff) << 48)
                | ((long) (buf[key_off + 2] & 0xff) << 40)
                | ((long) (buf[key_off + 3] & 0xff) << 32)
                | ((long) (buf[key_off + 4] & 0xff) << 24)
                | ((long) (buf[key_off + 5] & 0xff) << 16)
                | ((long) (buf[key_off + 6] & 0xff) << 8)
                | ((long) (buf[key_off + 7] & 0xff));
        /*
         * This is an infinite loop if the hash is 100% full,
         * but other parts should check that this does not happen.
         */
        for (i = DIGEST_START(key);
             DIGEST_COLLISION(i, key);
             i = DIGEST_NEXT(i, key))
                /* Skip */
            bookhashcollisions++;

        bookpos[i].wins += ((buf[wins_off] & 0xff) << 8) | (buf[wins_off + 1] & 0xff);
        bookpos[i].draws += ((buf[draws_off] & 0xff) << 8) | (buf[draws_off + 1] & 0xff);
        bookpos[i].losses += ((buf[losses_off] & 0xff) << 8) | (buf[losses_off + 1] & 0xff);
        bookpos[i].key = key;
    }

    static void book_to_buf(int index) {
        int k;

        for (k = 0; k < 2; k++) {
            buf[wins_off + k] = (byte) (((bookpos[index].wins) >>> ((1 - k) * 8)) & 0xff);
            buf[draws_off + k] = (byte) (((bookpos[index].draws) >>> ((1 - k) * 8)) & 0xff);
            buf[losses_off + k] = (byte) (((bookpos[index].losses) >>> ((1 - k) * 8)) & 0xff);
        }
        for (k = 0; k < 8; k++) {
            buf[key_off + k] = (byte) (((bookpos[index].key) >>> ((7 - k) * 8)) & 0xff);
        }
    }


/*
 * Reads an existing binary book from f. The header must
 * already be skipped, when you call this function. The
 * variable digest_bits must be set to the correct value
 * before calling this function. If any book was allocated
 * before, it will be lost.
 */
    static int read_book(InputStream f) throws IOException {
        if (book_allocated) {
            book_allocated = false;
        }
        bookpos = new hashtype[DIGEST_SIZE()];
        for (int i = 0; i < bookpos.length; i++) {
            bookpos[i] = new hashtype();
        }
        book_allocated = true;
        bookcnt = 0;
        bookhashcollisions = 0;
        while (0 <= f.read(buf)) {
            buf_to_book();
            bookcnt++;
        }
        return BOOK_SUCCESS;
    }

/*
 * Return values are defined in common.h
 */

    static int BookBuilderOpen() {
        InputStream rfp;
        OutputStream wfp;
        int res;
        try {
            try {
                rfp = new FileInputStream(BOOKRUN);
                System.out.println("Opened existing book!\n");
                if (!check_magic(rfp)) {
                    System.err.println("File " + BOOKRUN + " does not conform to the current format.\n" +
                            "Consider rebuilding your book.");
                    return BOOK_EFORMAT;
                }
                /*
                 * We have to read the size header, but in book building we
                 * use the maximum-sized hash table, so we discard the value.
                 */
                digest_bits = MAX_DIGEST_BITS;
                read_size(rfp);
                res = read_book(rfp);
                rfp.close();
                if (res != BOOK_SUCCESS) {
                    return res;
                }
                System.out.println("Read " + bookcnt + " book positions");
                System.out.println("Got " + bookhashcollisions + " hash collisions");
            } catch (FileNotFoundException e) {
                try {
                    wfp = new FileOutputStream(BOOKRUN);
                } catch (FileNotFoundException e1) {
                    e1.printStackTrace();
                    System.err.println("Could not create " + BOOKRUN + " file.");
                    return BOOK_EIO;
                }
                if (write_magic(wfp) != BOOK_SUCCESS) {
                    System.err.println("Could not write to " + BOOKRUN);
                    return BOOK_EIO;
                }
                wfp.close();
                System.out.println("Created new book " + BOOKRUN + "!");
                rfp = new FileInputStream(BOOKRUN);
                digest_bits = MAX_DIGEST_BITS;
                /* We use read_book() here only to allocate memory */
                if (read_book(rfp) == BOOK_ENOMEM) {
                    return BOOK_ENOMEM;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
            return BOOK_EIO;
        }
        return BOOK_SUCCESS;
    }

/*
 * Store the position and results of last search
 * if and only if in the first 10 moves of the game.
 * This routine is called after the computer makes its
 * most recent move. Lastly, after the 10th move, on
 * the 11th move, store the data out to the running file.
 */
/*
 * NOTE: Before you build a book, you have to call
 * BookBuilderOpen() now, after building it BookBuilderClose()
 * in order to actually write the book to disk.
 */

    static int BookBuilder(int result, int side) // uint_8
    {
        int i;

        /* Only first BOOKDEPTH moves */
        if (GameCnt > BOOKDEPTH)
            return BOOK_EMIDGAME;
        Hash.CalcHashKey();
        for (i = DIGEST_START(HashKey);
                ;
             i = DIGEST_NEXT(i, HashKey)) {
            if (DIGEST_MATCH(i, HashKey)) {
                existpos++;
                break;
            } else if (DIGEST_EMPTY(i)) {
                if (bookcnt > DIGEST_LIMIT())
                    return BOOK_EFULL;
                bookpos[i].key = HashKey;
                newpos++;
                bookcnt++;
                break;
            } else {
                bookhashcollisions++;
            }
        }
        if (side == white) {
            if (result == R_WHITE_WINS)
                bookpos[i].wins++;
            else if (result == R_BLACK_WINS)
                bookpos[i].losses++;
            else if (result == R_DRAW)
                bookpos[i].draws++;
        } else {
            if (result == R_WHITE_WINS)
                bookpos[i].losses++;
            else if (result == R_BLACK_WINS)
                bookpos[i].wins++;
            else if (result == R_DRAW)
                bookpos[i].draws++;
        }
        return BOOK_SUCCESS;
    }

    static int BookBuilderClose() {
        /*
         * IMHO the following part needs no temporary file.
         * If two gnu4j invocations try to write the same
         * file at the same time, this goes wrong anyway.
         * Please correct me if I am wrong. If you generate
         * a temporary file, try to generate it in some writable
         * directory.
         */
        FileOutputStream wfp;
        int i;
        int errcode = BOOK_SUCCESS;

        try {
            wfp = new FileOutputStream(BOOKRUN);
            try {
                write_magic(wfp);
                write_size(wfp, bookcnt);
                for (i = 0; i < DIGEST_SIZE(); i++) {
                    if (!is_empty(i)) {
                        book_to_buf(i);
                        wfp.write(buf, 0, buf.length);
                    }
                }
                System.out.println("Got " + bookhashcollisions + " hash collisions");
                return errcode;
            } finally {
                try {
                    wfp.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }

            }

        } catch (FileNotFoundException e) {
            e.printStackTrace();
            book_allocated = false;

            /* Let BookQuery allocate again */
            bookloaded = false;

            return BOOK_EIO;
        } catch (IOException e) {
            e.printStackTrace();
            book_allocated = false;

            /* Let BookQuery allocate again */
            bookloaded = false;
            return BOOK_EIO;

        }
    }

/*
 * Return values are defined in common.h
 * NOTE: This function now returns BOOK_SUCCESS on
 * success, which is 0. So all the possible callers
 * should adjust to this. (At present it is only called
 * in iterate.c.) It used to return 1 on success before.
 */

    static class ResultRecord {
        int wins;
        int losses;
        int draws;
    }

    static int BookQuery(int BKquery) {
        /*
         * BKquery denotes format; 0 text, 1 engine
         * In general out put is engine compliant, lines start with a blank
         * and end with emtpy line
         */
        int i, j, k, icnt = 0, mcnt, tot, maxdistribution;
        int[] matches = new int[MAXMATCH];
        Leaf[] m = new Leaf[MAXMOVES];
        for (int im = 0; im < m.length; im++)
            m[im] = new Leaf();
        Leaf[] pref = new Leaf[MAXMOVES];
        for (int ip = 0; ip < pref.length; ip++)
            pref[ip] = new Leaf();
        ResultRecord[] r = new ResultRecord[MAXMOVES];
        for (int ir = 0; ir < r.length; ir++)
           r[ir] = new ResultRecord();

        FileInputStream rfp = null;
        int p;
        int side, xside, temp;
        int booksize;
        int res;
        String bookname = "";

        if (bookloaded && !book_allocated) {
            /* Something failed during loading the book */
            return BOOK_ENOBOOK;
        }
        if (!bookloaded) {
            bookloaded = true;
            for (int booktry = 0; booktry < bookbin.length; booktry++) {
                bookname = bookbin[booktry];
                if ((flags & XBOARD) == 0) {
                    ofp.println("Looking for opening book in " + bookname + "...");
                }
                try {
                    rfp = new FileInputStream(bookname);
                } catch (FileNotFoundException e) {
                    continue;
                }
                if ((flags & XBOARD) == 0)
                    ofp.println("Read opening book (" + bookname + ")...");
                if (!check_magic(rfp)) {
                    System.err.println(" File " + bookname + " does not conform to the current format.\n" +
                            " Consider rebuilding it.\n");
                    try {
                        rfp.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                } else
                    break; /* Success, stop search */
            }
            if (rfp == null) {
                /* If appropriate print error */
                if ((flags & XBOARD) == 0 || BKquery == 1)
                    ofp.println(" No book found.\n\n");
                return BOOK_ENOBOOK;
            }
            if ((flags & XBOARD) == 0) {
                ofp.println("Loading book from " + bookname + ".");
            }
            /*
             * The factor 1.06 is the minimum additional amount of
             * free space in the hash
             */
            try {
                booksize = (int) (read_size(rfp) * 1.06);
                for (digest_bits = 1; booksize != 0; booksize >>= 1) {
                    digest_bits++;
                }
                res = read_book(rfp);
                if (res != BOOK_SUCCESS) {
                    return res;
                }
                if ((flags & XBOARD) != 0)
                    ofp.println(bookhashcollisions + " hash collisions... ");
            } catch (IOException e) {
                e.printStackTrace();
            }

        }

        mcnt = -1;
        side = board.side;
        xside = 1 ^ side;
        TreePtr[2] = TreePtr[1];
        GenMove.GenMoves(1);
        GenMove.FilterIllegalMoves(1);
        for (p = TreePtr[1]; p != TreePtr[2]; p++) {
            Tree[p].move = Move.MakeMove(side, Tree[p].move);
            m[icnt].move = Tree[p].move;
            posshash[icnt] = HashKey;
            icnt++;
            Tree[p].move = Move.UnmakeMove(xside, Tree[p].move);
        }
        outer:
        for (i = 0; i < icnt; i++) {
            for (j = DIGEST_START(posshash[i]);
                 !DIGEST_EMPTY(j);
                 j = DIGEST_NEXT(j, posshash[i])) {
                if (DIGEST_MATCH(j, posshash[i])) {
                    boolean found = false;
                    for (k = 0; k < mcnt; k++)
                        if (matches[k] == i) {
                            found = true;
                            break;
                        }
                    /* Position must have at least some wins to be played by book */
                    if (!found) {
                        matches[++mcnt] = i;
                        pref[mcnt].move = m[matches[mcnt]].move;
                        r[i].losses = bookpos[j].losses;
                        r[i].wins = bookpos[j].wins;
                        r[i].draws = bookpos[j].draws;

                        /* by percent score starting from this book position */

                        pref[mcnt].score = m[i].score =
                                100 * (r[i].wins + (r[i].draws / 2)) /
                                (MAX(r[i].wins + r[i].losses + r[i].draws, 1)) + r[i].wins / 2;

                    }
                    if (mcnt >= MAXMATCH) {
                        ofp.println(" Too many matches in book.\n\n");
                        break outer;
                    }
                    break; /* If we found a match, the following positions can not match */
                }
            }
        }

          if ((flags & XBOARD) == 0 || BKquery == 1) {
              ofp.println(" Opening database: " + bookcnt + " book positions.");
              if (mcnt + 1 == 0) {
                  ofp.println(" In this position, there is no book move.\n");
              } else if (mcnt + 1 == 1) {
                  ofp.println(" In this position, there is one book move:");
              } else {
                  ofp.println(" In this position, there are " + (mcnt + 1) + " book moves:");
              }
          }
        /* No book moves */
        if (mcnt == -1) {
            return BOOK_ENOMOVES;
        }
        k = 0;
        if (mcnt + 1 != 0) {
            if ((flags & XBOARD) == 0 || BKquery == 1) {
                for (i = 0; i <= mcnt; i++) {
                    if ((flags & XBOARD) == 0 || BKquery == 1) {
                        Move.SANMove(m[matches[i]].move, 1);
                        tot = r[matches[i]].wins + r[matches[i]].draws + r[matches[i]].losses;
                        // %s(%2.0f/%d/%d/%d)
                        ofp.println(" " + SANmv + "(" +
                                pad(round(100.0 * (r[matches[i]].wins + (r[matches[i]].draws / 2.0)) / tot, 0), 0) + "/" +
                                r[matches[i]].wins + "/" +
                                r[matches[i]].losses + "/" +
                                r[matches[i]].draws + ")");
                        if ((i + 1) % 4 == 0) ofp.println();
                    }
                }
                if ((flags & XBOARD) == 0 || BKquery == 1)
                    if (i % 4 != 0) ofp.println(" \n \n");
            }
            if (bookmode == BOOKRAND) {
                k = random.nextInt();
                k = k % (mcnt + 1);
                RootPV = m[matches[k]].move;
                if ((flags & XBOARD) == 0) {
                    System.out.println("\n(Random picked move #" + k + " " +
                            algbr[FROMSQ(RootPV)] + algbr[TOSQ(RootPV)] + " from above list)");
                    tot = r[matches[k]].wins + r[matches[k]].draws + r[matches[k]].losses;
                    if (tot != 0)
                        System.out.println("B p=" + pad(round(100.0 * (r[matches[k]].wins + r[matches[k]].draws) / tot, 0), 0));

                    else
                        System.out.println("p=NO EXPERIENCES");
                }
            } else if (bookmode == BOOKBEST) {
                Arrays.sort(pref, 0, mcnt + 1);
                RootPV = pref[0].move;
            } else if (bookmode == BOOKWORST) {
                Arrays.sort(pref, 0, mcnt + 1);
                RootPV = pref[mcnt].move;
            } else if (bookmode == BOOKPREFER) {
                Arrays.sort(pref, 0, mcnt + 1);
                for (i = 0; i <= mcnt; i++) {
                    if ((flags & XBOARD) == 0 || BKquery == 1) {
                        Move.SANMove(pref[i].move, 1);
                        System.out.println(" " + SANmv + "(" + pref[i].score + ") ");
                    }
                    m[i].move = pref[i].move;
                    if ((flags & XBOARD) == 0 || BKquery == 1)
                        if ((i + 1) % 8 == 0) ofp.println();
                }
                if ((flags & XBOARD) == 0 || BKquery == 1)
                    if (i % 8 != 0) ofp.println(" \n \n");
                temp = (bookfirstlast > mcnt + 1 ? mcnt + 1 : bookfirstlast);
                /* Choose from the top preferred moves based on distribution */
                maxdistribution = 0;
                for (i = 0; i < temp; i++)
                    maxdistribution += pref[i].score;
                /* Do not play moves that have only losses! */
                if (maxdistribution == 0)
                    return BOOK_ENOMOVES;
                k = (random.nextInt() & 0x7fffffff) % maxdistribution;
                maxdistribution = 0;
                for (i = 0; i < temp; i++) {
                    maxdistribution += pref[i].score;
                    if (k >= maxdistribution - pref[i].score &&
                            k < maxdistribution) {
                        k = i;
                        RootPV = m[k].move;
                        break;
                    }
                }
            }
        }
        return BOOK_SUCCESS;
    }

}
