/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

import java.util.Iterator;
@SuppressWarnings("all")
/**
 * Set of classes that tries to capture the functionality of the
 * UNIX long_option struct and functions.
 * <p/>
 * todo distinguish between required and optional parameters
 * <p/>
 * Created by Xan Gregg.
 * Date: Apr 24, 2005
 */
public class CommandLine {
    public final static int NO_PARAM = 0;
    public final static int OPT_PARAM = 1;
    public final static int REQ_PARAM = 2;

    /**
     * Parses the options in a command line using the given descriptors
     * and starting with the given arg index.
     *
     * @param descriptors array of option descriptors, possible empty or null
     * @param args        command line arguments, possibly empty
     * @param startArg    first argument to process (useful for skipping certain m_args)
     * @return iterator that steps through OptionInstance objects
     */
    static public Iterator parse(Descriptor[] descriptors, String[] args, int startArg) {
        return new OptionIterator(descriptors, args, startArg);
    }

    /**
     * Parses the options in a command line using the given descriptors.
     *
     * @param descriptors array of option descriptors, possible empty or null
     * @param args        command line arguments, possibly empty
     * @return iterator that steps through OptionInstance objects
     */
    static public Iterator parse(Descriptor[] descriptors, String[] args) {
        return parse(descriptors, args, 0);
    }

    /**
     * Parses the options in a command line.
     *
     * @param args     command line arguments, possibly empty
     * @param startArg first argument to process (useful for skipping certain m_args)
     * @return iterator that steps through OptionInstance objects
     */
    static public Iterator parse(String[] args, int startArg) {
        return parse(null, args, startArg);
    }

    /**
     * Parses the options in a command line.
     *
     * @param args command line arguments, possibly empty
     * @return iterator that steps through OptionInstance objects
     */
    static public Iterator parse(String[] args) {
        return parse(args, 0);
    }

    /**
     * Descriptor for a single option.  An option must have a long name,
     * a short name or both.
     */
    static public class Descriptor {

        public String longName;
        public int hasParam;
        public char shortName;

        public Descriptor(String longName, int param, char shortName) {
            this.longName = longName;
            this.hasParam = param;
            this.shortName = shortName;
        }
    }

    /**
     * An instance of a command line option.  The option field will always be set,
     * descriptor may be null, and param will be set if the descriptor
     * indicates that the option takes a parameter.
     */
    static public class OptionInstance {

        public char shortName;
        public Descriptor descriptor;   // may be null if it's an unknown option
        public String param;

        public OptionInstance(Descriptor descriptor) {
            this.descriptor = descriptor;
            this.shortName = descriptor.shortName;
        }

        public OptionInstance(char option) {
            this.shortName = option;
        }
    }

    /**
     * Iterator for command line options.
     */
    static public class OptionIterator implements Iterator {
        Descriptor[] m_descriptors;
        String[] m_args;
        String m_currentArg;
        int m_nextChar;    // char offset within m_currentArg
        int m_nextArg;     // arg index

        public OptionIterator(Descriptor[] descriptors, String[] args) {
            this(descriptors, args, 0);
        }

        public OptionIterator(Descriptor[] descriptors, String[] args, int startArg) {
            m_descriptors = descriptors;
            if (m_descriptors == null)
                m_descriptors = new Descriptor[0];
            m_args = args;
            m_nextArg = startArg;
        }

        public boolean hasNext() {
            return m_nextArg < m_args.length
                    || m_currentArg != null && m_nextChar < m_currentArg.length();
        }

        public Object next() {
            OptionInstance instance = null;
            if (m_currentArg != null && m_nextChar < m_currentArg.length()) {
                instance = nextShortParam();
            } else if (m_nextArg < m_args.length) {
                m_currentArg = m_args[m_nextArg++];
                // check for a long name
                if (m_currentArg.startsWith("--")) {
                    m_currentArg = m_currentArg.substring(2);
                    for (int i = 0; i < m_descriptors.length; i++) {
                        if (m_currentArg.equals(m_descriptors[i].longName)) {
                            instance = new OptionInstance(m_descriptors[i]);
                            m_nextChar = m_currentArg.length();
                            break;
                        }
                        else if (m_currentArg.startsWith(m_descriptors[i].longName + "=")) {
                            instance = new OptionInstance(m_descriptors[i]);
                            instance.param = m_currentArg.substring((m_descriptors[i].longName + "=").length());
                            m_nextChar = m_currentArg.length();
                            break;
                        }
                    }
                } else
                if (m_currentArg.startsWith("-")) {
                    // so look for short params
                    m_nextChar = 1;
                    instance = nextShortParam();
                }
            }
            return instance;
        }

        private OptionInstance nextShortParam() {
            OptionInstance instance = null;
            char option = m_currentArg.charAt(m_nextChar++);
            // check for a known short param
            for (int i = 0; i < m_descriptors.length; i++) {
                if (option == m_descriptors[i].shortName) {
                    instance = new OptionInstance(m_descriptors[i]);
                    if (m_descriptors[i].hasParam != NO_PARAM && m_nextArg < m_args.length)
                        instance.param = m_args[m_nextArg++];
                    break;
                }
            }
            if (instance == null) {
                // just return the char as an unknown option (no descriptor)
                instance = new OptionInstance(option);
            }
            return instance;
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
