/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

import java.io.*;
@SuppressWarnings("all")
/**
 * Created by Xan Gregg.
 * Date: May 9, 2005
 */
public class EPD extends Common {

    static final int EPDCLOSE = 1;

    static BufferedReader fp;

    /**
     * Reads in an EPD file.  The first call will read the first EPD line,
     * the second call will read the 2nd line and so on.   To improve
     * performance, the file is never closed.  Closing of the file happens
     * only on 2 conditions:  (i) A ReadEPDFile failed because there is no
     * more lines to read.  (ii) A called to ReadEPDFile to explicitly
     * request that the file is closed (op = EPDCLOSE);
     * If op == 2, then we work silently.
     */
    static boolean ReadEPDFile(String file, int op) {
        String line;

        /*  If first time through, must open file  */
        if (fp == null) {
            try {
                fp = new BufferedReader(new FileReader(file));
            } catch (FileNotFoundException e) {
                System.out.println("Error opening file " + file);
                return (false);
            }
        }

        /*  Is this a close request? */
        if (op == EPDCLOSE) {
            try {
                fp.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            fp = null;
            return (false);
        }

        next_line:
     while (true) {
         /*  Okay, we read in an EPD entry  */
         try {
             line = fp.readLine();
             if (line != null) {
                 int ret = ParseEPD(line);

                 /* For now just ignore malformed lines */
                 if (ret != EPD_SUCCESS) continue;
                 if (op != 2)
                     System.out.println("\n" + id + " : Best move = " + solution);
                 return (true);
             }
             /* finished, must close file */
             else {
                 try {
                     fp.close();
                 } catch (IOException e) {
                     e.printStackTrace();
                 }
                 fp = null;
                 return (false);
             }
         } catch (IOException e) {
             e.printStackTrace();
             fp = null;
             return (false);
         }
     }
    }

/*
 * Returns EPD_SUCCESS on success, EPD_ERROR on error. We try to be
 * quite tough on the format. However, as of yet no legality checking
 * is done and the board is not reset on error, this should be done by
 * the caller.
 */

    /**
     * Parses an EPD input line.  A few global variables are updated e.g.
     * current board, side to move, en passant, castling status, etc.
     */
    static int ParseEPD(String line) {
        int r, c, sq;
        int p = 0;

        r = 56;
        c = 0;
        board = new Board();

        while (p >= 0 && p < line.length() && line.charAt(p) != ' ') {
            sq = r + c;
            switch (line.charAt(p)) {
                case 'P':
                    board.b[white][pawn] |= Util.longBit(sq);   // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[white] += ValueP;
                    break;
                case 'N':
                    board.b[white][knight] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[white] += ValueN;
                    break;
                case 'B':
                    board.b[white][bishop] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[white] += ValueB;
                    break;
                case 'R':
                    board.b[white][rook] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[white] += ValueR;
                    break;
                case 'Q':
                    board.b[white][queen] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[white] += ValueQ;
                    break;
                case 'K':
                    board.b[white][king] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    break;
                case 'p':
                    board.b[black][pawn] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[black] += ValueP;
                    break;
                case 'n':
                    board.b[black][knight] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[black] += ValueN;
                    break;
                case 'b':
                    board.b[black][bishop] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[black] += ValueB;
                    break;
                case 'r':
                    board.b[black][rook] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[black] += ValueR;
                    break;
                case 'q':
                    board.b[black][queen] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    board.material[black] += ValueQ;
                    break;
                case 'k':
                    board.b[black][king] |= Util.longBit(sq);  // SETBIT
                    board.blockerr90 |= Util.longBit(r90[sq]);  // SETBIT
                    board.blockerr45 |= Util.longBit(r45[sq]);  // SETBIT
                    board.blockerr315 |= Util.longBit(r315[sq]);  // SETBIT
                    break;
                case '/':
                    r -= 8;
                    c = -1;
                    break;
                default  :
                    break;
            }
            if (Character.isDigit(line.charAt(p)))
                c += (line.charAt(p) - '0');
            else
                c++;

            /*
             * Special case, a trailing "/" is accepted on the
             * end of the board settings.
             */

            if (r == -8 && line.charAt(p + 1) == ' ')
                r = 0;

            if (r < 0 || c > 8) return EPD_ERROR;
            if (c == 8 && line.charAt(p + 1) != '/' && line.charAt(p + 1) != ' ') return EPD_ERROR;
            p++;
        }

        board.pmaterial[white] = (short) (board.material[white] -
                Util.nbits(board.b[white][pawn]) * ValueP);
        board.pmaterial[black] = (short) (board.material[black] -
                Util.nbits(board.b[black][pawn]) * ValueP);
        board.king[white] = (short) Util.leadz(board.b[white][king]);
        board.king[black] = (short) Util.leadz(board.b[black][king]);
        Util.UpdateFriends();
        Util.UpdateCBoard();
        Util.UpdateMvboard();

        /*  Get side to move  */
        if (++p >= line.length()) return EPD_ERROR;  // fixc original: (!++p)  -- shouldn't that be (!*++p)?
        if (line.charAt(p) == 'w')
            board.side = white;
        else if (line.charAt(p) == 'b')
            board.side = black;
        else
            return EPD_ERROR;

        /* Isn't this one cute? */
        if (++p >= line.length() || line.charAt(p) != ' ' || ++p >= line.length()) return EPD_ERROR;

        /*  Castling status  */
        while (p < line.length() && line.charAt(p) != ' ') {
            if (line.charAt(p) == 'K')
                board.flag |= WKINGCASTLE;
            else if (line.charAt(p) == 'Q')
                board.flag |= WQUEENCASTLE;
            else if (line.charAt(p) == 'k')
                board.flag |= BKINGCASTLE;
            else if (line.charAt(p) == 'q')
                board.flag |= BQUEENCASTLE;
            else if (line.charAt(p) == '-') {
                p++;
                break;
            } else return EPD_ERROR;
            p++;
        }
        if (p >= line.length() || line.charAt(p) != ' ' || ++p >= line.length()) return EPD_ERROR;

        /*
         * En passant square, can only be '-' or [a-h][36]
         * In fact, one could add more sanity checks here.
         */
        if (line.charAt(p) != '-') {
            if (p + 1 >= line.length() || line.charAt(p) < 'a' || line.charAt(p) > 'h' ||
                    !(line.charAt(p + 1) == '3' || line.charAt(p + 1) == '6'))
                return EPD_ERROR;
            board.ep = (short) ((line.charAt(p) - 'a') + (line.charAt(p + 1) - '1') * 8);
            p++;
        } else {
            board.ep = -1;
        }

        solution = "";
        id = "";

        if (++p >= line.length()) return EPD_SUCCESS;

        /* The opcodes are optional, so we should not generate errors here */

        /*  Read in best move; "bm" operator */
        int str_p = line.indexOf("bm", p);
        if (str_p >= 0) {
            str_p += 3;  // skip "bm "
            int semi = line.indexOf(";", str_p);
            solution = line.substring(str_p, semi);
            p = semi;
        }

        /*  Read in the description; "id" operator */
        str_p = line.indexOf("id", p);
        if (str_p >= 0) {
            str_p += 3;  // skip "id "
            int semi = line.indexOf(";", str_p);
            id = line.substring(str_p, semi);
            p = semi;
        }

        Hash.CalcHashKey();
        phase = PHASE();

        return EPD_SUCCESS;
    }


    /**
     * This routine reads in the next or the Nth position in the file.
     */
    static void LoadEPD(String p) {
        String file = p.substring(0, 31);
        p = p.substring(31).trim();
        int N;
        try {
            N = Integer.parseInt(p);
        } catch (NumberFormatException e) {
            N = 1;
        }

//       sscanf (p, "%31s %d ", file, &N);
        if (file.equals("next")) {
            ReadEPDFile(file, 0);
        } else {
            ReadEPDFile(file, 1);
            while (--N != 0) {
                if (ReadEPDFile(file, 2) == false) {
                    System.out.println("File position exceeded");
                    return;
                }
            }
            ReadEPDFile(file, 0);
        }
        Output.ShowBoard();
        Init.NewPosition();
    }


    /**
     * This routine appends the current position in EPD format into a file.
     */
    static void SaveEPD(String p) {
        String file = p.trim();
        PrintWriter fp;
        int r, c, sq, k;
        char c1;

//       sscanf (p, "%s ", file);
        try {
            fp = new PrintWriter(new FileWriter(file, true));
            for (r = A8; r >= A1; r -= 8) {
                k = 0;
                for (c = 0; c < 8; c++) {
                    sq = r + c;
                    if (cboard[sq] == empty)
                        k++;
                    else {
                        if (k != 0)
                            fp.print(k);  // "%1d"
                        k = 0;
                        c1 = notation[cboard[sq]];
                        if ((BitPosArray[sq] & board.friends[black]) != 0)
                            c1 = Character.toLowerCase(c1);
                        fp.print(c1);
                    }
                }
                if (k != 0)
                    fp.print(k);  // "%1d"
                if (r > A1)
                    fp.print("/");
            }

            /* Print other stuff */
            fp.print((board.side == white ? " w " : " b "));

            if ((board.flag & WKINGCASTLE) != 0)
                fp.print("K");
            if ((board.flag & WQUEENCASTLE) != 0)
                fp.print("Q");
            if ((board.flag & BKINGCASTLE) != 0)
                fp.print("k");
            if ((board.flag & BQUEENCASTLE) != 0)
                fp.print("q");
            if ((board.flag & (WCASTLE | BCASTLE)) == 0)
                fp.print("-");

            fp.print(" " + (board.ep > -1 ? algbr[board.ep] : "-"));
            fp.print(" bm 1; id 1;");
            fp.println();
            fp.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
