/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * Input processing happens on a side thread.  The static member variable
 * input_status acts as a semaphore to gate reading and filling of the
 * inputstr buffer.
 *
 * Created by Xan Gregg.
 * Date: Apr 24, 2005
 */
public class Input implements Runnable {
/* Variable used to communicate with the main thread */
    public static final int INPUT_NONE = 0;
    public static final int INPUT_AVAILABLE = 1;

    public static int input_status = INPUT_NONE;

    public static String inputstr;
    static Thread input_thread;
    static BufferedReader in;
    static boolean stop_requested;

/* The generic input routine */

    public static void getline(String prompt) {
        if ((Common.flags & Common.XBOARD) == 0) {
            System.out.println(prompt);
            System.out.flush();
        }
        try {
            inputstr = in.readLine();
        } catch (IOException e) {
            e.printStackTrace();
            inputstr = "";
        }
    }

/*
 * Posix threads are used to allow efficent polling for
 * input when the program is pondering.
 *
 * input_status is a boolean to indicate if a command
 * is being parsed and processed. It is set by
 * input function, and must be cleared by the thread
 * that uses the input.
 *
 * The main loop may explicitly wait_for_input, or
 * when pondering will examine the input_status
 * variable in Iterate.
 *
 */

    public void run() {
        String prompt = "";

        while ((Common.flags & Common.QUIT) == 0 && !stop_requested) {
            if ((Common.flags & Common.XBOARD) == 0) {
                prompt = (Common.RealSide == Common.black ? "Black" : "White")
                        + " (" + ((Common.RealGameCnt + 1) / 2 + 1) + ") : ";
            }
            getline(prompt);
            input_status = INPUT_AVAILABLE;
            /*
             * Posix waits can wake up spuriously
             * so we must ensure that we keep waiting
             * until we are woken by something that has
             * consumed the input
             */
            while (input_status == INPUT_AVAILABLE && !stop_requested) {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    stop_requested = true;
                }
            }
        }
    }

    public static void input_wakeup() {
        input_status = INPUT_NONE;
    }

    public static void wait_for_input() {
        // Read without mutex -- whoops
        while (input_status == INPUT_NONE) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
            }
        }
    }

    public static void InitInput() {
        /* Start input thread */
        stop_requested = false;
        in = new BufferedReader(new InputStreamReader(System.in));
        input_thread = new Thread(new Input());
        input_thread.start();
    }

    public static void CleanupInput() {
        stop_requested = true;
        // fixme we could be stuck in a readLine call
        try {
            input_thread.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
}
