/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

/**
 * Created by Xan Gregg.
 * Date: May 8, 2005
 */
public class Swap extends Common {
    static final boolean[] xray = {false, true, false, true, true, true, false};

    /**
     * A Static Exchange Evaluator (or SEE for short).
     * First determine the target square.  Create a bitboard of all squares
     * attacking the target square for both sides.  Using these 2 bitboards,
     * we take turn making captures from smallest piece to largest piece.
     * When a sliding piece makes a capture, we check behind it to see if
     * another attacker piece has been exposed.  If so, add this to the bitboard
     * as well.  When performing the "captures", we stop if one side is ahead
     * and doesn't need to capture, a form of pseudo-minimaxing.
     */
    static int SwapOff(int move) {
        int f, t, sq, piece, lastval;
        int side, xside;
        int[] swaplist = new int[MAXPLYDEPTH];
        int n;
        /*BitBoard*/long b, c, r;
        /*BitBoard*/long[] d, e;

        f = FROMSQ(move);
        t = TOSQ(move);
        side = ((board.friends[white] & BitPosArray[f]) != 0 ? white : black);
        xside = 1 ^ side;

        /*  Squares attacking t for side and xside  */
        b = Atak.AttackTo(t, side);
        c = Atak.AttackTo(t, xside);
        b &= ~Util.longBit(f); //CLEARBIT(b, f);
        /*BitBoard*/long[] refParams = new /*BitBoard*/long[2];
        if (xray[cboard[f]]) {
            refParams[0] = b;
            refParams[1] = c;
            AddXrayPiece(t, f, side, refParams);
            b = refParams[0];
            c = refParams[1];
        }

        d = board.b[side];
        e = board.b[xside];
        if ((move & PROMOTION) != 0) {
            swaplist[0] = Value[PROMOTEPIECE(move)] - ValueP;
            lastval = -Value[PROMOTEPIECE(move)];
        } else {
            swaplist[0] = ((move & ENPASSANT) != 0 ? ValueP : Value[cboard[t]]);
            lastval = -Value[cboard[f]];
        }
        n = 1;
        while (true) {
            if (c == NULLBITBOARD)
                break;
            for (piece = pawn; piece <= king; piece++) {
                r = c & e[piece];
                if (r != 0) {
                    sq = Util.leadz(r);
                    c &= ~Util.longBit(sq); //CLEARBIT (c, sq);
                    if (xray[piece]) {
                        refParams[0] = c;
                        refParams[1] = b;
                        AddXrayPiece(t, sq, xside, refParams);
                        c = refParams[0];
                        b = refParams[1];
                    }
                    swaplist[n] = swaplist[n - 1] + lastval;
                    n++;
                    lastval = Value[piece];
                    break;
                }
            }

            if (b == NULLBITBOARD)
                break;
            for (piece = pawn; piece <= king; piece++) {
                r = b & d[piece];
                if (r != 0) {
                    sq = Util.leadz(r);
                    b &= ~Util.longBit(sq); //CLEARBIT (b, sq);
                    if (xray[piece]) {
                        refParams[0] = b;
                        refParams[1] = c;
                        AddXrayPiece(t, sq, side, refParams);
                        b = refParams[0];
                        c = refParams[1];
                    }
                    swaplist[n] = swaplist[n - 1] + lastval;
                    n++;
                    lastval = -Value[piece];
                    break;
                }
            }
        }

        /****************************************************************************
         *
         *  At this stage, we have the swap scores in a list.  We just need to
         *  mini-max the scores from the bottom up to the top of the list.
         *
         ****************************************************************************/
        --n;
        while (n != 0) {
            if ((n & 1) != 0) {
                if (swaplist[n] <= swaplist[n - 1])
                    swaplist[n - 1] = swaplist[n];
            } else {
                if (swaplist[n] >= swaplist[n - 1])
                    swaplist[n - 1] = swaplist[n];
            }
            --n;
        }
        return (swaplist[0]);
    }


    /**
     * ************************************************************************
     * <p/>
     * The purpose of this routine is to find a piece which attack through
     * another piece (e.g. two rooks, Q+B, B+P, etc.)  Side is the side attacking
     * the square where the swapping is to be done.
     * <p/>
     * *************************************************************************
     */
    static void AddXrayPiece(int t, int sq, int side, /*BitBoard*/long[] refParams) {
        int dir, nsq, piece;
        /*BitBoard*/long a;

        dir = directions[t][sq];
        a = Ray[sq][dir] & board.blocker;
        if (a == NULLBITBOARD)
            return;
        nsq = (t < sq ? Util.leadz(a) : Util.trailz(a));
        piece = cboard[nsq];
        if ((piece == queen) || (piece == rook && dir > 3) ||
                (piece == bishop && dir < 4)) {
            if ((BitPosArray[nsq] & board.friends[side]) != 0)
                refParams[0] |= BitPosArray[nsq];
            else
                refParams[1] |= BitPosArray[nsq];
        }
        return;
    }
}
