/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;


/**
 * Created by Xan Gregg.
 * Date: May 8, 2005
 */
public class TTable extends Common {

    /**
     * Uses a two-tier depth-based transposition table.  The criteria for
     * replacement is as follows.
     * 1.  The first element is replaced whenever we have a position with
     * at least the same depth. In that case the element is moved to
     * the second slot.
     * 2.  The second slot is otherwise always replaced.
     * Problem may be that the first elements eventually get filled with
     * outdated entries. Might add an age counter later.
     * The & ~1 is a trick to clear the last bit making the offset even.
     */
    static void TTPut(int side, int depth, int ply, int alpha, int beta,
                      int score, int move) {

        HashSlot[] hs = HashTab[side];
        int t = (int) ((HashKey & TTHashMask) & ~1L);
//        if (depth < hs[t].depth)
//            t++;
//        else if (hs[t].flag != 0)
            hs[t + 1].copy(hs[t]);

        if (hs[t].flag != 0)
            CollHashCnt++;
        TotalPutHashCnt++;
        hs[t].move = move;
        hs[t].key = HashKey;
        hs[t].depth = (byte) depth;
        if (depth == 0)
            hs[t].flag = QUIESCENT;
        else if (score >= beta)
            hs[t].flag = LOWERBOUND;
        else if (score <= alpha)
            hs[t].flag = UPPERBOUND;
        else
            hs[t].flag = EXACTSCORE;

        if (MATESCORE(score) != 0)
            hs[t].score = (short) (score + (score > 0 ? ply : -ply));
        else
            hs[t].score = (short) score;
    }


    /**
     * Probe the transposition table.  There are 2 entries to be looked at as
     * we are using a 2-tier transposition table.
     */
    static int TTGet(int side, int depth, int ply, int[] scoreMove) {

        TotalGetHashCnt++;
        HashSlot[] hs = HashTab[side];
        int t = (int) ((HashKey & TTHashMask) & ~1L);
        if (HashKey != hs[t].key && HashKey != hs[++t].key)
            return (0);

        GoodGetHashCnt++;
//        if (!board.equals(hs[t].board))
//            GoodGetHashCnt--;

        scoreMove[1] = hs[t].move;
        scoreMove[0] = hs[t].score;
        if (hs[t].depth == 0)
            return (QUIESCENT);
        if (hs[t].depth < depth && MATESCORE(hs[t].score) == 0)
            return (POORDRAFT);
        if (MATESCORE(scoreMove[0]) != 0)
            scoreMove[0] -= (scoreMove[0] > 0 ? ply : -ply);
        return (hs[t].flag);
    }


    /**
     * Probe the transposition table.  There are 2 entries to be looked at as
     * we are using a 2-tier transposition table.  This routine merely wants to
     * get the PV from the hash, nothing else.
     */
    static int TTGetPV(int side, int ply, int score) {
        HashSlot[] hs = HashTab[side];
        int t = (int) ((HashKey & TTHashMask) & ~1L);
        int s;

        s = hs[t].score;
        if (MATESCORE(s) != 0)
            s -= (s > 0 ? ply : -ply);
        if (HashKey == hs[t].key && (((ply & 1) != 0 && score == s) || ((ply & 1) == 0 && score == -s))) {
            return hs[t].move;
        }
        t++;
        s = hs[t].score;
        if (MATESCORE(s) != 0)
            s -= (s > 0 ? ply : -ply);
        if (HashKey == hs[t].key && (((ply & 1) != 0 && score == s) || ((ply & 1) == 0 && score == -s))) {
            return hs[t].move;
        }
        return -1;
    }


    /**
     * Zero out the transposition table.
     */
    static void TTClear() {
        for (int i = 0; i < HashTab[white].length; i++)
            HashTab[white][i] = new HashSlot();
        for (int i = 0; i < HashTab[black].length; i++)
            HashTab[black][i] = new HashSlot();
    }


    /**
     * Zero out the pawn transposition table.
     */
    static void PTClear() {
        for (int i = 0; i < PawnTab[white].length; i++)
            PawnTab[white][i] = new PawnSlot();
        for (int i = 0; i < PawnTab[black].length; i++)
            PawnTab[black][i] = new PawnSlot();
    }
}
