/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

/**
 * Created by Xan Gregg.
 * Date: May 9, 2005
 */
public class Test extends Common {
    /**
     * This routine benchmarks the speed of the bitmap move generation.
     * The test case is BK.epd, the 24 positions from the Brat-Kopec test
     * suite.
     */
    static void TestMoveGenSpeed() {
        long i;
        long t1, t2;
        double et;

        GenCnt = 0;
        et = 0;

        t1 = System.currentTimeMillis();
        for (i = 0; i < 2000000; i++) {
            TreePtr[2] = TreePtr[1];
            GenMove.GenMoves(1);
        }
        t2 = System.currentTimeMillis();
        et += (t2 - t1) / 1e3;
        System.out.println("Time = " + et);

        System.out.println("No. of moves generated = " + GenCnt);
        System.out.println("Time taken = " + et + " secs");
        if (et > 0)
            System.out.println("Rate = " + GenCnt / et + " moves/sec.");
    }


    /**
     * This routine benchmarks the speed of the bitmap move generation
     * for non capturing moves.
     * The test case is BK.epd, the 24 positions from the Brat-Kopec test
     * suite.
     */
    static void TestNonCaptureGenSpeed() {
        long i;
        long t1, t2;
        double et;

        GenCnt = 0;
        et = 0;
        while (EPD.ReadEPDFile("test/wac.epd", 0)) {
            t1 = System.currentTimeMillis();
            for (i = 0; i < 100000; i++) {
                TreePtr[2] = TreePtr[1];
                GenMove.GenNonCaptures(1);
            }
            t2 = System.currentTimeMillis();
            et += (t2 - t1) / 1e3;
            System.out.println("Time = " + et);
        }
        System.out.println("No. of moves generated = " + GenCnt);
        System.out.println("Time taken = " + et);
        if (et > 0)
            System.out.println("Rate = " + GenCnt / et);
    }


    /**
     * This routine benchmarks the speed of the bitmap move generation
     * for captures.
     * The test case is BK.epd, the 24 positions from the Brat-Kopec test
     * suite.
     */
    static void TestCaptureGenSpeed() {
        long i;
        long t1, t2;
        double et;

        GenCnt = 0;
        et = 0;
        while (EPD.ReadEPDFile("test//wac.epd", 0)) {
            t1 = System.currentTimeMillis();
            for (i = 0; i < 200000; i++) {
                TreePtr[2] = TreePtr[1];
                GenMove.GenCaptures(1);
            }
            t2 = System.currentTimeMillis();
            et += (t2 - t1) / 1e3;
            System.out.println("Time = " + et);
        }
        System.out.println("No. of moves generated = " + GenCnt);
        System.out.println("Time taken = " + et);
        if (et > 0)
            System.out.println("Rate = " + GenCnt / et);
    }


    /**
     * This routine reads in a *.epd file (EPD notation) and prints the legal
     * moves for that position.
     */
    static void TestMoveList() {
        while (EPD.ReadEPDFile("TEST/GMG1.epd", 0)) {
            Output.ShowBoard();
            GenCnt = 0;
            TreePtr[2] = TreePtr[1];
            GenMove.GenMoves(1);
            GenMove.FilterIllegalMoves(1);
            Output.ShowMoveList(1);
            System.out.println("No. of moves generated = " + GenCnt);
        }
    }


    /**
     * This routine reads in a *.epd file (EPD notation) and prints the
     * non-capturing moves for that position.
     */
    static void TestNonCaptureList() {
        while (EPD.ReadEPDFile("test/GMG1.epd", 0)) {
            Output.ShowBoard();
            GenCnt = 0;
            TreePtr[2] = TreePtr[1];
            GenMove.GenNonCaptures(1);
            GenMove.FilterIllegalMoves(1);
            Output.ShowMoveList(1);
            System.out.println("No. of moves generated = " + GenCnt);
        }
    }


    /**
     * This routine reads in a *.epd file (EPD notation) and prints the capture
     * moves for that position.
     */
    static void TestCaptureList() {
        while (EPD.ReadEPDFile("test/GMG1.epd", 0)) {
            Output.ShowBoard();
            GenCnt = 0;
            TreePtr[2] = TreePtr[1];
            GenMove.GenCaptures(1);
            GenMove.FilterIllegalMoves(1);
            Output.ShowMoveList(1);
            System.out.println("No. of moves generated = " + GenCnt);
        }
    }


    static final int NEVALS = 30000;


    /**
     * This routine reads in the BK.epd and test the speed of the
     * evaluation routines.
     */
    static void TestEvalSpeed() {
        long i;
        long t1, t2;
        double et;

        et = 0;
        EvalCnt = 0;
        while (EPD.ReadEPDFile("test/wac.epd", 0)) {
            t1 = System.currentTimeMillis();
            for (i = 0; i < NEVALS; i++) {
                Eval.Evaluate(-INFINITY, INFINITY);
            }
            t2 = System.currentTimeMillis();
            et += (t2 - t1) / 1e3;
            System.out.println("Time = " + et);
        }
        System.out.println("No. of positions evaluated = " + EvalCnt);
        System.out.println("Time taken = " + et);
        if (et > 0)
            System.out.println("Rate = " + EvalCnt / et);
    }


    /**
     * To test the evaluation routines,  read from the BK.epd test file.
     * Print out the score.  This can be improved by being more verbose
     * and printing out salient features of the board, e.g. King safety,
     * double bishops, rook on seventh rank, weak pawns, doubled pawns,
     * bad bishops, passwd pawns, etc etc.
     */
    static void TestEval() {
        int score;

        SetFlag(TESTT);
        while (EPD.ReadEPDFile("test/wac.epd", 0)) {
            Output.ShowBoard();
            score = Eval.Evaluate(-INFINITY, INFINITY);
            System.out.println(board.side == white ? "W : " : "B : ");
            System.out.println("score = " + score);
            System.out.println();
        }
        ClearFlag(TESTT);
    }

}
