/* This file is part of chessbox_gnu4j, a Java port of GNU Chess (5.0.7).

   Copyright (c) 2005 Xan Gregg (Java port)
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   chessbox_gnu4j is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 or (at your option)
   any later version.

   chessbox_gnu4j is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info:
     bug-gnu-chess@gnu.org
     xan.gregg@pobox.com
*/
package org.chessbox.gnu4j;

import java.util.Arrays;

/**
 * Created by Xan Gregg.
 * Date: May 5, 2005
 */
public class Util extends Common {
    /**
     * Returns the leading 1 bit in a bitboard.  Leftmost bit is 0 and
     * rightmost bit is 63.  Thanks to Robert Hyatt for this algorithm.
     */
    static int leadz(/*BitBoard*/long b) {
       if (b >>> 32 != 0) {
           if (b >>> 48 != 0) return lzArray[((int) (b >>> 48))];
           return lzArray[((int) (b >>> 32))] + 16;
       }
//        if (b >>> 48 != 0) return lzArray[((int) (b >>> 48))];
//        if (b >>> 32 != 0) return lzArray[((int) (b >>> 32))] + 16;
        if (b >>> 16 != 0) return lzArray[((int) (b >>> 16))] + 32;
        return lzArray[((int) b)] + 48;
    }

    /**
     * Returns the trailing 1 bit in a bitboard.  Leftmost bit is 0 and
     * rightmost bit is 63.
     */
    public static int trailz(long b) {
        return leadz((b) & ((~b) + 1));
    }

    /**
     * Count the number of bits in b.
     */
    static int nbits(/*BitBoard*/long b) {
        return BitCount[((int) (b >>> 48))] + BitCount[((int) ((b >>> 32) & 0xffff))]
                + BitCount[((int) ((b >>> 16) & 0xffff))] + BitCount[((int) (b & 0xffff))];
    }

    static long longBit(int n) {
        return 0x8000000000000000L >>> n;   // 1L << (63 - n);   //
    }

    /**
     * Update friend and enemy bitboard.
     */
    static void UpdateFriends() {
        /*BitBoard*/long[] w, b;

        w = board.b[white];
        b = board.b[black];
        board.friends[white] =
                w[pawn] | w[knight] | w[bishop] | w[rook] | w[queen] | w[king];
        board.friends[black] =
                b[pawn] | b[knight] | b[bishop] | b[rook] | b[queen] | b[king];
        board.blocker = board.friends[white] | board.friends[black];
    }


    /**
     * Updates cboard[].  cboard[i] returns the piece on square i.
     */
    static void UpdateCBoard() {
        /*BitBoard*/long b;
        int piece, sq;

        Arrays.fill(cboard, 0);
        for (piece = pawn; piece <= king; piece++) {
            b = board.b[white][piece] | board.b[black][piece];
            while (b != 0) {
                sq = leadz(b);
                b &= ~Util.longBit(sq);  //CLEARBIT (b, sq);
                cboard[sq] = piece;
            }
        }
    }


    static final int[] OrigCboard =
            {rook, knight, bishop, queen, king, bishop, knight, rook,
                pawn, pawn, pawn, pawn, pawn, pawn, pawn, pawn,
                empty, empty, empty, empty, empty, empty, empty, empty,
                empty, empty, empty, empty, empty, empty, empty, empty,
                empty, empty, empty, empty, empty, empty, empty, empty,
                empty, empty, empty, empty, empty, empty, empty, empty,
                pawn, pawn, pawn, pawn, pawn, pawn, pawn, pawn,
                rook, knight, bishop, queen, king, bishop, knight, rook};

    /**
     * Updates Mvboard[].  Mvboard[i] returns the number of times the piece
     * on square i have moved.  When loading from EPD, if a piece is not on
     * its original square, set it to 1, otherwise set to 0.
     */
    static void UpdateMvboard() {
        int sq;

        for (sq = 0; sq < 64; sq++) {
            if (cboard[sq] == empty || cboard[sq] == OrigCboard[sq])
                Mvboard[sq] = 0;
            else
                Mvboard[sq] = 1;
        }
    }


    /**
     * User has pressed Ctrl-C.  Just set flags TIMEOUT to be true.
     */
    static void EndSearch(int sig) {
        SetFlag(TIMEOUT);
        // fixjava signal(SIGINT, EndSearch);
    }


    /**
     * Check the board to make sure that its valid.  Some things to check are
     * a.  Both sides have only 1 king.
     * b.  Side not on the move must not be in check.
     * c.  If en passant square is set, check it is possible.
     * d.  Check if castling status are all correct.
     */
    static boolean ValidateBoard() {
        int side, xside, sq;

        if (nbits(board.b[white][king]) != 1)
            return (false);
        if (nbits(board.b[black][king]) != 1)
            return (false);

        side = board.side;
        xside = 1 ^ side;
        if (Atak.SqAtakd(board.king[xside], side))
            return (false);

        if (board.ep > -1) {
            sq = board.ep + (xside == white ? 8 : -8);
            if ((BitPosArray[sq] & board.b[xside][pawn]) == 0)
                return (false);
        }

        if ((board.flag & WKINGCASTLE) != 0) {
            if ((BitPosArray[E1] & board.b[white][king]) == 0)
                return (false);
            if ((BitPosArray[H1] & board.b[white][rook]) == 0)
                return (false);
        }
        if ((board.flag & WQUEENCASTLE) != 0) {
            if ((BitPosArray[E1] & board.b[white][king]) == 0)
                return (false);
            if ((BitPosArray[A1] & board.b[white][rook]) == 0)
                return (false);
        }
        if ((board.flag & BKINGCASTLE) != 0) {
            if ((BitPosArray[E8] & board.b[black][king]) == 0)
                return (false);
            if ((BitPosArray[H8] & board.b[black][rook]) == 0)
                return (false);
        }
        if ((board.flag & BQUEENCASTLE) != 0) {
            if ((BitPosArray[E8] & board.b[black][king]) == 0)
                return (false);
            if ((BitPosArray[A8] & board.b[black][rook]) == 0)
                return (false);
        }

        return (true);
    }

}
